@@ -314,7 +314,7 @@ void SDHC1_Handler(void) __attribute__((weak, alias("Dummy_Handler")));
 #endif
 
 /* Exception Table */
-__attribute__((section(".vectors"))) const DeviceVectors exception_table
+__attribute__((section(".vectors"), used)) const DeviceVectors exception_table
     = {
 
         /* Configure Initial Stack Pointer, using linker-generated symbols */
@@ -660,7 +660,7 @@ void Reset_Handler(void)
 #endif
 
 	/* Initialize the C library */
-	__libc_init_array();
+	//__libc_init_array();
 
 	/* Branch to main function */
 	main();
