@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007-2016, Cameron Rich
+ * Copyright (c) 2007-2019, Cameron Rich
  * 
  * All rights reserved.
  * 
@@ -295,7 +295,7 @@ EXP_FUNC int STDCALL ssl_read(SSL *ssl, uint8_t **in_data)
     int ret = basic_read(ssl, in_data);
 
     /* check for return code so we can send an alert */
-    if (ret < SSL_OK && ret != SSL_CLOSE_NOTIFY)
+    if (ret < SSL_OK && ret != SSL_CLOSE_NOTIFY && ret != SSL_EAGAIN)
     {
         if (ret != SSL_ERROR_CONN_LOST)
         {
@@ -1093,7 +1093,6 @@ static int send_raw_packet(SSL *ssl, uint8_t protocol)
 #endif
     }
 
-    SET_SSL_FLAG(SSL_NEED_RECORD);  /* reset for next time */
     ssl->bm_index = 0;
 
     if (protocol != PT_APP_PROTOCOL_DATA)  
@@ -1174,13 +1173,11 @@ int send_packet(SSL *ssl, uint8_t protocol, const uint8_t *in, int length)
         if (ssl->version >= SSL_PROTOCOL_VERSION_TLS1_1)
         {
             uint8_t iv_size = ssl->cipher_info->iv_size;
-            uint8_t *t_buf = alloca(msg_length + iv_size);
-            memcpy(t_buf + iv_size, ssl->bm_data, msg_length);
-            if (get_random(iv_size, t_buf) < 0)
+            memmove(ssl->bm_data + iv_size, ssl->bm_data, msg_length);
+            if (get_random(iv_size, ssl->bm_data) < 0)
                 return SSL_NOT_OK;
 
             msg_length += iv_size;
-            memcpy(ssl->bm_data, t_buf, msg_length);
         }
 
         /* now encrypt the packet */
@@ -1313,7 +1310,7 @@ int basic_read(SSL *ssl, uint8_t **in_data)
 #else
         if (SOCKET_ERRNO() == EAGAIN || SOCKET_ERRNO() == EWOULDBLOCK)
 #endif
-            return 0;
+            return SSL_EAGAIN;
     }
 
     /* connection has gone, so die */
@@ -1355,6 +1352,7 @@ int basic_read(SSL *ssl, uint8_t **in_data)
         if (ssl->need_bytes > RT_MAX_PLAIN_LENGTH+RT_EXTRA-BM_RECORD_OFFSET)
         {
             printf("TLS buffer overflow, record size: %u (+5)\n", ssl->need_bytes);
+            ssl->need_bytes = SSL_RECORD_SIZE;
             ret = SSL_ERROR_RECORD_OVERFLOW;              
             goto error;
         }
@@ -2430,8 +2428,9 @@ EXP_FUNC void STDCALL ssl_display_error(int error_code) {}
 
 #ifdef CONFIG_BINDINGS
 #if !defined(CONFIG_SSL_ENABLE_CLIENT)
-EXP_FUNC SSL * STDCALL ssl_client_new(SSL_CTX *ssl_ctx, long client_fd, const
-        uint8_t *session_id, uint8_t sess_id_size)
+EXP_FUNC SSL * STDCALL ssl_client_new(SSL_CTX *ssl_ctx, long client_fd,
+        const uint8_t *session_id, uint8_t sess_id_size, 
+        SSL_EXTENSIONS* ssl_ext)
 {
     printf("%s", unsupported_str);
     return NULL;
