@@ -350,6 +350,7 @@ static const u32 Te4[256] =
   0xb0b0b0b0U, 0x54545454U, 0xbbbbbbbbU, 0x16161616U,
 };
 
+#ifdef ENABLE_RIJNDAEL_DECRYPT
 static const u32 Td0[256] =
 {
   0x51f4a750U, 0x7e416553U, 0x1a17a4c3U, 0x3a275e96U,
@@ -689,12 +690,13 @@ static const u32 Td4[256] =
   0xe1e1e1e1U, 0x69696969U, 0x14141414U, 0x63636363U,
   0x55555555U, 0x21212121U, 0x0c0c0c0cU, 0x7d7d7d7dU,
 };
+#endif
 
 static const u32 rcon[] =
 {
-  0x01000000, 0x02000000, 0x04000000, 0x08000000,
-  0x10000000, 0x20000000, 0x40000000, 0x80000000,
-  0x1B000000, 0x36000000,
+  0x01000000U, 0x02000000U, 0x04000000U, 0x08000000U,
+  0x10000000U, 0x20000000U, 0x40000000U, 0x80000000U,
+  0x1B000000U, 0x36000000U,
   /* for 128-bit blocks, Rijndael never uses more than 10 rcon values */
 };
 
@@ -713,142 +715,160 @@ static const u32 rcon[] =
  *
  * @return the number of rounds for the given cipher key size.
  */
-int rijndaelSetupEncrypt(u32 *rk, const u8 *key, int keybits)
+int rijndaelSetupEncrypt(uint32_t *rk, const uint8_t *key, int keybits)
 {
+  u32 * rk_ = rk;
+
+  int rounds = 0;
   int i = 0;
   u32 temp;
 
-  rk[0] = GETU32(key     );
-  rk[1] = GETU32(key +  4);
-  rk[2] = GETU32(key +  8);
-  rk[3] = GETU32(key + 12);
+  rk_[0] = GETU32(key     );
+  rk_[1] = GETU32(key +  4);
+  rk_[2] = GETU32(key +  8);
+  rk_[3] = GETU32(key + 12);
   if (keybits == 128)
   {
     for (;;)
     {
-      temp  = rk[3];
-      rk[4] = rk[0] ^
+      temp  = rk_[3];
+      rk_[4] = rk_[0] ^
         (Te4[(temp >> 16) & 0xff] & 0xff000000) ^
         (Te4[(temp >>  8) & 0xff] & 0x00ff0000) ^
         (Te4[(temp      ) & 0xff] & 0x0000ff00) ^
         (Te4[(temp >> 24)       ] & 0x000000ff) ^
         rcon[i];
-      rk[5] = rk[1] ^ rk[4];
-      rk[6] = rk[2] ^ rk[5];
-      rk[7] = rk[3] ^ rk[6];
-      if (++i == 10)
-        return 10;
-      rk += 4;
+      rk_[5] = rk_[1] ^ rk_[4];
+      rk_[6] = rk_[2] ^ rk_[5];
+      rk_[7] = rk_[3] ^ rk_[6];
+      if (++i == 10) {
+		  rounds = 10;
+		  break;
+	  }
+      rk_ += 4;
     }
   }
-  rk[4] = GETU32(key + 16);
-  rk[5] = GETU32(key + 20);
+
+#if defined(ENABLE_RIJNDAEL_192) || defined (ENABLE_RIJNDAEL_256)
+
+  rk_[4] = GETU32(key + 16);
+  rk_[5] = GETU32(key + 20);
   if (keybits == 192)
   {
     for (;;)
     {
-      temp = rk[ 5];
-      rk[ 6] = rk[ 0] ^
+      temp = rk_[ 5];
+      rk_[ 6] = rk_[ 0] ^
         (Te4[(temp >> 16) & 0xff] & 0xff000000) ^
         (Te4[(temp >>  8) & 0xff] & 0x00ff0000) ^
         (Te4[(temp      ) & 0xff] & 0x0000ff00) ^
         (Te4[(temp >> 24)       ] & 0x000000ff) ^
         rcon[i];
-      rk[ 7] = rk[ 1] ^ rk[ 6];
-      rk[ 8] = rk[ 2] ^ rk[ 7];
-      rk[ 9] = rk[ 3] ^ rk[ 8];
-      if (++i == 8)
-        return 12;
-      rk[10] = rk[ 4] ^ rk[ 9];
-      rk[11] = rk[ 5] ^ rk[10];
-      rk += 6;
+      rk_[ 7] = rk_[ 1] ^ rk_[ 6];
+      rk_[ 8] = rk_[ 2] ^ rk_[ 7];
+      rk_[ 9] = rk_[ 3] ^ rk_[ 8];
+      if (++i == 8){
+      	rounds = 12;
+      }
+      rk_[10] = rk_[ 4] ^ rk_[ 9];
+      rk_[11] = rk_[ 5] ^ rk_[10];
+      rk_ += 6;
     }
   }
-  rk[6] = GETU32(key + 24);
-  rk[7] = GETU32(key + 28);
+#endif
+
+#ifdef ENABLE_RIJNDAEL_256
+  rk_[6] = GETU32(key + 24);
+  rk_[7] = GETU32(key + 28);
   if (keybits == 256)
   {
     for (;;)
     {
-      temp = rk[ 7];
-      rk[ 8] = rk[ 0] ^
+      temp = rk_[ 7];
+      rk_[ 8] = rk_[ 0] ^
         (Te4[(temp >> 16) & 0xff] & 0xff000000) ^
         (Te4[(temp >>  8) & 0xff] & 0x00ff0000) ^
         (Te4[(temp      ) & 0xff] & 0x0000ff00) ^
         (Te4[(temp >> 24)       ] & 0x000000ff) ^
         rcon[i];
-      rk[ 9] = rk[ 1] ^ rk[ 8];
-      rk[10] = rk[ 2] ^ rk[ 9];
-      rk[11] = rk[ 3] ^ rk[10];
-      if (++i == 7)
-        return 14;
-      temp = rk[11];
-      rk[12] = rk[ 4] ^
+      rk_[ 9] = rk_[ 1] ^ rk_[ 8];
+      rk_[10] = rk_[ 2] ^ rk_[ 9];
+      rk_[11] = rk_[ 3] ^ rk_[10];
+      if (++i == 7){
+      	rounds = 14;
+      	break;
+	  }
+      temp = rk_[11];
+      rk_[12] = rk_[ 4] ^
         (Te4[(temp >> 24)       ] & 0xff000000) ^
         (Te4[(temp >> 16) & 0xff] & 0x00ff0000) ^
         (Te4[(temp >>  8) & 0xff] & 0x0000ff00) ^
         (Te4[(temp      ) & 0xff] & 0x000000ff);
-      rk[13] = rk[ 5] ^ rk[12];
-      rk[14] = rk[ 6] ^ rk[13];
-      rk[15] = rk[ 7] ^ rk[14];
-      rk += 8;
+      rk_[13] = rk_[ 5] ^ rk_[12];
+      rk_[14] = rk_[ 6] ^ rk_[13];
+      rk_[15] = rk_[ 7] ^ rk_[14];
+      rk_ += 8;
     }
   }
-  return 0;
+#endif
+	return rounds;
 }
 
+#ifdef ENABLE_RIJNDAEL_DECRYPT
 /**
  * Expand the cipher key into the decryption key schedule.
  *
  * @return the number of rounds for the given cipher key size.
  */
-int rijndaelSetupDecrypt(u32 *rk, const u8 *key, int keybits)
+int rijndaelSetupDecrypt(uint32_t *rk, const uint8_t *key, int keybits)
 {
+  u32 * rk_ = rk;
   int nrounds, i, j;
   u32 temp;
 
   /* expand the cipher key: */
-  nrounds = rijndaelSetupEncrypt(rk, key, keybits);
+  nrounds = rijndaelSetupEncrypt(rk_, key, keybits);
   /* invert the order of the round keys: */
   for (i = 0, j = 4*nrounds; i < j; i += 4, j -= 4)
   {
-    temp = rk[i    ]; rk[i    ] = rk[j    ]; rk[j    ] = temp;
-    temp = rk[i + 1]; rk[i + 1] = rk[j + 1]; rk[j + 1] = temp;
-    temp = rk[i + 2]; rk[i + 2] = rk[j + 2]; rk[j + 2] = temp;
-    temp = rk[i + 3]; rk[i + 3] = rk[j + 3]; rk[j + 3] = temp;
+    temp = rk_[i    ]; rk_[i    ] = rk_[j    ]; rk_[j    ] = temp;
+    temp = rk_[i + 1]; rk_[i + 1] = rk_[j + 1]; rk_[j + 1] = temp;
+    temp = rk_[i + 2]; rk_[i + 2] = rk_[j + 2]; rk_[j + 2] = temp;
+    temp = rk_[i + 3]; rk_[i + 3] = rk_[j + 3]; rk_[j + 3] = temp;
   }
   /* apply the inverse MixColumn transform to all round keys but the first and the last: */
   for (i = 1; i < nrounds; i++)
   {
-    rk += 4;
-    rk[0] =
-      Td0[Te4[(rk[0] >> 24)       ] & 0xff] ^
-      Td1[Te4[(rk[0] >> 16) & 0xff] & 0xff] ^
-      Td2[Te4[(rk[0] >>  8) & 0xff] & 0xff] ^
-      Td3[Te4[(rk[0]      ) & 0xff] & 0xff];
-    rk[1] =
-      Td0[Te4[(rk[1] >> 24)       ] & 0xff] ^
-      Td1[Te4[(rk[1] >> 16) & 0xff] & 0xff] ^
-      Td2[Te4[(rk[1] >>  8) & 0xff] & 0xff] ^
-      Td3[Te4[(rk[1]      ) & 0xff] & 0xff];
-    rk[2] =
-      Td0[Te4[(rk[2] >> 24)       ] & 0xff] ^
-      Td1[Te4[(rk[2] >> 16) & 0xff] & 0xff] ^
-      Td2[Te4[(rk[2] >>  8) & 0xff] & 0xff] ^
-      Td3[Te4[(rk[2]      ) & 0xff] & 0xff];
-    rk[3] =
-      Td0[Te4[(rk[3] >> 24)       ] & 0xff] ^
-      Td1[Te4[(rk[3] >> 16) & 0xff] & 0xff] ^
-      Td2[Te4[(rk[3] >>  8) & 0xff] & 0xff] ^
-      Td3[Te4[(rk[3]      ) & 0xff] & 0xff];
+    rk_ += 4;
+    rk_[0] =
+      Td0[Te4[(rk_[0] >> 24)       ] & 0xff] ^
+      Td1[Te4[(rk_[0] >> 16) & 0xff] & 0xff] ^
+      Td2[Te4[(rk_[0] >>  8) & 0xff] & 0xff] ^
+      Td3[Te4[(rk_[0]      ) & 0xff] & 0xff];
+    rk_[1] =
+      Td0[Te4[(rk_[1] >> 24)       ] & 0xff] ^
+      Td1[Te4[(rk_[1] >> 16) & 0xff] & 0xff] ^
+      Td2[Te4[(rk_[1] >>  8) & 0xff] & 0xff] ^
+      Td3[Te4[(rk_[1]      ) & 0xff] & 0xff];
+    rk_[2] =
+      Td0[Te4[(rk_[2] >> 24)       ] & 0xff] ^
+      Td1[Te4[(rk_[2] >> 16) & 0xff] & 0xff] ^
+      Td2[Te4[(rk_[2] >>  8) & 0xff] & 0xff] ^
+      Td3[Te4[(rk_[2]      ) & 0xff] & 0xff];
+    rk_[3] =
+      Td0[Te4[(rk_[3] >> 24)       ] & 0xff] ^
+      Td1[Te4[(rk_[3] >> 16) & 0xff] & 0xff] ^
+      Td2[Te4[(rk_[3] >>  8) & 0xff] & 0xff] ^
+      Td3[Te4[(rk_[3]      ) & 0xff] & 0xff];
   }
   return nrounds;
 }
+#endif
 
-void rijndaelEncrypt(const u32 *rk, int nrounds, const u8 plaintext[16],
-  u8 ciphertext[16])
+void rijndaelEncrypt(const uint32_t *rk, int nrounds, const uint8_t plaintext[16], uint8_t ciphertext[16])
 {
 
+  const u32 * rk_ = rk;
   u32 s0, s1, s2, s3, t0, t1, t2, t3;
   #ifndef FULL_UNROLL
     int r;
@@ -857,83 +877,92 @@ void rijndaelEncrypt(const u32 *rk, int nrounds, const u8 plaintext[16],
    * map byte array block to cipher state
    * and add initial round key:
   */
-  s0 = GETU32(plaintext     ) ^ rk[0];
-  s1 = GETU32(plaintext +  4) ^ rk[1];
-  s2 = GETU32(plaintext +  8) ^ rk[2];
-  s3 = GETU32(plaintext + 12) ^ rk[3];
+  s0 = GETU32(plaintext     ) ^ rk_[0];
+  s1 = GETU32(plaintext +  4) ^ rk_[1];
+  s2 = GETU32(plaintext +  8) ^ rk_[2];
+  s3 = GETU32(plaintext + 12) ^ rk_[3];
   #ifdef FULL_UNROLL
     /* round 1: */
-    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[ 4];
-    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk[ 5];
-    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk[ 6];
-    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk[ 7];
+    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk_[ 4];
+    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk_[ 5];
+    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk_[ 6];
+    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk_[ 7];
     /* round 2: */
-    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk[ 8];
-    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk[ 9];
-    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk[10];
-    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk[11];
+    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk_[ 8];
+    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk_[ 9];
+    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk_[10];
+    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk_[11];
     /* round 3: */
-    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[12];
-    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk[13];
-    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk[14];
-    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk[15];
+    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk_[12];
+    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk_[13];
+    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk_[14];
+    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk_[15];
     /* round 4: */
-    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk[16];
-    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk[17];
-    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk[18];
-    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk[19];
+    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk_[16];
+    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk_[17];
+    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk_[18];
+    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk_[19];
     /* round 5: */
-    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[20];
-    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk[21];
-    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk[22];
-    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk[23];
+    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk_[20];
+    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk_[21];
+    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk_[22];
+    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk_[23];
     /* round 6: */
-    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk[24];
-    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk[25];
-    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk[26];
-    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk[27];
+    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk_[24];
+    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk_[25];
+    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk_[26];
+    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk_[27];
     /* round 7: */
-    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[28];
-    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk[29];
-    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk[30];
-    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk[31];
+    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk_[28];
+    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk_[29];
+    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk_[30];
+    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk_[31];
     /* round 8: */
-    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk[32];
-    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk[33];
-    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk[34];
-    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk[35];
+    s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk_[32];
+    s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk_[33];
+    s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk_[34];
+    s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk_[35];
     /* round 9: */
-    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[36];
-    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk[37];
-    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk[38];
-    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk[39];
+    t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk_[36];
+    t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk_[37];
+    t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk_[38];
+    t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk_[39];
+
+#if defined(ENABLE_RIJNDAEL_192) || defined (ENABLE_RIJNDAEL_256)
+
     if (nrounds > 10)
     {
       /* round 10: */
-      s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk[40];
-      s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk[41];
-      s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk[42];
-      s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk[43];
+      s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk_[40];
+      s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk_[41];
+      s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk_[42];
+      s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk_[43];
       /* round 11: */
-      t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[44];
-      t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk[45];
-      t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk[46];
-      t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk[47];
+      t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk_[44];
+      t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk_[45];
+      t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk_[46];
+      t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk_[47];
+
+#ifdef ENABLE_RIJNDAEL_256
+
       if (nrounds > 12)
       {
         /* round 12: */
-        s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk[48];
-        s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk[49];
-        s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk[50];
-        s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk[51];
+        s0 = Te0[t0 >> 24] ^ Te1[(t1 >> 16) & 0xff] ^ Te2[(t2 >>  8) & 0xff] ^ Te3[t3 & 0xff] ^ rk_[48];
+        s1 = Te0[t1 >> 24] ^ Te1[(t2 >> 16) & 0xff] ^ Te2[(t3 >>  8) & 0xff] ^ Te3[t0 & 0xff] ^ rk_[49];
+        s2 = Te0[t2 >> 24] ^ Te1[(t3 >> 16) & 0xff] ^ Te2[(t0 >>  8) & 0xff] ^ Te3[t1 & 0xff] ^ rk_[50];
+        s3 = Te0[t3 >> 24] ^ Te1[(t0 >> 16) & 0xff] ^ Te2[(t1 >>  8) & 0xff] ^ Te3[t2 & 0xff] ^ rk_[51];
         /* round 13: */
-        t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[52];
-        t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk[53];
-        t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk[54];
-        t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk[55];
+        t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk_[52];
+        t1 = Te0[s1 >> 24] ^ Te1[(s2 >> 16) & 0xff] ^ Te2[(s3 >>  8) & 0xff] ^ Te3[s0 & 0xff] ^ rk_[53];
+        t2 = Te0[s2 >> 24] ^ Te1[(s3 >> 16) & 0xff] ^ Te2[(s0 >>  8) & 0xff] ^ Te3[s1 & 0xff] ^ rk_[54];
+        t3 = Te0[s3 >> 24] ^ Te1[(s0 >> 16) & 0xff] ^ Te2[(s1 >>  8) & 0xff] ^ Te3[s2 & 0xff] ^ rk_[55];
       }
+#endif
     }
-    rk += nrounds << 2;
+#endif
+
+    rk_ += nrounds << 2;
   #else  /* !FULL_UNROLL */
     /*
     * nrounds - 1 full rounds:
@@ -946,26 +975,26 @@ void rijndaelEncrypt(const u32 *rk, int nrounds, const u8 plaintext[16],
         Te1[(s1 >> 16) & 0xff] ^
         Te2[(s2 >>  8) & 0xff] ^
         Te3[(s3      ) & 0xff] ^
-        rk[4];
+        rk_[4];
       t1 =
         Te0[(s1 >> 24)       ] ^
         Te1[(s2 >> 16) & 0xff] ^
         Te2[(s3 >>  8) & 0xff] ^
         Te3[(s0      ) & 0xff] ^
-        rk[5];
+        rk_[5];
       t2 =
         Te0[(s2 >> 24)       ] ^
         Te1[(s3 >> 16) & 0xff] ^
         Te2[(s0 >>  8) & 0xff] ^
         Te3[(s1      ) & 0xff] ^
-        rk[6];
+        rk_[6];
       t3 =
         Te0[(s3 >> 24)       ] ^
         Te1[(s0 >> 16) & 0xff] ^
         Te2[(s1 >>  8) & 0xff] ^
         Te3[(s2      ) & 0xff] ^
-        rk[7];
-        rk += 8;
+        rk_[7];
+        rk_ += 8;
         if (--r == 0)
             break;
       s0 =
@@ -973,25 +1002,25 @@ void rijndaelEncrypt(const u32 *rk, int nrounds, const u8 plaintext[16],
         Te1[(t1 >> 16) & 0xff] ^
         Te2[(t2 >>  8) & 0xff] ^
         Te3[(t3      ) & 0xff] ^
-        rk[0];
+        rk_[0];
       s1 =
         Te0[(t1 >> 24)       ] ^
         Te1[(t2 >> 16) & 0xff] ^
         Te2[(t3 >>  8) & 0xff] ^
         Te3[(t0      ) & 0xff] ^
-        rk[1];
+        rk_[1];
       s2 =
         Te0[(t2 >> 24)       ] ^
         Te1[(t3 >> 16) & 0xff] ^
         Te2[(t0 >>  8) & 0xff] ^
         Te3[(t1      ) & 0xff] ^
-        rk[2];
+        rk_[2];
       s3 =
         Te0[(t3 >> 24)       ] ^
         Te1[(t0 >> 16) & 0xff] ^
         Te2[(t1 >>  8) & 0xff] ^
         Te3[(t2      ) & 0xff] ^
-        rk[3];
+        rk_[3];
      }
  #endif /* ?FULL_UNROLL */
   /*
@@ -1003,40 +1032,39 @@ void rijndaelEncrypt(const u32 *rk, int nrounds, const u8 plaintext[16],
     (Te4[(t1 >> 16) & 0xff] & 0x00ff0000) ^
     (Te4[(t2 >>  8) & 0xff] & 0x0000ff00) ^
     (Te4[(t3      ) & 0xff] & 0x000000ff) ^
-    rk[0];
+    rk_[0];
   PUTU32(ciphertext     , s0);
   s1 =
     (Te4[(t1 >> 24)       ] & 0xff000000) ^
     (Te4[(t2 >> 16) & 0xff] & 0x00ff0000) ^
     (Te4[(t3 >>  8) & 0xff] & 0x0000ff00) ^
     (Te4[(t0      ) & 0xff] & 0x000000ff) ^
-    rk[1];
+    rk_[1];
   PUTU32(ciphertext +  4, s1);
   s2 =
     (Te4[(t2 >> 24)       ] & 0xff000000) ^
     (Te4[(t3 >> 16) & 0xff] & 0x00ff0000) ^
     (Te4[(t0 >>  8) & 0xff] & 0x0000ff00) ^
     (Te4[(t1      ) & 0xff] & 0x000000ff) ^
-    rk[2];
+    rk_[2];
   PUTU32(ciphertext +  8, s2);
   s3 =
     (Te4[(t3 >> 24)       ] & 0xff000000) ^
     (Te4[(t0 >> 16) & 0xff] & 0x00ff0000) ^
     (Te4[(t1 >>  8) & 0xff] & 0x0000ff00) ^
     (Te4[(t2      ) & 0xff] & 0x000000ff) ^
-    rk[3];
+    rk_[3];
   PUTU32(ciphertext + 12, s3);
 
 
 }
 
-void rijndaelDecrypt(const u32 *rk, int nrounds, const u8 ciphertext[16],
-  u8 plaintext[16])
+#ifdef ENABLE_RIJNDAEL_DECRYPT
+void rijndaelDecrypt(const uint32_t *rk, int nrounds, const uint8_t ciphertext[16], uint8_t plaintext[16])
 {
 
-
-	
-u32 s0, s1, s2, s3, t0, t1, t2, t3;
+  const u32 * rk_ = rk;
+  u32 s0, s1, s2, s3, t0, t1, t2, t3;
   #ifndef FULL_UNROLL
     int r;
   #endif /* ?FULL_UNROLL */
@@ -1045,83 +1073,92 @@ u32 s0, s1, s2, s3, t0, t1, t2, t3;
   * map byte array block to cipher state
   * and add initial round key:
   */
-    s0 = GETU32(ciphertext     ) ^ rk[0];
-    s1 = GETU32(ciphertext +  4) ^ rk[1];
-    s2 = GETU32(ciphertext +  8) ^ rk[2];
-    s3 = GETU32(ciphertext + 12) ^ rk[3];
+    s0 = GETU32(ciphertext     ) ^ rk_[0];
+    s1 = GETU32(ciphertext +  4) ^ rk_[1];
+    s2 = GETU32(ciphertext +  8) ^ rk_[2];
+    s3 = GETU32(ciphertext + 12) ^ rk_[3];
   #ifdef FULL_UNROLL
     /* round 1: */
-    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk[ 4];
-    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk[ 5];
-    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk[ 6];
-    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk[ 7];
+    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk_[ 4];
+    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk_[ 5];
+    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk_[ 6];
+    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk_[ 7];
     /* round 2: */
-    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk[ 8];
-    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk[ 9];
-    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk[10];
-    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk[11];
+    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk_[ 8];
+    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk_[ 9];
+    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk_[10];
+    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk_[11];
     /* round 3: */
-    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk[12];
-    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk[13];
-    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk[14];
-    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk[15];
+    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk_[12];
+    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk_[13];
+    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk_[14];
+    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk_[15];
     /* round 4: */
-    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk[16];
-    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk[17];
-    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk[18];
-    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk[19];
+    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk_[16];
+    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk_[17];
+    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk_[18];
+    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk_[19];
     /* round 5: */
-    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk[20];
-    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk[21];
-    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk[22];
-    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk[23];
+    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk_[20];
+    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk_[21];
+    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk_[22];
+    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk_[23];
     /* round 6: */
-    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk[24];
-    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk[25];
-    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk[26];
-    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk[27];
+    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk_[24];
+    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk_[25];
+    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk_[26];
+    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk_[27];
     /* round 7: */
-    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk[28];
-    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk[29];
-    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk[30];
-    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk[31];
+    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk_[28];
+    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk_[29];
+    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk_[30];
+    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk_[31];
     /* round 8: */
-    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk[32];
-    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk[33];
-    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk[34];
-    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk[35];
+    s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk_[32];
+    s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk_[33];
+    s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk_[34];
+    s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk_[35];
     /* round 9: */
-    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk[36];
-    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk[37];
-    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk[38];
-    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk[39];
+    t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk_[36];
+    t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk_[37];
+    t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk_[38];
+    t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk_[39];
+
+#if defined(ENABLE_RIJNDAEL_192) || defined (ENABLE_RIJNDAEL_256)
+
     if (nrounds > 10)
     {
       /* round 10: */
-      s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk[40];
-      s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk[41];
-      s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk[42];
-      s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk[43];
+      s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk_[40];
+      s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk_[41];
+      s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk_[42];
+      s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk_[43];
       /* round 11: */
-      t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk[44];
-      t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk[45];
-      t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk[46];
-      t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk[47];
-      if (nrounds > 12)
+      t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk_[44];
+      t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk_[45];
+      t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk_[46];
+      t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk_[47];
+
+ #ifdef ENABLE_RIJNDAEL_256
+     if (nrounds > 12)
       {
         /* round 12: */
-        s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk[48];
-        s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk[49];
-        s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk[50];
-        s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk[51];
+        s0 = Td0[t0 >> 24] ^ Td1[(t3 >> 16) & 0xff] ^ Td2[(t2 >>  8) & 0xff] ^ Td3[t1 & 0xff] ^ rk_[48];
+        s1 = Td0[t1 >> 24] ^ Td1[(t0 >> 16) & 0xff] ^ Td2[(t3 >>  8) & 0xff] ^ Td3[t2 & 0xff] ^ rk_[49];
+        s2 = Td0[t2 >> 24] ^ Td1[(t1 >> 16) & 0xff] ^ Td2[(t0 >>  8) & 0xff] ^ Td3[t3 & 0xff] ^ rk_[50];
+        s3 = Td0[t3 >> 24] ^ Td1[(t2 >> 16) & 0xff] ^ Td2[(t1 >>  8) & 0xff] ^ Td3[t0 & 0xff] ^ rk_[51];
         /* round 13: */
-        t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk[52];
-        t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk[53];
-        t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk[54];
-        t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk[55];
+        t0 = Td0[s0 >> 24] ^ Td1[(s3 >> 16) & 0xff] ^ Td2[(s2 >>  8) & 0xff] ^ Td3[s1 & 0xff] ^ rk_[52];
+        t1 = Td0[s1 >> 24] ^ Td1[(s0 >> 16) & 0xff] ^ Td2[(s3 >>  8) & 0xff] ^ Td3[s2 & 0xff] ^ rk_[53];
+        t2 = Td0[s2 >> 24] ^ Td1[(s1 >> 16) & 0xff] ^ Td2[(s0 >>  8) & 0xff] ^ Td3[s3 & 0xff] ^ rk_[54];
+        t3 = Td0[s3 >> 24] ^ Td1[(s2 >> 16) & 0xff] ^ Td2[(s1 >>  8) & 0xff] ^ Td3[s0 & 0xff] ^ rk_[55];
       }
+#endif
+
     }
-    rk += nrounds << 2;
+#endif
+
+    rk_ += nrounds << 2;
   #else  /* !FULL_UNROLL */
     /*
     * nrounds - 1 full rounds:
@@ -1134,26 +1171,26 @@ u32 s0, s1, s2, s3, t0, t1, t2, t3;
         Td1[(s3 >> 16) & 0xff] ^
         Td2[(s2 >>  8) & 0xff] ^
         Td3[(s1      ) & 0xff] ^
-        rk[4];
+        rk_[4];
       t1 =
         Td0[(s1 >> 24)       ] ^
         Td1[(s0 >> 16) & 0xff] ^
         Td2[(s3 >>  8) & 0xff] ^
         Td3[(s2      ) & 0xff] ^
-        rk[5];
+        rk_[5];
       t2 =
         Td0[(s2 >> 24)       ] ^
         Td1[(s1 >> 16) & 0xff] ^
         Td2[(s0 >>  8) & 0xff] ^
         Td3[(s3      ) & 0xff] ^
-        rk[6];
+        rk_[6];
       t3 =
         Td0[(s3 >> 24)       ] ^
         Td1[(s2 >> 16) & 0xff] ^
         Td2[(s1 >>  8) & 0xff] ^
         Td3[(s0      ) & 0xff] ^
-        rk[7];
-      rk += 8;
+        rk_[7];
+      rk_ += 8;
       if (--r == 0)
           break;
       s0 =
@@ -1161,25 +1198,25 @@ u32 s0, s1, s2, s3, t0, t1, t2, t3;
         Td1[(t3 >> 16) & 0xff] ^
         Td2[(t2 >>  8) & 0xff] ^
         Td3[(t1      ) & 0xff] ^
-        rk[0];
+        rk_[0];
       s1 =
         Td0[(t1 >> 24)       ] ^
         Td1[(t0 >> 16) & 0xff] ^
         Td2[(t3 >>  8) & 0xff] ^
         Td3[(t2      ) & 0xff] ^
-        rk[1];
+        rk_[1];
       s2 =
         Td0[(t2 >> 24)       ] ^
         Td1[(t1 >> 16) & 0xff] ^
         Td2[(t0 >>  8) & 0xff] ^
         Td3[(t3      ) & 0xff] ^
-        rk[2];
+        rk_[2];
       s3 =
         Td0[(t3 >> 24)       ] ^
         Td1[(t2 >> 16) & 0xff] ^
         Td2[(t1 >>  8) & 0xff] ^
         Td3[(t0      ) & 0xff] ^
-        rk[3];
+        rk_[3];
     }
   #endif /* ?FULL_UNROLL */
   /*
@@ -1191,30 +1228,31 @@ u32 s0, s1, s2, s3, t0, t1, t2, t3;
     (Td4[(t3 >> 16) & 0xff] & 0x00ff0000) ^
     (Td4[(t2 >>  8) & 0xff] & 0x0000ff00) ^
     (Td4[(t1      ) & 0xff] & 0x000000ff) ^
-    rk[0];
+    rk_[0];
   PUTU32(plaintext     , s0);
   s1 =
     (Td4[(t1 >> 24)       ] & 0xff000000) ^
     (Td4[(t0 >> 16) & 0xff] & 0x00ff0000) ^
     (Td4[(t3 >>  8) & 0xff] & 0x0000ff00) ^
     (Td4[(t2      ) & 0xff] & 0x000000ff) ^
-    rk[1];
+    rk_[1];
   PUTU32(plaintext +  4, s1);
   s2 =
     (Td4[(t2 >> 24)       ] & 0xff000000) ^
     (Td4[(t1 >> 16) & 0xff] & 0x00ff0000) ^
     (Td4[(t0 >>  8) & 0xff] & 0x0000ff00) ^
     (Td4[(t3      ) & 0xff] & 0x000000ff) ^
-    rk[2];
+    rk_[2];
   PUTU32(plaintext +  8, s2);
   s3 =
     (Td4[(t3 >> 24)       ] & 0xff000000) ^
     (Td4[(t2 >> 16) & 0xff] & 0x00ff0000) ^
     (Td4[(t1 >>  8) & 0xff] & 0x0000ff00) ^
     (Td4[(t0      ) & 0xff] & 0x000000ff) ^
-    rk[3];
+    rk_[3];
 
 	PUTU32(plaintext + 12, s3);
 	
 
 }
+#endif
