@@ -20,8 +20,8 @@
  * THIS SOFTWARE IS PROVIDED BY BLUEKITCHEN GMBH AND CONTRIBUTORS
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
- * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL MATTHIAS
- * RINGWALD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL BLUEKITCHEN
+ * GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
  * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
@@ -38,7 +38,7 @@
 #define BTSTACK_FILE__ "hog_keyboard_demo.c"
 
 // *****************************************************************************
-/* EXAMPLE_START(hog_keyboard_demo): HID-over-GATT Keyboard
+/* EXAMPLE_START(hog_keyboard_demo): HID Keyboard LE
  */
 // *****************************************************************************
 
@@ -166,6 +166,7 @@ static btstack_packet_callback_registration_t hci_event_callback_registration;
 static btstack_packet_callback_registration_t sm_event_callback_registration;
 static uint8_t battery = 100;
 static hci_con_handle_t con_handle = HCI_CON_HANDLE_INVALID;
+static uint8_t protocol_mode = 1;
 
 static void packet_handler (uint8_t packet_type, uint16_t channel, uint8_t *packet, uint16_t size);
 
@@ -185,9 +186,6 @@ static void le_keyboard_setup(void){
 
     l2cap_init();
 
-    // setup le device db
-    le_device_db_init();
-
     // setup SM: Display only
     sm_init();
     sm_set_io_capabilities(IO_CAPABILITY_DISPLAY_ONLY);
@@ -254,10 +252,18 @@ static int keycode_and_modifer_us_for_character(uint8_t character, uint8_t * key
 }
 
 // HID Report sending
-
 static void send_report(int modifier, int keycode){
-    uint8_t report[] = { /* 0xa1, */ modifier, 0, 0, keycode, 0, 0, 0, 0, 0};
-    hids_device_send_input_report(con_handle, report, sizeof(report));
+    uint8_t report[] = {  modifier, 0, keycode, 0, 0, 0, 0, 0};
+    switch (protocol_mode){
+        case 0:
+            hids_device_send_boot_keyboard_input_report(con_handle, report, sizeof(report));
+            break;
+        case 1:
+           hids_device_send_input_report(con_handle, report, sizeof(report));
+           break;
+        default:
+            break;
+    }
 }
 
 // Demo Application
@@ -394,48 +400,51 @@ static void packet_handler (uint8_t packet_type, uint16_t channel, uint8_t *pack
     UNUSED(channel);
     UNUSED(size);
 
-    switch (packet_type) {
-        case HCI_EVENT_PACKET:
-            switch (hci_event_packet_get_type(packet)) {
-                case HCI_EVENT_DISCONNECTION_COMPLETE:
-                    con_handle = HCI_CON_HANDLE_INVALID;
-                    printf("Disconnected\n");
-                    break;
-                case SM_EVENT_JUST_WORKS_REQUEST:
-                    printf("Just Works requested\n");
-                    sm_just_works_confirm(sm_event_just_works_request_get_handle(packet));
-                    break;
-                case SM_EVENT_NUMERIC_COMPARISON_REQUEST:
-                    printf("Confirming numeric comparison: %"PRIu32"\n", sm_event_numeric_comparison_request_get_passkey(packet));
-                    sm_numeric_comparison_confirm(sm_event_passkey_display_number_get_handle(packet));
-                    break;
-                case SM_EVENT_PASSKEY_DISPLAY_NUMBER:
-                    printf("Display Passkey: %"PRIu32"\n", sm_event_passkey_display_number_get_passkey(packet));
-                    break;
-                case HCI_EVENT_HIDS_META:
-                    switch (hci_event_hids_meta_get_subevent_code(packet)){
-                        case HIDS_SUBEVENT_INPUT_REPORT_ENABLE:
-                            con_handle = hids_subevent_input_report_enable_get_con_handle(packet);
-                            printf("Report Characteristic Subscribed %u\n", hids_subevent_input_report_enable_get_enable(packet));
+    if (packet_type != HCI_EVENT_PACKET) return;
+
+    switch (hci_event_packet_get_type(packet)) {
+        case HCI_EVENT_DISCONNECTION_COMPLETE:
+            con_handle = HCI_CON_HANDLE_INVALID;
+            printf("Disconnected\n");
+            break;
+        case SM_EVENT_JUST_WORKS_REQUEST:
+            printf("Just Works requested\n");
+            sm_just_works_confirm(sm_event_just_works_request_get_handle(packet));
+            break;
+        case SM_EVENT_NUMERIC_COMPARISON_REQUEST:
+            printf("Confirming numeric comparison: %"PRIu32"\n", sm_event_numeric_comparison_request_get_passkey(packet));
+            sm_numeric_comparison_confirm(sm_event_passkey_display_number_get_handle(packet));
+            break;
+        case SM_EVENT_PASSKEY_DISPLAY_NUMBER:
+            printf("Display Passkey: %"PRIu32"\n", sm_event_passkey_display_number_get_passkey(packet));
+            break;
+        case HCI_EVENT_HIDS_META:
+            switch (hci_event_hids_meta_get_subevent_code(packet)){
+                case HIDS_SUBEVENT_INPUT_REPORT_ENABLE:
+                    con_handle = hids_subevent_input_report_enable_get_con_handle(packet);
+                    printf("Report Characteristic Subscribed %u\n", hids_subevent_input_report_enable_get_enable(packet));
 #ifndef HAVE_BTSTACK_STDIN
-                            hid_embedded_start_typing();
+                    hid_embedded_start_typing();
 #endif
-                            break;
-                        case HIDS_SUBEVENT_BOOT_KEYBOARD_INPUT_REPORT_ENABLE:
-                            con_handle = hids_subevent_boot_keyboard_input_report_enable_get_con_handle(packet);
-                            printf("Boot Keyboard Characteristic Subscribed %u\n", hids_subevent_boot_keyboard_input_report_enable_get_enable(packet));
-                            break;
-                        case HIDS_SUBEVENT_PROTOCOL_MODE:
-                            printf("Protocol Mode: %s mode\n", hids_subevent_protocol_mode_get_protocol_mode(packet) ? "Report" : "Boot");
-                            break;
-                        case HIDS_SUBEVENT_CAN_SEND_NOW:
-                            typing_can_send_now();
-                            break;
-                        default:
-                            break;
-                    }
+                    break;
+                case HIDS_SUBEVENT_BOOT_KEYBOARD_INPUT_REPORT_ENABLE:
+                    con_handle = hids_subevent_boot_keyboard_input_report_enable_get_con_handle(packet);
+                    printf("Boot Keyboard Characteristic Subscribed %u\n", hids_subevent_boot_keyboard_input_report_enable_get_enable(packet));
+                    break;
+                case HIDS_SUBEVENT_PROTOCOL_MODE:
+                    protocol_mode = hids_subevent_protocol_mode_get_protocol_mode(packet);
+                    printf("Protocol Mode: %s mode\n", hids_subevent_protocol_mode_get_protocol_mode(packet) ? "Report" : "Boot");
+                    break;
+                case HIDS_SUBEVENT_CAN_SEND_NOW:
+                    typing_can_send_now();
+                    break;
+                default:
+                    break;
             }
             break;
+            
+        default:
+            break;
     }
 }
 
