@@ -83,7 +83,6 @@
 #include "ble/att_dispatch.h"
 #include "ble/att_server.h"
 #include "ble/gatt-service/ancs_client.h"
-#include "ble/gatt-service/audio_input_control_service_server.h"
 #include "ble/gatt-service/battery_service_client.h"
 #include "ble/gatt-service/battery_service_server.h"
 #include "ble/gatt-service/bond_management_service_server.h"
@@ -94,17 +93,9 @@
 #include "ble/gatt-service/heart_rate_service_server.h"
 #include "ble/gatt-service/hids_client.h"
 #include "ble/gatt-service/hids_device.h"
-#include "ble/gatt-service/microphone_control_service_client.h"
-#include "ble/gatt-service/microphone_control_service_server.h"
 #include "ble/gatt-service/scan_parameters_service_client.h"
 #include "ble/gatt-service/scan_parameters_service_server.h"
 #include "ble/gatt-service/tx_power_service_server.h"
-#include "ble/gatt-service/volume_control_service_server.h"
-#include "ble/gatt-service/volume_offset_control_service_server.h"
-#ifdef ENABLE_MESH
-#include "ble/gatt-service/mesh_provisioning_service_server.h"
-#include "ble/gatt-service/mesh_proxy_service_server.h"
-#endif
 #include "ble/gatt_client.h"
 #include "ble/le_device_db.h"
 #include "ble/sm.h"
@@ -124,6 +115,7 @@
 #include "classic/avrcp_browsing_controller.h"
 #include "classic/avrcp_browsing_target.h"
 #include "classic/avrcp_controller.h"
+#include "classic/avrcp_cover_art_client.h"
 #include "classic/avrcp_media_item_iterator.h"
 #include "classic/avrcp_target.h"
 #include "classic/bnep.h"
@@ -152,12 +144,17 @@
 #include "classic/spp_server.h"
 #endif
 
+#include "le-audio/le_audio.h"
+#include "le-audio/le_audio_util.h"
+#include "le-audio/gatt-service/broadcast_audio_scan_service_client.h"
+#include "le-audio/gatt-service/broadcast_audio_scan_service_server.h"
+
 #ifdef ENABLE_MESH
-#include "ble/gatt-service/mesh_provisioning_service_server.h"
-#include "ble/gatt-service/mesh_proxy_service_server.h"
 #include "mesh/adv_bearer.h"
 #include "mesh/beacon.h"
 #include "mesh/gatt_bearer.h"
+#include "mesh/gatt-service/mesh_provisioning_service_server.h"
+#include "mesh/gatt-service/mesh_proxy_service_server.h"
 #include "mesh/mesh.h"
 #include "mesh/mesh_access.h"
 #include "mesh/mesh_configuration_client.h"
