@@ -10,13 +10,18 @@
 #define HAVE_MALLOC
 #define HAVE_POSIX_FILE_IO
 #define HAVE_POSIX_TIME
-
+#define HAVE_ASSERT
 
 // BTstack features that can be enabled
+#define ENABLE_CLASSIC
 #define ENABLE_BLE
 #define ENABLE_BTP
-#define ENABLE_LE_CENTRAL
 #define ENABLE_L2CAP_LE_CREDIT_BASED_FLOW_CONTROL_MODE
+#define ENABLE_LE_CENTRAL
+#define ENABLE_LE_DATA_LENGTH_EXTENSION
+#define ENABLE_LE_EXTENDED_ADVERTISING
+#define ENABLE_LE_ISOCHRONOUS_STREAMS
+#define ENABLE_LE_PERIODIC_ADVERTISING
 #define ENABLE_LE_PERIPHERAL
 #define ENABLE_LE_SECURE_CONNECTIONS
 #define ENABLE_LE_SIGNED_WRITE
