@@ -19,6 +19,7 @@
 #define ENABLE_PRINTF_HEXDUMP
 #define ENABLE_SDP_DES_DUMP
 #define ENABLE_SDP_EXTRA_QUERIES
+#define ENABLE_SOFTWARE_AES128
 
 // #define ENABLE_LE_SECURE_CONNECTIONS
 #define ENABLE_L2CAP_ENHANCED_RETRANSMISSION_MODE
@@ -35,4 +36,7 @@
 
 #define NVM_NUM_LINK_KEYS 2
 
+// testing
+#define FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
+
 #endif
