@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -48,6 +48,8 @@
  #include "queue.h"
  #include "timers.h"
  #include "semphr.h"
+#elif   (BSP_CFG_RTOS == 1)
+ #include "tx_api.h"
 #endif                                 /* #if (BSP_CFG_RTOS == 2) */
 
 /* Common macro for FSP header files. There is also a corresponding FSP_FOOTER macro at the end of this file. */
@@ -57,10 +59,6 @@ FSP_HEADER
  * Macro definitions
  ******************************************************************************/
 
-/* USB Version Info */
-#define USB_API_VERSION_MINOR      (2U)      // DEPRECATED      ///< Minor version of the API.
-#define USB_API_VERSION_MAJOR      (1U)      // DEPRECATED      ///< Major version of the API.
-
 /* USB Request Type Register */
 #define USB_BREQUEST               (0xFF00U) ///< b15-8
 
@@ -159,7 +157,7 @@ FSP_HEADER
 #define USB_IFCLS_CDCD             (0x0AU)   ///< CDC-Data Class
 #define USB_IFCLS_CHIP             (0x0BU)   ///< Chip/Smart Card Class
 #define USB_IFCLS_CNT              (0x0CU)   ///< Content-Security Class
-#define USB_IFCLS_VID              (0x0DU)   ///< Video Class
+#define USB_IFCLS_VID              (0x0EU)   ///< Video Class
 #define USB_IFCLS_DIAG             (0xDCU)   ///< Diagnostic Device
 #define USB_IFCLS_WIRE             (0xE0U)   ///< Wireless Controller
 #define USB_IFCLS_APL              (0xFEU)   ///< Application-Specific
@@ -233,6 +231,10 @@ typedef enum e_usb_class
     USB_CLASS_PCDC2,                   ///< PCDC2 Class
     USB_CLASS_PCDCC2,                  ///< PCDCC2 Class
     USB_CLASS_PHID,                    ///< PHID Class
+    USB_CLASS_PHID2,                   ///< PHID2 Class
+    USB_CLASS_PAUD,                    ///< PAUD Class
+    USB_CLASS_PPRN,                    ///< PPRN Class
+    USB_CLASS_DFU,                     ///< DFU Class
     USB_CLASS_PVND,                    ///< PVND Class
     USB_CLASS_HCDC,                    ///< HCDC Class
     USB_CLASS_HCDCC,                   ///< HCDCC Class
@@ -240,6 +242,8 @@ typedef enum e_usb_class
     USB_CLASS_HVND,                    ///< HVND Class
     USB_CLASS_HMSC,                    ///< HMSC Class
     USB_CLASS_PMSC,                    ///< PMSC Class
+    USB_CLASS_HPRN,                    ///< HPRN Class
+    USB_CLASS_HUVC,                    ///< HUVC Class
     USB_CLASS_REQUEST,                 ///< USB Class Request
     USB_CLASS_END                      ///< USB Class End Code
 } usb_class_t;
@@ -302,8 +306,12 @@ typedef enum e_usb_address
     USB_ADDRESS5,
 } usb_address_t;
 
-typedef void    usb_ctrl_t;
-typedef void (* usb_compliance_cb_t)(void *);
+/** USB control block.  Allocate an instance specific control block to pass into the USB API calls.
+ * @par Implemented as
+ * - usb_instance_ctrl_t
+ */
+typedef void usb_ctrl_t;
+typedef void (usb_compliance_cb_t)(void *);
 
 typedef struct st_usb_descriptor
 {
@@ -361,13 +369,25 @@ typedef struct st_usb_event_info
     const transfer_instance_t * p_transfer_rx;  ///< Receive context
 } usb_event_info_t;
 
+#if (BSP_CFG_RTOS == 0)
+typedef void (usb_callback_t)(void *);
+#endif
+#if (BSP_CFG_RTOS == 1)
+typedef TX_THREAD * usb_hdl_t;
+typedef void        (usb_callback_t)(usb_event_info_t *, usb_hdl_t, usb_onoff_t);
+#endif
 #if (BSP_CFG_RTOS == 2)
 typedef TaskHandle_t usb_hdl_t;
 typedef void         (usb_callback_t)(usb_event_info_t *, usb_hdl_t, usb_onoff_t);
-#else                                  /* #if (BSP_CFG_RTOS == 2) */
-typedef void (usb_callback_t)(void *);
-#endif /* #if (BSP_CFG_RTOS == 2) */
+#endif                                 /* #if (BSP_CFG_RTOS == 2) */
 
+#if (BSP_CFG_RTOS != 1)
+typedef uint32_t ULONG;
+#endif
+
+typedef void (usb_otg_callback_t)(ULONG);
+
+/** USB configuration. */
 typedef struct st_usb_cfg
 {
     usb_mode_t                  usb_mode;           ///< USB_MODE_HOST/USB_MODE_PERI
@@ -375,7 +395,7 @@ typedef struct st_usb_cfg
     uint8_t                     module_number;      ///< USB module number (USB_IP0/USB_IP1)
     usb_class_t                 type;               ///< USB device class etc
     usb_descriptor_t          * p_usb_reg;          ///< Pointer to the usb_decriptor_t structure area
-    usb_compliance_cb_t         usb_complience_cb;
+    usb_compliance_cb_t       * usb_complience_cb;
     IRQn_Type                   irq;                ///< USBI dedicated interrupt number storage variable.
     IRQn_Type                   irq_r;              ///< USBR dedicated interrupt number storage variable.
     IRQn_Type                   irq_d0;             ///< FS D0FIFO dedicated interrupt number storage variable.
@@ -390,7 +410,7 @@ typedef struct st_usb_cfg
     uint8_t                     hsipl;              ///< Variable to store the interrupt priority of USBIR.
     uint8_t                     hsipl_d0;           ///< Variable to store the interrupt priority of HS D0FIFO.
     uint8_t                     hsipl_d1;           ///< Variable to store the interrupt priority of HS D1FIFO.
-    usb_callback_t            * p_usb_apl_callback; ///< Aplication Callback
+    usb_callback_t            * p_usb_apl_callback; ///< Application Callback
     void const                * p_context;          ///< Other Context
     const transfer_instance_t * p_transfer_tx;      ///< Send context
     const transfer_instance_t * p_transfer_rx;      ///< Receive context
@@ -534,14 +554,6 @@ typedef struct st_usb_api
      */
     fsp_err_t (* pipeInfoGet)(usb_ctrl_t * const p_api_ctrl, usb_pipe_t * p_info, uint8_t pipe_number);
 
-    /* DEPRECATED Get the driver version
-     * @par Implemented as
-     * - @ref R_USB_VersionGet()
-     *
-     * @param[out] p_version      Version number.
-     */
-    fsp_err_t (* versionGet)(fsp_version_t * const p_version);
-
     /** Return USB-related completed events (OS less only)
      * @par Implemented as
      * - @ref R_USB_EventGet()
@@ -679,6 +691,23 @@ typedef struct st_usb_api
      * @param[out] setup            Setup type to get.
      */
     fsp_err_t (* setupGet)(usb_ctrl_t * const p_api_ctrl, usb_setup_t * setup);
+
+    /** This API sets the callback function for OTG.
+     * @par Implemented as
+     * - @ref R_USB_OtgCallbackSet()
+     *
+     * @param[in]  p_api_ctrl       USB control structure.
+     * @param[in]  p_callback       Pointer to the callback function for OTG.
+     */
+    fsp_err_t (* otgCallbackSet)(usb_ctrl_t * const p_api_ctrl, usb_otg_callback_t * p_callback);
+
+    /** This API starts SRP processing for OTG.
+     * @par Implemented as
+     * - @ref R_USB_OtgSRP()
+     *
+     * @param[in]  p_api_ctrl       USB control structure.
+     */
+    fsp_err_t (* otgSRP)(usb_ctrl_t * const p_api_ctrl);
 } usb_api_t;
 
 /** This structure encompasses everything that is needed to use an instance of this interface. */
