@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -36,6 +36,9 @@
 /* Mask to select CP bits( 0xF00000 ) */
 #define CP_MASK                                       (0xFU << 20)
 
+/* Startup value for CCR to enable instruction cache, branch prediction and LOB extension */
+#define CCR_CACHE_ENABLE                              (0x000E0201)
+
 /* Value to write to OAD register of MPU stack monitor to enable NMI when a stack overflow is detected. */
 #define BSP_STACK_POINTER_MONITOR_NMI_ON_DETECTION    (0xA500U)
 
@@ -43,19 +46,8 @@
 #define BSP_PRV_PRCR_KEY                              (0xA500U)
 #define BSP_PRV_PRCR_PRC1_UNLOCK                      ((BSP_PRV_PRCR_KEY) | 0x2U)
 #define BSP_PRV_PRCR_LOCK                             ((BSP_PRV_PRCR_KEY) | 0x0U)
-
-#if defined(__ICCARM__)
- #define BSP_PRV_STACK_LIMIT                          ((uint32_t) __section_begin(".stack"))
- #define BSP_PRV_STACK_TOP                            ((uint32_t) __section_end(".stack"))
-#elif defined(__ARMCC_VERSION)
- #define BSP_PRV_STACK_LIMIT                          ((uint32_t) &Image$$STACK$$ZI$$Base)
- #define BSP_PRV_STACK_TOP                            ((uint32_t) &Image$$STACK$$ZI$$Base + \
-                                                       (uint32_t) &Image$$STACK$$ZI$$Length)
-#elif defined(__GNUC__)
- #define BSP_PRV_STACK_LIMIT                          ((uint32_t) &__StackLimit)
- #define BSP_PRV_STACK_TOP                            ((uint32_t) &__StackTop)
-#endif
-
+#define BSP_PRV_STACK_LIMIT                           ((uint32_t) __Vectors[0] - BSP_CFG_STACK_MAIN_BYTES)
+#define BSP_PRV_STACK_TOP                             ((uint32_t) __Vectors[0])
 #define BSP_TZ_STACK_SEAL_VALUE                       (0xFEF5EDA5)
 
 /***********************************************************************************************************************
@@ -67,7 +59,7 @@
  **********************************************************************************************************************/
 
 /** System Clock Frequency (Core Clock) */
-uint32_t SystemCoreClock = 0U;
+uint32_t SystemCoreClock BSP_SECTION_EARLY_INIT;
 
 #if defined(__ARMCC_VERSION)
 extern uint32_t Image$$BSS$$ZI$$Base;
@@ -77,6 +69,11 @@ extern uint32_t Image$$DATA$$Base;
 extern uint32_t Image$$DATA$$Length;
 extern uint32_t Image$$STACK$$ZI$$Base;
 extern uint32_t Image$$STACK$$ZI$$Length;
+ #if BSP_FEATURE_BSP_HAS_DTCM == 1
+extern uint32_t Load$$DTCM_DATA_INIT$$Base;
+extern uint32_t Image$$DTCM_DATA_INIT$$Base;
+extern uint32_t Image$$DTCM_DATA_INIT$$Length;
+ #endif
 #elif defined(__GNUC__)
 
 /* Generated by linker. */
@@ -87,11 +84,20 @@ extern uint32_t __bss_start__;
 extern uint32_t __bss_end__;
 extern uint32_t __StackLimit;
 extern uint32_t __StackTop;
+ #if BSP_FEATURE_BSP_HAS_DTCM == 1
+extern uint32_t __dtcm_data_init_start;
+extern uint32_t __dtcm_data_start__;
+extern uint32_t __dtcm_data_end__;
+ #endif
 #elif defined(__ICCARM__)
  #pragma section=".bss"
  #pragma section=".data"
  #pragma section=".data_init"
  #pragma section=".stack"
+ #if BSP_FEATURE_BSP_HAS_DTCM == 1
+  #pragma section=".dtcm_data"
+  #pragma section=".dtcm_data_init"
+ #endif
 #endif
 
 /* Initialize static constructors */
@@ -135,11 +141,25 @@ void R_BSP_WarmStart(bsp_warm_start_event_t event) __attribute__((weak));
 
 #endif
 
+#if BSP_CFG_EARLY_INIT
+static void bsp_init_uninitialized_vars(void);
+
+#endif
+
 /*******************************************************************************************************************//**
  * Initialize the MCU and the runtime environment.
  **********************************************************************************************************************/
 void SystemInit (void)
 {
+#if defined(RENESAS_CORTEX_M85)
+
+    /* Enable the ARM core instruction cache, branch prediction and low-overhead-branch extension.
+     * See Section 5.5 of the Cortex-M55 TRM and Section D1.2.9 in the ARMv8-M Architecture Reference Manual */
+    SCB->CCR = (uint32_t) CCR_CACHE_ENABLE;
+    __DSB();
+    __ISB();
+#endif
+
 #if __FPU_USED
 
     /* Enable the FPU only when it is used.
@@ -154,8 +174,8 @@ void SystemInit (void)
     /* Seal the main stack for secure projects. Reference:
      * https://developer.arm.com/documentation/100720/0300
      * https://developer.arm.com/support/arm-security-updates/armv8-m-stack-sealing */
-    uint32_t * p_main_stack = (uint32_t *) __Vectors[0];
-    p_main_stack[BSP_CFG_STACK_MAIN_BYTES / sizeof(uint32_t)] = BSP_TZ_STACK_SEAL_VALUE;
+    uint32_t * p_main_stack_top = (uint32_t *) __Vectors[0];
+    *p_main_stack_top = BSP_TZ_STACK_SEAL_VALUE;
 #endif
 
 #if !BSP_TZ_NONSECURE_BUILD
@@ -185,6 +205,16 @@ void SystemInit (void)
  #endif
 #endif
 
+#if BSP_FEATURE_TFU_SUPPORTED
+    R_BSP_MODULE_START(FSP_IP_TFU, 0U);
+#endif
+
+#if BSP_CFG_EARLY_INIT
+
+    /* Initialize uninitialized BSP variables early for use in R_BSP_WarmStart. */
+    bsp_init_uninitialized_vars();
+#endif
+
     /* Call pre clock initialization hook. */
     R_BSP_WarmStart(BSP_WARM_START_RESET);
 
@@ -206,6 +236,16 @@ void SystemInit (void)
  #endif
 #endif
 
+#if BSP_FEATURE_BSP_HAS_GRAPHICS_DOMAIN
+
+    /* Turn on graphics power domain.
+     * This requires MOCO to be enabled, but MOCO is always enabled after bsp_clock_init(). */
+    R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_OM_LPC_BATT);
+    R_SYSTEM->PDCTRGD = 0;
+    (void) R_SYSTEM->PDCTRGD;
+    R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_OM_LPC_BATT);
+#endif
+
     /* Call post clock initialization hook. */
     R_BSP_WarmStart(BSP_WARM_START_POST_CLOCK);
 
@@ -232,9 +272,14 @@ void SystemInit (void)
 #endif
 
 #if BSP_FEATURE_TZ_HAS_TRUSTZONE
+ #if 33U == __CORTEX_M
 
     /* Use CM33 stack monitor. */
     __set_MSPLIM(BSP_PRV_STACK_LIMIT);
+ #else
+
+    /* CM85 stack monitor not yet supported. */
+ #endif
 #endif
 
 #if BSP_CFG_C_RUNTIME_INIT
@@ -252,10 +297,21 @@ void SystemInit (void)
     /* Copy initialized RAM data from ROM to RAM. */
  #if defined(__ARMCC_VERSION)
     memcpy((uint8_t *) &Image$$DATA$$Base, (uint8_t *) &Load$$DATA$$Base, (uint32_t) &Image$$DATA$$Length);
+  #if BSP_FEATURE_BSP_HAS_DTCM == 1
+    memcpy((uint8_t *) &Image$$DTCM_DATA_INIT$$Base,
+           (uint8_t *) &Load$$DTCM_DATA_INIT$$Base,
+           (uint32_t) &Image$$DTCM_DATA_INIT$$Length);
+  #endif
  #elif defined(__GNUC__)
     memcpy(&__data_start__, &__etext, ((uint32_t) &__data_end__ - (uint32_t) &__data_start__));
+  #if BSP_FEATURE_BSP_HAS_DTCM == 1
+    memcpy(&__dtcm_data_start__,
+           &__dtcm_data_init_start,
+           ((uint32_t) &__dtcm_data_end__ - (uint32_t) &__dtcm_data_start__));
+  #endif
  #elif defined(__ICCARM__)
-    memcpy((uint32_t *) __section_begin(".data"), (uint32_t *) __section_begin(".data_init"),
+    memcpy((uint32_t *) __section_begin(".data"),
+           (uint32_t *) __section_begin(".data_init"),
            (uint32_t) __section_size(".data"));
 
     /* Copy functions to be executed from RAM. */
@@ -268,8 +324,14 @@ void SystemInit (void)
     /* Copy main thread TLS to RAM. */
   #pragma section="__DLIB_PERTHREAD_init"
   #pragma section="__DLIB_PERTHREAD"
-    memcpy((uint32_t *) __section_begin("__DLIB_PERTHREAD"), (uint32_t *) __section_begin("__DLIB_PERTHREAD_init"),
+    memcpy((uint32_t *) __section_begin("__DLIB_PERTHREAD"),
+           (uint32_t *) __section_begin("__DLIB_PERTHREAD_init"),
            (uint32_t) __section_size("__DLIB_PERTHREAD_init"));
+  #if BSP_FEATURE_BSP_HAS_DTCM == 1
+    memcpy((uint32_t *) __section_begin(".dtcm_data"),
+           (uint32_t *) __section_begin(".dtcm_data_init"),
+           (uint32_t) __section_size(".dtcm_data"));
+  #endif
  #endif
 
     /* Initialize static constructors */
@@ -281,14 +343,12 @@ void SystemInit (void)
             (void (*)(void))((uint32_t) &Image$$INIT_ARRAY$$Base + (uint32_t) Image$$INIT_ARRAY$$Base[i]);
         p_init_func();
     }
-
  #elif defined(__GNUC__)
     int32_t count = __init_array_end - __init_array_start;
     for (int32_t i = 0; i < count; i++)
     {
         __init_array_start[i]();
     }
-
  #elif defined(__ICCARM__)
     void const * pibase = __section_begin("SHT$$PREINIT_ARRAY");
     void const * ilimit = __section_end("SHT$$INIT_ARRAY");
@@ -299,8 +359,18 @@ void SystemInit (void)
     /* Initialize SystemCoreClock variable. */
     SystemCoreClockUpdate();
 
+#if BSP_FEATURE_RTC_IS_AVAILABLE || BSP_FEATURE_RTC_HAS_TCEN || BSP_FEATURE_SYSC_HAS_VBTICTLR
+
+    /* For TZ project, it should be called by the secure application, whether RTC module is to be configured as secure or not. */
+ #if !BSP_TZ_NONSECURE_BUILD
+
+    /* Perform RTC reset sequence to avoid unintended operation. */
+    R_BSP_Init_RTC();
+ #endif
+#endif
+
 #if !BSP_CFG_PFS_PROTECT
- #if BSP_TZ_SECURE_BUILD
+ #if BSP_TZ_SECURE_BUILD || (BSP_CFG_MCU_PART_SERIES == 8)
     R_PMISC->PWPRS = 0;                              ///< Clear BOWI bit - writing to PFSWE bit enabled
     R_PMISC->PWPRS = 1U << BSP_IO_PWPR_PFSWE_OFFSET; ///< Set PFSWE bit - writing to PFS register enabled
  #else
@@ -410,4 +480,34 @@ static void bsp_reset_trng_circuit (void)
 
 #endif
 
+#if BSP_CFG_EARLY_INIT
+
+/*******************************************************************************************************************//**
+ * Initialize BSP variables not handled by C runtime startup.
+ **********************************************************************************************************************/
+static void bsp_init_uninitialized_vars (void)
+{
+    g_protect_pfswe_counter = 0;
+
+    extern volatile uint16_t g_protect_counters[];
+    for (uint32_t i = 0; i < 4; i++)
+    {
+        g_protect_counters[i] = 0;
+    }
+
+    extern bsp_grp_irq_cb_t g_bsp_group_irq_sources[];
+    for (uint32_t i = 0; i < 16; i++)
+    {
+        g_bsp_group_irq_sources[i] = 0;
+    }
+
+ #if BSP_CFG_EARLY_INIT
+
+    /* Set SystemCoreClock to MOCO */
+    SystemCoreClock = BSP_MOCO_HZ;
+ #endif
+}
+
+#endif
+
 /** @} (end addtogroup BSP_MCU) */
