@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -147,6 +147,16 @@
 /***********************************************************************************************************************
  * Typedef definitions
  ***********************************************************************************************************************/
+#if defined(__ARMCC_VERSION) || defined(__ICCARM__)
+typedef void (BSP_CMSE_NONSECURE_CALL * ether_prv_ns_callback)(ether_callback_args_t * p_args);
+#elif defined(__GNUC__)
+typedef BSP_CMSE_NONSECURE_CALL void (*volatile ether_prv_ns_callback)(ether_callback_args_t * p_args);
+#endif
+
+/***********************************************************************************************************************
+ * Exported global functions (to be accessed by other files)
+ ***********************************************************************************************************************/
+void ether_eint_isr(void);
 
 /***********************************************************************************************************************
  * Exported global variables (to be accessed by other files)
@@ -165,6 +175,7 @@ static void      ether_enable_icu(ether_instance_ctrl_t * const p_instance_ctrl)
 static void      ether_disable_icu(ether_instance_ctrl_t * const p_instance_ctrl);
 static void      ether_reset_mac(R_ETHERC_EDMAC_Type * const p_reg);
 static void      ether_init_descriptors(ether_instance_ctrl_t * const p_instance_ctrl);
+static void      ether_init_buffers(ether_instance_ctrl_t * const p_instance_ctrl);
 static fsp_err_t ether_buffer_get(ether_instance_ctrl_t * const p_instance_ctrl,
                                   void ** const                 p_buffer,
                                   uint32_t                    * p_buffer_size);
@@ -180,6 +191,7 @@ static fsp_err_t ether_do_link(ether_instance_ctrl_t * const p_instance_ctrl, co
 static fsp_err_t ether_link_status_check(ether_instance_ctrl_t const * const p_instance_ctrl);
 static uint8_t   ether_check_magic_packet_detection_bit(ether_instance_ctrl_t const * const p_instance_ctrl);
 static void      ether_configure_padding(ether_instance_ctrl_t * const p_instance_ctrl);
+static void      ether_call_callback(ether_instance_ctrl_t * p_instance_ctrl, ether_callback_args_t * p_callback_args);
 
 /***********************************************************************************************************************
  * Private global variables
@@ -195,31 +207,22 @@ static const char g_module_name[] = "ether";
 /* This structure is affected by warnings from a GCC compiler bug. This pragma suppresses the warnings in this
  * structure only.*/
 
-/*LDRA_INSPECTED 69 S */
  #pragma GCC diagnostic ignored "-Wmissing-field-initializers"
 #endif
 
-/** ETHER HAL module version data structure */
-static const fsp_version_t module_version =
-{
-    .api_version_minor  = ETHER_API_VERSION_MINOR,
-    .api_version_major  = ETHER_API_VERSION_MAJOR,
-    .code_version_major = ETHER_CODE_VERSION_MAJOR,
-    .code_version_minor = ETHER_CODE_VERSION_MINOR
-};
-
-/** ETHER HAL API mapping for Ethernet Controller interface */
-/*LDRA_INSPECTED 27 D This structure must be accessible in user code. It cannot be static. */
+/** ETHER HAL API mapping for Ethernet Controller interface. */
 const ether_api_t g_ether_on_ether =
 {
     .open            = R_ETHER_Open,
     .close           = R_ETHER_Close,
     .read            = R_ETHER_Read,
     .bufferRelease   = R_ETHER_BufferRelease,
+    .rxBufferUpdate  = R_ETHER_RxBufferUpdate,
     .write           = R_ETHER_Write,
     .linkProcess     = R_ETHER_LinkProcess,
     .wakeOnLANEnable = R_ETHER_WakeOnLANEnable,
-    .versionGet      = R_ETHER_VersionGet
+    .txStatusGet     = R_ETHER_TxStatusGet,
+    .callbackSet     = R_ETHER_CallbackSet,
 };
 
 /*
@@ -259,7 +262,7 @@ static const ether_pause_resolution_t pause_resolution[ETHER_PAUSE_TABLE_ENTRIES
  *                                                  instance. Call close() then open() to reconfigure.
  * @retval  FSP_ERR_ETHER_ERROR_PHY_COMMUNICATION   Initialization of PHY-LSI failed.
  * @retval  FSP_ERR_INVALID_CHANNEL                 Invalid channel number is given.
- * @retval  FSP_ERR_INVALID_POINTER                 Pointer to MAC address is NULL.
+ * @retval  FSP_ERR_INVALID_POINTER                 Pointer to extend config structure or MAC address is NULL.
  * @retval  FSP_ERR_INVALID_ARGUMENT                Interrupt is not enabled.
  * @retval  FSP_ERR_ETHER_PHY_ERROR_LINK            Initialization of PHY-LSI failed.
  ***********************************************************************************************************************/
@@ -267,9 +270,9 @@ fsp_err_t R_ETHER_Open (ether_ctrl_t * const p_ctrl, ether_cfg_t const * const p
 {
     fsp_err_t               err             = FSP_SUCCESS;
     ether_instance_ctrl_t * p_instance_ctrl = (ether_instance_ctrl_t *) p_ctrl;
+    ether_extended_cfg_t  * p_ether_extended_cfg;
     R_ETHERC0_Type        * p_reg_etherc;
     R_ETHERC_EDMAC_Type   * p_reg_edmac;
-    uint32_t                i;
 
     fsp_err_t phy_ret;
 
@@ -279,6 +282,7 @@ fsp_err_t R_ETHER_Open (ether_ctrl_t * const p_ctrl, ether_cfg_t const * const p
     err = ether_open_param_check(p_instance_ctrl, p_cfg); /** check arguments */
     ETHER_ERROR_RETURN((FSP_SUCCESS == err), err);
 #endif
+    p_ether_extended_cfg = (ether_extended_cfg_t *) p_cfg->p_extend;
 
     /** Make sure this channel exists. */
     p_instance_ctrl->p_reg_etherc = ((R_ETHERC0_Type *) (R_ETHERC0_BASE + (ETHER_ETHERC_REG_SIZE * p_cfg->channel)));
@@ -298,24 +302,22 @@ fsp_err_t R_ETHER_Open (ether_ctrl_t * const p_ctrl, ether_cfg_t const * const p
     p_instance_ctrl->previous_link_status  = ETHER_PREVIOUS_LINK_STATUS_DOWN;
 
     /* Initialize the transmit and receive descriptor */
-    memset(p_instance_ctrl->p_ether_cfg->p_rx_descriptors,
+    memset(p_ether_extended_cfg->p_rx_descriptors,
            0x00,
            sizeof(ether_instance_descriptor_t) *
            p_instance_ctrl->p_ether_cfg->num_rx_descriptors);
-    memset(p_instance_ctrl->p_ether_cfg->p_tx_descriptors,
+    memset(p_ether_extended_cfg->p_tx_descriptors,
            0x00,
            sizeof(ether_instance_descriptor_t) *
            p_instance_ctrl->p_ether_cfg->num_tx_descriptors);
 
-    /* Initialize the Ether buffer */
-    for (i = 0;
-         i < (p_instance_ctrl->p_ether_cfg->num_tx_descriptors + p_instance_ctrl->p_ether_cfg->num_rx_descriptors);
-         i++)
-    {
-        memset(p_instance_ctrl->p_ether_cfg->pp_ether_buffers[i],
-               0x00,
-               p_instance_ctrl->p_ether_cfg->ether_buffer_size);
-    }
+    /* Initialize the Ethernet buffer */
+    ether_init_buffers(p_instance_ctrl);
+
+    /* Set callback and context pointers, if configured */
+    p_instance_ctrl->p_callback        = p_cfg->p_callback;
+    p_instance_ctrl->p_context         = p_cfg->p_context;
+    p_instance_ctrl->p_callback_memory = NULL;
 
     R_BSP_MODULE_START(FSP_IP_ETHER, p_instance_ctrl->p_ether_cfg->channel);
 
@@ -496,6 +498,74 @@ fsp_err_t R_ETHER_BufferRelease (ether_ctrl_t * const p_ctrl)
     return err;
 }                                      /* End of function R_ETHER_BufferRelease() */
 
+/********************************************************************************************************************//**
+ * @brief Change the buffer pointer of the current rx buffer descriptor. Implements @ref ether_api_t::rxBufferUpdate.
+ *
+ * @retval  FSP_SUCCESS                             Processing completed successfully.
+ * @retval  FSP_ERR_ASSERTION                       A pointer argument is NULL.
+ * @retval  FSP_ERR_NOT_OPEN                        The control block has not been opened.
+ * @retval  FSP_ERR_INVALID_POINTER                 The pointer of buffer is NULL or not aligned on a 32-bit boundary.
+ * @retval  FSP_ERR_INVALID_MODE                    Driver is configured to non zero copy mode.
+ * @retval  FSP_ERR_ETHER_RECEIVE_BUFFER_ACTIVE     All descriptor is active.
+ ***********************************************************************************************************************/
+fsp_err_t R_ETHER_RxBufferUpdate (ether_ctrl_t * const p_ctrl, void * const p_buffer)
+{
+    fsp_err_t               err = FSP_SUCCESS;
+    R_ETHERC_EDMAC_Type   * p_reg_edmac;
+    ether_instance_ctrl_t * p_instance_ctrl = (ether_instance_ctrl_t *) p_ctrl;
+    uint32_t                status;
+
+    /* Check argument */
+#if (ETHER_CFG_PARAM_CHECKING_ENABLE)
+    FSP_ASSERT(p_instance_ctrl);
+    ETHER_ERROR_RETURN(ETHER_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
+    ETHER_ERROR_RETURN(NULL != p_buffer, FSP_ERR_INVALID_POINTER);
+    ETHER_ERROR_RETURN(0 == ((uint32_t) p_buffer & (uint32_t) ETHER_BUFFER_32BYTE_ALIGNMENT_MASK),
+                       FSP_ERR_INVALID_POINTER);
+    ETHER_ERROR_RETURN(ETHER_ZEROCOPY_ENABLE == p_instance_ctrl->p_ether_cfg->zerocopy, FSP_ERR_INVALID_MODE);
+#endif
+
+    if (ETHER_RD0_RACT != (p_instance_ctrl->p_rx_descriptor->status & ETHER_RD0_RACT))
+    {
+        p_instance_ctrl->p_rx_descriptor->p_buffer = p_buffer;
+
+        /* Reset current descriptor */
+        status  = ETHER_RD0_RFP1;
+        status |= ETHER_RD0_RFP0;
+        status |= ETHER_RD0_RFE;
+        status |= ETHER_RD0_RFS9_RFOVER;
+        status |= ETHER_RD0_RFS8_RAD;
+        status |= ETHER_RD0_RFS7_RMAF;
+        status |= ETHER_RD0_RFS4_RRF;
+        status |= ETHER_RD0_RFS3_RTLF;
+        status |= ETHER_RD0_RFS2_RTSF;
+        status |= ETHER_RD0_RFS1_PRE;
+        status |= ETHER_RD0_RFS0_CERF;
+
+        p_instance_ctrl->p_rx_descriptor->status &= (~status);
+
+        /* Enable current descriptor */
+        p_instance_ctrl->p_rx_descriptor->status |= ETHER_RD0_RACT;
+
+        /* Move to next descriptor */
+        p_instance_ctrl->p_rx_descriptor = p_instance_ctrl->p_rx_descriptor->p_next;
+
+        p_reg_edmac = (R_ETHERC_EDMAC_Type *) p_instance_ctrl->p_reg_edmac;
+
+        if (ETHER_EDMAC_EDRRR_RECEIVE_REQUEST != p_reg_edmac->EDRRR)
+        {
+            /* Restart if stopped */
+            p_reg_edmac->EDRRR = ETHER_EDMAC_EDRRR_RECEIVE_REQUEST;
+        }
+    }
+    else
+    {
+        err = FSP_ERR_ETHER_RECEIVE_BUFFER_ACTIVE;
+    }
+
+    return err;
+}
+
 /********************************************************************************************************************//**
  * @brief The Link up processing, the Link down processing, and the magic packet detection processing are executed.
  *  Implements @ref ether_api_t::linkProcess.
@@ -520,24 +590,28 @@ fsp_err_t R_ETHER_LinkProcess (ether_ctrl_t * const p_ctrl)
     ether_callback_args_t                 callback_arg;
     ether_cfg_t const                   * p_ether_cfg;
     volatile ether_previous_link_status_t previous_link_status;
-
-    uint32_t i;
+    ether_extended_cfg_t                * p_ether_extended_cfg;
 
 #if (ETHER_CFG_PARAM_CHECKING_ENABLE)
     FSP_ASSERT(p_instance_ctrl);
     ETHER_ERROR_RETURN(ETHER_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
 #endif
+    p_ether_extended_cfg = (ether_extended_cfg_t *) p_instance_ctrl->p_ether_cfg->p_extend;
 
     /* When the magic packet is detected. */
     if (ETHER_MAGIC_PACKET_DETECTED == p_instance_ctrl->magic_packet)
     {
         p_instance_ctrl->magic_packet = ETHER_MAGIC_PACKET_NOT_DETECTED;
 
-        if (NULL != p_instance_ctrl->p_ether_cfg->p_callback)
+        /* If a callback is provided, then call it with callback argument. */
+        if (NULL != p_instance_ctrl->p_callback)
         {
-            callback_arg.channel = p_instance_ctrl->p_ether_cfg->channel;
-            callback_arg.event   = ETHER_EVENT_WAKEON_LAN;
-            (*p_instance_ctrl->p_ether_cfg->p_callback)((void *) &callback_arg);
+            callback_arg.channel     = p_instance_ctrl->p_ether_cfg->channel;
+            callback_arg.event       = ETHER_EVENT_WAKEON_LAN;
+            callback_arg.status_ecsr = 0;
+            callback_arg.status_eesr = 0;
+            callback_arg.p_context   = p_instance_ctrl->p_ether_cfg->p_context;
+            ether_call_callback(p_instance_ctrl, &callback_arg);
         }
 
         /*
@@ -609,23 +683,15 @@ fsp_err_t R_ETHER_LinkProcess (ether_ctrl_t * const p_ctrl)
             p_instance_ctrl->link_change = ETHER_LINK_CHANGE_LINK_DOWN;
 
             /* Initialize the transmit and receive descriptor */
-            memset(p_instance_ctrl->p_ether_cfg->p_rx_descriptors,
+            memset(p_ether_extended_cfg->p_rx_descriptors,
                    0x00,
                    sizeof(ether_instance_descriptor_t) * p_instance_ctrl->p_ether_cfg->num_rx_descriptors);
-            memset(p_instance_ctrl->p_ether_cfg->p_tx_descriptors,
+            memset(p_ether_extended_cfg->p_tx_descriptors,
                    0x00,
                    sizeof(ether_instance_descriptor_t) * p_instance_ctrl->p_ether_cfg->num_tx_descriptors);
 
-            /* Initialize the Ether buffer */
-            for (i = 0;
-                 i <
-                 (p_instance_ctrl->p_ether_cfg->num_tx_descriptors + p_instance_ctrl->p_ether_cfg->num_rx_descriptors);
-                 i++)
-            {
-                memset(p_instance_ctrl->p_ether_cfg->pp_ether_buffers[i],
-                       0x00,
-                       p_instance_ctrl->p_ether_cfg->ether_buffer_size);
-            }
+            /* Initialize the Ethernet buffer */
+            ether_init_buffers(p_instance_ctrl);
 
             p_instance_ctrl->link_establish_status = ETHER_LINK_ESTABLISH_STATUS_UP;
 
@@ -640,11 +706,15 @@ fsp_err_t R_ETHER_LinkProcess (ether_ctrl_t * const p_ctrl)
 
             if (FSP_SUCCESS == err)
             {
-                if (NULL != p_instance_ctrl->p_ether_cfg->p_callback)
+                /* If a callback is provided, then call it with callback argument. */
+                if (NULL != p_instance_ctrl->p_callback)
                 {
-                    callback_arg.channel = p_instance_ctrl->p_ether_cfg->channel;
-                    callback_arg.event   = ETHER_EVENT_LINK_ON;
-                    (*p_instance_ctrl->p_ether_cfg->p_callback)((void *) &callback_arg);
+                    callback_arg.channel     = p_instance_ctrl->p_ether_cfg->channel;
+                    callback_arg.event       = ETHER_EVENT_LINK_ON;
+                    callback_arg.status_ecsr = 0;
+                    callback_arg.status_eesr = 0;
+                    callback_arg.p_context   = p_instance_ctrl->p_ether_cfg->p_context;
+                    ether_call_callback(p_instance_ctrl, &callback_arg);
                 }
             }
             else
@@ -669,22 +739,15 @@ fsp_err_t R_ETHER_LinkProcess (ether_ctrl_t * const p_ctrl)
         p_instance_ctrl->link_change = ETHER_LINK_CHANGE_NO_CHANGE;
 
         /* Initialize the transmit and receive descriptor */
-        memset(p_instance_ctrl->p_ether_cfg->p_rx_descriptors,
+        memset(p_ether_extended_cfg->p_rx_descriptors,
                0x00,
                sizeof(ether_instance_descriptor_t) * p_instance_ctrl->p_ether_cfg->num_rx_descriptors);
-        memset(p_instance_ctrl->p_ether_cfg->p_tx_descriptors,
+        memset(p_ether_extended_cfg->p_tx_descriptors,
                0x00,
                sizeof(ether_instance_descriptor_t) * p_instance_ctrl->p_ether_cfg->num_tx_descriptors);
 
-        /* Initialize the Ether buffer */
-        for (i = 0;
-             i < (p_instance_ctrl->p_ether_cfg->num_tx_descriptors + p_instance_ctrl->p_ether_cfg->num_rx_descriptors);
-             i++)
-        {
-            memset(p_instance_ctrl->p_ether_cfg->pp_ether_buffers[i],
-                   0x00,
-                   p_instance_ctrl->p_ether_cfg->ether_buffer_size);
-        }
+        /* Initialize the Ethernet buffer */
+        ether_init_buffers(p_instance_ctrl);
 
         p_instance_ctrl->link_establish_status = ETHER_LINK_ESTABLISH_STATUS_UP;
 
@@ -699,11 +762,15 @@ fsp_err_t R_ETHER_LinkProcess (ether_ctrl_t * const p_ctrl)
 
         if (FSP_SUCCESS == err)
         {
-            if (NULL != p_instance_ctrl->p_ether_cfg->p_callback)
+            /* If a callback is provided, then call it with callback argument. */
+            if (NULL != p_instance_ctrl->p_callback)
             {
-                callback_arg.channel = p_instance_ctrl->p_ether_cfg->channel;
-                callback_arg.event   = ETHER_EVENT_LINK_ON;
-                (*p_instance_ctrl->p_ether_cfg->p_callback)((void *) &callback_arg);
+                callback_arg.channel     = p_instance_ctrl->p_ether_cfg->channel;
+                callback_arg.event       = ETHER_EVENT_LINK_ON;
+                callback_arg.status_ecsr = 0;
+                callback_arg.status_eesr = 0;
+                callback_arg.p_context   = p_instance_ctrl->p_ether_cfg->p_context;
+                ether_call_callback(p_instance_ctrl, &callback_arg);
             }
         }
         else
@@ -737,11 +804,15 @@ fsp_err_t R_ETHER_LinkProcess (ether_ctrl_t * const p_ctrl)
 
             p_instance_ctrl->link_establish_status = ETHER_LINK_ESTABLISH_STATUS_DOWN;
 
-            if (NULL != p_instance_ctrl->p_ether_cfg->p_callback)
+            /* If a callback is provided, then call it with callback argument. */
+            if (NULL != p_instance_ctrl->p_callback)
             {
-                callback_arg.channel = p_instance_ctrl->p_ether_cfg->channel;
-                callback_arg.event   = ETHER_EVENT_LINK_OFF;
-                (*p_instance_ctrl->p_ether_cfg->p_callback)((void *) &callback_arg);
+                callback_arg.channel     = p_instance_ctrl->p_ether_cfg->channel;
+                callback_arg.event       = ETHER_EVENT_LINK_OFF;
+                callback_arg.status_ecsr = 0;
+                callback_arg.status_eesr = 0;
+                callback_arg.p_context   = p_instance_ctrl->p_ether_cfg->p_context;
+                ether_call_callback(p_instance_ctrl, &callback_arg);
             }
         }
         else
@@ -758,11 +829,15 @@ fsp_err_t R_ETHER_LinkProcess (ether_ctrl_t * const p_ctrl)
 
         p_instance_ctrl->link_establish_status = ETHER_LINK_ESTABLISH_STATUS_DOWN;
 
-        if (NULL != p_instance_ctrl->p_ether_cfg->p_callback)
+        /* If a callback is provided, then call it with callback argument. */
+        if (NULL != p_instance_ctrl->p_callback)
         {
-            callback_arg.channel = p_instance_ctrl->p_ether_cfg->channel;
-            callback_arg.event   = ETHER_EVENT_LINK_OFF;
-            (*p_instance_ctrl->p_ether_cfg->p_callback)((void *) &callback_arg);
+            callback_arg.channel     = p_instance_ctrl->p_ether_cfg->channel;
+            callback_arg.event       = ETHER_EVENT_LINK_OFF;
+            callback_arg.status_ecsr = 0;
+            callback_arg.status_eesr = 0;
+            callback_arg.p_context   = p_instance_ctrl->p_ether_cfg->p_context;
+            ether_call_callback(p_instance_ctrl, &callback_arg);
         }
 #endif
     }
@@ -995,11 +1070,6 @@ fsp_err_t R_ETHER_Write (ether_ctrl_t * const p_ctrl, void * const p_buffer, uin
     ETHER_ERROR_RETURN(NULL != p_buffer, FSP_ERR_INVALID_POINTER);
     ETHER_ERROR_RETURN((ETHER_MINIMUM_FRAME_SIZE <= frame_length) && (ETHER_MAXIMUM_FRAME_SIZE >= frame_length),
                        FSP_ERR_INVALID_ARGUMENT);
-    if (ETHER_ZEROCOPY_ENABLE == p_instance_ctrl->p_ether_cfg->zerocopy)
-    {
-        ETHER_ERROR_RETURN(0 == ((uint32_t) p_buffer & (uint32_t) ETHER_BUFFER_32BYTE_ALIGNMENT_MASK),
-                           FSP_ERR_INVALID_POINTER);
-    }
 #endif
 
     /* When the Link up processing is not completed, return error */
@@ -1061,22 +1131,112 @@ fsp_err_t R_ETHER_Write (ether_ctrl_t * const p_ctrl, void * const p_buffer, uin
     return err;
 }                                      /* End of function R_ETHER_Write() */
 
-/************************************************************************************************************************
- * DEPRECATED Provides API and code version in the user provided pointer. Implements @ref ether_api_t::versionGet.
+/**********************************************************************************************************************//**
+ * Provides status of Ethernet driver in the user provided pointer. Implements @ref ether_api_t::txStatusGet.
  *
- * @retval  FSP_SUCCESS                  Version information stored in provided p_version.
- * @retval  FSP_ERR_ASSERTION            p_version is NULL.
+ * @retval  FSP_SUCCESS                  Transmit buffer address is stored in provided p_buffer_address.
+ * @retval  FSP_ERR_ASSERTION            Pointer to ETHER control block is NULL.
+ * @retval  FSP_ERR_NOT_OPEN             The control block has not been opened.
+ * @retval  FSP_ERR_INVALID_POINTER      p_status is NULL.
+ * @retval  FSP_ERR_NOT_FOUND            Transmit buffer address has been overwritten in transmit descriptor.
  ***********************************************************************************************************************/
-__INLINE fsp_err_t R_ETHER_VersionGet (fsp_version_t * const p_version)
+fsp_err_t R_ETHER_TxStatusGet (ether_ctrl_t * const p_ctrl, void * const p_buffer_address)
 {
+    ether_instance_ctrl_t       * p_instance_ctrl = (ether_instance_ctrl_t *) p_ctrl;
+    ether_extended_cfg_t        * p_ether_extended_cfg;
+    R_ETHERC_EDMAC_Type         * p_reg_edmac;
+    ether_instance_descriptor_t * p_descriptor;
+    uint8_t ** p_sent_buffer_address = (uint8_t **) p_buffer_address;
+    fsp_err_t  err = FSP_ERR_NOT_FOUND;
+
 #if (ETHER_CFG_PARAM_CHECKING_ENABLE)
-    FSP_ASSERT(p_version);
+    FSP_ASSERT(p_instance_ctrl);
+    ETHER_ERROR_RETURN(ETHER_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
+    ETHER_ERROR_RETURN(NULL != p_buffer_address, FSP_ERR_INVALID_POINTER);
+#endif
+    p_ether_extended_cfg = (ether_extended_cfg_t *) p_instance_ctrl->p_ether_cfg->p_extend;
+
+    p_reg_edmac = (R_ETHERC_EDMAC_Type *) p_instance_ctrl->p_reg_edmac;
+
+    p_descriptor = (ether_instance_descriptor_t *) p_reg_edmac->TDFAR;
+
+    /* Descriptor is NULL, when no packet transmitted. */
+    if (NULL != p_descriptor)
+    {
+        uint32_t num_tx_descriptors = p_instance_ctrl->p_ether_cfg->num_tx_descriptors;
+        ether_instance_descriptor_t * p_tx_descriptors = p_ether_extended_cfg->p_tx_descriptors;
+
+        p_descriptor = (ether_instance_descriptor_t *) ((uint8_t *) p_descriptor - sizeof(ether_instance_descriptor_t));
+
+        if (p_descriptor < p_tx_descriptors)
+        {
+            p_descriptor = &p_tx_descriptors[num_tx_descriptors - 1];
+        }
+
+        /* Check that the descriptor is not overridden. */
+        if ((NULL != p_descriptor->p_buffer) && (ETHER_TD0_TACT != (p_descriptor->status & ETHER_TD0_TACT)))
+        {
+            *p_sent_buffer_address = p_descriptor->p_buffer;
+            err = FSP_SUCCESS;
+        }
+        else
+        {
+            ;
+        }
+    }
+
+    return err;
+}                                      /* End of function R_ETHER_VersionGet() */
+
+/*******************************************************************************************************************//**
+ * Updates the user callback with the option to provide memory for the callback argument structure.
+ * Implements @ref ether_api_t::callbackSet.
+ *
+ * @retval  FSP_SUCCESS                  Callback updated successfully.
+ * @retval  FSP_ERR_ASSERTION            A required pointer is NULL.
+ * @retval  FSP_ERR_NOT_OPEN             The control block has not been opened.
+ * @retval  FSP_ERR_NO_CALLBACK_MEMORY   p_callback is non-secure and p_callback_memory is either secure or NULL.
+ **********************************************************************************************************************/
+fsp_err_t R_ETHER_CallbackSet (ether_ctrl_t * const          p_api_ctrl,
+                               void (                      * p_callback)(ether_callback_args_t *),
+                               void const * const            p_context,
+                               ether_callback_args_t * const p_callback_memory)
+{
+    ether_instance_ctrl_t * p_ctrl = (ether_instance_ctrl_t *) p_api_ctrl;
+
+#if ETHER_CFG_PARAM_CHECKING_ENABLE
+    FSP_ASSERT(p_ctrl);
+    FSP_ASSERT(p_callback);
+    FSP_ERROR_RETURN(ETHER_OPEN == p_ctrl->open, FSP_ERR_NOT_OPEN);
 #endif
 
-    *p_version = module_version;
+#if BSP_TZ_SECURE_BUILD && BSP_FEATURE_ETHER_SUPPORTS_TZ_SECURE
+
+    /* Get security state of p_callback */
+    bool callback_is_secure =
+        (NULL == cmse_check_address_range((void *) p_callback, sizeof(void *), CMSE_AU_NONSECURE));
+
+ #if ETHER_CFG_PARAM_CHECKING_ENABLE
+
+    /* In secure projects, p_callback_memory must be provided in non-secure space if p_callback is non-secure */
+    ether_callback_args_t * const p_callback_memory_checked = cmse_check_pointed_object(p_callback_memory,
+                                                                                        CMSE_AU_NONSECURE);
+    FSP_ERROR_RETURN(callback_is_secure || (NULL != p_callback_memory_checked), FSP_ERR_NO_CALLBACK_MEMORY);
+ #endif
+#endif
+
+    /* Store callback and context */
+#if BSP_TZ_SECURE_BUILD && BSP_FEATURE_ETHER_SUPPORTS_TZ_SECURE
+    p_ctrl->p_callback = callback_is_secure ? p_callback :
+                         (void (*)(ether_callback_args_t *))cmse_nsfptr_create(p_callback);
+#else
+    p_ctrl->p_callback = p_callback;
+#endif
+    p_ctrl->p_context         = p_context;
+    p_ctrl->p_callback_memory = p_callback_memory;
 
     return FSP_SUCCESS;
-}                                      /* End of function R_ETHER_VersionGet() */
+}
 
 /*******************************************************************************************************************//**
  * @} (end addtogroup ETHER)
@@ -1099,7 +1259,7 @@ __INLINE fsp_err_t R_ETHER_VersionGet (fsp_version_t * const p_version)
  * @retval  FSP_ERR_ALREADY_OPEN         Control block has already been opened
  * @retval  FSP_ERR_INVALID_CHANNEL      Invalid channel number is given.
  * @retval  FSP_ERR_INVALID_POINTER      Pointer to MAC address is NULL.
- * @retval  FSP_ERR_INVALID_ARGUMENT      Irq number lower then 0.
+ * @retval  FSP_ERR_INVALID_ARGUMENT     Irq number lower then 0.
  **********************************************************************************************************************/
 static fsp_err_t ether_open_param_check (ether_instance_ctrl_t const * const p_instance_ctrl,
                                          ether_cfg_t const * const           p_cfg)
@@ -1107,15 +1267,21 @@ static fsp_err_t ether_open_param_check (ether_instance_ctrl_t const * const p_i
     FSP_ASSERT(p_instance_ctrl);
     ETHER_ERROR_RETURN((NULL != p_cfg), FSP_ERR_INVALID_POINTER);
     ETHER_ERROR_RETURN((NULL != p_cfg->p_mac_address), FSP_ERR_INVALID_POINTER);
+    ETHER_ERROR_RETURN((NULL != p_cfg->p_extend), FSP_ERR_INVALID_POINTER);
     ETHER_ERROR_RETURN((BSP_FEATURE_ETHER_MAX_CHANNELS > p_cfg->channel), FSP_ERR_INVALID_CHANNEL);
     ETHER_ERROR_RETURN((0 <= p_cfg->irq), FSP_ERR_INVALID_ARGUMENT);
-    ETHER_ERROR_RETURN((p_cfg->padding <= ETEHR_PADDING_3BYTE), FSP_ERR_INVALID_ARGUMENT);
+    ETHER_ERROR_RETURN((p_cfg->padding <= ETHER_PADDING_3BYTE), FSP_ERR_INVALID_ARGUMENT);
 
     if (p_cfg->padding != ETHER_PADDING_DISABLE)
     {
         ETHER_ERROR_RETURN((p_cfg->padding_offset <= ETHER_MAXIMUM_PADDING_OFFSET), FSP_ERR_INVALID_ARGUMENT);
     }
 
+    if (p_cfg->zerocopy == ETHER_ZEROCOPY_DISABLE)
+    {
+        ETHER_ERROR_RETURN((p_cfg->pp_ether_buffers != NULL), FSP_ERR_INVALID_ARGUMENT);
+    }
+
     ETHER_ERROR_RETURN((ETHER_OPEN != p_instance_ctrl->open), FSP_ERR_ALREADY_OPEN);
 
     return FSP_SUCCESS;
@@ -1152,54 +1318,106 @@ static void ether_reset_mac (R_ETHERC_EDMAC_Type * const p_reg)
 static void ether_init_descriptors (ether_instance_ctrl_t * const p_instance_ctrl)
 {
     ether_instance_descriptor_t * p_descriptor = NULL;
-    uint32_t i;
+    uint32_t               i;
+    ether_extended_cfg_t * p_ether_extended_cfg = (ether_extended_cfg_t *) p_instance_ctrl->p_ether_cfg->p_extend;
 
     /* Initialize the receive descriptors */
     for (i = 0; i < p_instance_ctrl->p_ether_cfg->num_rx_descriptors; i++)
     {
-        p_descriptor              = &p_instance_ctrl->p_ether_cfg->p_rx_descriptors[i];
-        p_descriptor->p_buffer    = p_instance_ctrl->p_ether_cfg->pp_ether_buffers[i];
+        p_descriptor              = &p_ether_extended_cfg->p_rx_descriptors[i];
         p_descriptor->buffer_size = (uint16_t) p_instance_ctrl->p_ether_cfg->ether_buffer_size;
         p_descriptor->size        = 0;
-        p_descriptor->status      = ETHER_RD0_RACT;
-        p_descriptor->p_next      = &p_instance_ctrl->p_ether_cfg->p_rx_descriptors[(i + 1)];
+        p_descriptor->p_next      = &p_ether_extended_cfg->p_rx_descriptors[(i + 1)];
+
+        if (NULL != p_instance_ctrl->p_ether_cfg->pp_ether_buffers)
+        {
+            p_descriptor->p_buffer = p_instance_ctrl->p_ether_cfg->pp_ether_buffers[i];
+            p_descriptor->status   = ETHER_RD0_RACT;
+        }
+        else
+        {
+            p_descriptor->p_buffer = NULL;
+        }
     }
 
     if (NULL != p_descriptor)
     {
         /* The last descriptor points back to the start */
         p_descriptor->status |= ETHER_RD0_RDLE;
-        p_descriptor->p_next  = &p_instance_ctrl->p_ether_cfg->p_rx_descriptors[0];
+        p_descriptor->p_next  = &p_ether_extended_cfg->p_rx_descriptors[0];
 
         /* Initialize application receive descriptor pointer */
-        p_instance_ctrl->p_rx_descriptor = &p_instance_ctrl->p_ether_cfg->p_rx_descriptors[0];
+        p_instance_ctrl->p_rx_descriptor = &p_ether_extended_cfg->p_rx_descriptors[0];
     }
 
     /* Initialize the transmit descriptors */
     for (i = 0; i < p_instance_ctrl->p_ether_cfg->num_tx_descriptors; i++)
     {
-        p_descriptor           = &p_instance_ctrl->p_ether_cfg->p_tx_descriptors[i];
-        p_descriptor->p_buffer =
-            p_instance_ctrl->p_ether_cfg->pp_ether_buffers[(p_instance_ctrl->p_ether_cfg->num_rx_descriptors + i)];
+        p_descriptor              = &p_ether_extended_cfg->p_tx_descriptors[i];
         p_descriptor->buffer_size = 1; /* Set a value equal to or greater than 1. (reference to UMH)
                                         * When transmitting data, the value of size is set to the function argument
                                         * R_ETHER_Write. */
         p_descriptor->size        = 0; /* Reserved : The write value should be 0. (reference to UMH) */
         p_descriptor->status      = 0;
-        p_descriptor->p_next      = &p_instance_ctrl->p_ether_cfg->p_tx_descriptors[(i + 1)];
+        p_descriptor->p_next      = &p_ether_extended_cfg->p_tx_descriptors[(i + 1)];
+
+        if ((ETHER_ZEROCOPY_DISABLE == p_instance_ctrl->p_ether_cfg->zerocopy) &&
+            (NULL != p_instance_ctrl->p_ether_cfg->pp_ether_buffers))
+        {
+            p_descriptor->p_buffer =
+                p_instance_ctrl->p_ether_cfg->pp_ether_buffers[(p_instance_ctrl->p_ether_cfg->num_rx_descriptors + i)];
+        }
+        else
+        {
+            p_descriptor->p_buffer = NULL;
+        }
     }
 
     if (NULL != p_descriptor)
     {
         /* The last descriptor points back to the start */
         p_descriptor->status |= ETHER_TD0_TDLE;
-        p_descriptor->p_next  = &p_instance_ctrl->p_ether_cfg->p_tx_descriptors[0];
+        p_descriptor->p_next  = &p_ether_extended_cfg->p_tx_descriptors[0];
 
         /* Initialize application transmit descriptor pointer */
-        p_instance_ctrl->p_tx_descriptor = &p_instance_ctrl->p_ether_cfg->p_tx_descriptors[0];
+        p_instance_ctrl->p_tx_descriptor = &p_ether_extended_cfg->p_tx_descriptors[0];
     }
 }                                      /* End of function ether_init_descriptors() */
 
+/***********************************************************************************************************************
+ * Function Name: ether_init_buffers
+ * Description  : The driver buffers are initialized.
+ * Arguments    : p_instance_ctrl -
+ *                    ETHERC control block.
+ * Return Value : none
+ ***********************************************************************************************************************/
+static void ether_init_buffers (ether_instance_ctrl_t * const p_instance_ctrl)
+{
+    uint32_t i;
+    uint32_t buffer_num;
+
+    if (NULL != p_instance_ctrl->p_ether_cfg->pp_ether_buffers)
+    {
+        if (ETHER_ZEROCOPY_DISABLE == p_instance_ctrl->p_ether_cfg->zerocopy)
+        {
+            buffer_num =
+                (uint32_t) (p_instance_ctrl->p_ether_cfg->num_tx_descriptors +
+                            p_instance_ctrl->p_ether_cfg->num_rx_descriptors);
+        }
+        else
+        {
+            buffer_num = (uint32_t) p_instance_ctrl->p_ether_cfg->num_rx_descriptors;
+        }
+
+        for (i = 0; i < buffer_num; i++)
+        {
+            memset(p_instance_ctrl->p_ether_cfg->pp_ether_buffers[i],
+                   0x00,
+                   p_instance_ctrl->p_ether_cfg->ether_buffer_size);
+        }
+    }
+}                                      /* End of function ether_init_buffers() */
+
 /********************************************************************************************************************//**
  * @brief Get Points to the buffer pointer used by the stack.
  * @param[in]  p_instance_ctrl                              Ethernet driver control block.
@@ -1548,9 +1766,7 @@ static fsp_err_t ether_do_link (ether_instance_ctrl_t * const p_instance_ctrl, c
                      * Enable PAUSE for full duplex link depending on
                      * the pause resolution results
                      */
-                    ether_pause_resolution(local_pause_bits,
-                                           partner_pause_bits,
-                                           &transmit_pause_set,
+                    ether_pause_resolution(local_pause_bits, partner_pause_bits, &transmit_pause_set,
                                            &receive_pause_set);
 
                     if (ETHER_PAUSE_XMIT_ON == transmit_pause_set)
@@ -1674,6 +1890,64 @@ static fsp_err_t ether_link_status_check (ether_instance_ctrl_t const * const p_
     return err;
 }                                      /* End of function ether_link_status_check() */
 
+/*******************************************************************************************************************//**
+ * Calls user callback.
+ *
+ * @param[in]     p_instance_ctrl      Pointer to ether instance control block
+ * @param[in]     p_callback_args      Pointer to callback args
+ **********************************************************************************************************************/
+static void ether_call_callback (ether_instance_ctrl_t * p_instance_ctrl, ether_callback_args_t * p_callback_args)
+{
+    ether_callback_args_t args;
+
+    /* Store callback arguments in memory provided by user if available.  This allows callback arguments to be
+     * stored in non-secure memory so they can be accessed by a non-secure callback function. */
+    ether_callback_args_t * p_args = p_instance_ctrl->p_callback_memory;
+    if (NULL == p_args)
+    {
+        /* Store on stack */
+        p_args = &args;
+    }
+    else
+    {
+        /* Save current arguments on the stack in case this is a nested interrupt. */
+        args = *p_args;
+    }
+
+    p_args->event       = p_callback_args->event;
+    p_args->status_ecsr = p_callback_args->status_ecsr;
+    p_args->status_eesr = p_callback_args->status_eesr;
+    p_args->channel     = p_instance_ctrl->p_ether_cfg->channel;
+    p_args->p_context   = p_instance_ctrl->p_context;
+
+#if BSP_TZ_SECURE_BUILD && BSP_FEATURE_ETHER_SUPPORTS_TZ_SECURE
+
+    /* p_callback can point to a secure function or a non-secure function. */
+    if (!cmse_is_nsfptr(p_instance_ctrl->p_callback))
+    {
+        /* If p_callback is secure, then the project does not need to change security state. */
+        p_instance_ctrl->p_callback(p_args);
+    }
+    else
+    {
+        /* If p_callback is Non-secure, then the project must change to Non-secure state in order to call the callback. */
+        ether_prv_ns_callback p_callback = (ether_prv_ns_callback) (p_instance_ctrl->p_callback);
+        p_callback(p_args);
+    }
+
+#else
+
+    /* If the project is not Trustzone Secure, then it will never need to change security state in order to call the callback. */
+    p_instance_ctrl->p_callback(p_args);
+#endif
+
+    if (NULL != p_instance_ctrl->p_callback_memory)
+    {
+        /* Restore callback memory in case this is a nested interrupt. */
+        *p_instance_ctrl->p_callback_memory = args;
+    }
+}
+
 /***********************************************************************************************************************
  * Function Name: ether_eint_isr
  * Description  : Interrupt handler for Ethernet receive and transmit interrupts.
@@ -1682,6 +1956,9 @@ static fsp_err_t ether_link_status_check (ether_instance_ctrl_t const * const p_
  ***********************************************************************************************************************/
 void ether_eint_isr (void)
 {
+    /* Save context if RTOS is used */
+    FSP_CONTEXT_SAVE
+
     uint32_t status_ecsr;
     uint32_t status_eesr;
 
@@ -1698,16 +1975,6 @@ void ether_eint_isr (void)
     status_ecsr = p_reg_etherc->ECSR;
     status_eesr = p_reg_edmac->EESR;
 
-    /* Callback : Interrupt handler */
-    if (NULL != p_instance_ctrl->p_ether_cfg->p_callback)
-    {
-        callback_arg.channel     = p_instance_ctrl->p_ether_cfg->channel;
-        callback_arg.event       = ETHER_EVENT_INTERRUPT;
-        callback_arg.status_ecsr = status_ecsr;
-        callback_arg.status_eesr = status_eesr;
-        (*p_instance_ctrl->p_ether_cfg->p_callback)((void *) &callback_arg);
-    }
-
     /* When the ETHERC status interrupt is generated */
     if (status_eesr & ETHER_EDMAC_INTERRUPT_FACTOR_ECI)
     {
@@ -1749,10 +2016,24 @@ void ether_eint_isr (void)
      */
     p_reg_edmac->EESR = status_eesr;      /* Clear EDMAC status bits */
 
+    /* If a callback is provided, then call it with callback argument. */
+    if (NULL != p_instance_ctrl->p_callback)
+    {
+        callback_arg.channel     = p_instance_ctrl->p_ether_cfg->channel;
+        callback_arg.event       = ETHER_EVENT_INTERRUPT;
+        callback_arg.status_ecsr = status_ecsr;
+        callback_arg.status_eesr = status_eesr;
+        callback_arg.p_context   = p_instance_ctrl->p_ether_cfg->p_context;
+        ether_call_callback(p_instance_ctrl, &callback_arg);
+    }
+
     /* Clear pending interrupt flag to make sure it doesn't fire again
      * after exiting. */
     R_BSP_IrqStatusClear(R_FSP_CurrentIrqGet());
-}/* End of function ether_eint_isr() */
+
+    /* Restore context if RTOS is used */
+    FSP_CONTEXT_RESTORE
+}                                      /* End of function ether_eint_isr() */
 
 /***********************************************************************************************************************
  * Function Name: ether_enable_icu
