@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -252,15 +252,6 @@ static void r_glcdc_gamma_correction(display_cfg_t const * const p_cfg);
  * Private global variables
  **********************************************************************************************************************/
 
-/* GLCDC HAL module version data structure */
-static const fsp_version_t module_version =
-{
-    .api_version_minor  = DISPLAY_API_VERSION_MINOR,
-    .api_version_major  = DISPLAY_API_VERSION_MAJOR,
-    .code_version_major = GLCDC_CODE_VERSION_MAJOR,
-    .code_version_minor = GLCDC_CODE_VERSION_MINOR
-};
-
 /* GLCDC HAL module API function pointer list */
 const display_api_t g_display_on_glcdc =
 {
@@ -275,8 +266,7 @@ const display_api_t g_display_on_glcdc =
 #if GLCDC_CFG_COLOR_CORRECTION_ENABLE
     .correction   = R_GLCDC_ColorCorrection,
 #endif
-    .statusGet  = R_GLCDC_StatusGet,
-    .versionGet = R_GLCDC_VersionGet
+    .statusGet    = R_GLCDC_StatusGet,
 };
 
 /* GLCDC control block */
@@ -302,7 +292,7 @@ static volatile bool g_clut_data_latched[2] = {true, true};
 /* Look-up table for r_glcdc_tcon_set */
 static uint32_t volatile * g_tcon_lut[] =
 {
-    &(R_GLCDC->TCON.STVA2), &(R_GLCDC->TCON.STVB2), &(R_GLCDC->TCON.STHA2), &(R_GLCDC->TCON.STHB2)
+    &(R_GLCDC->TCON.STVA2), &(R_GLCDC->TCON.STVB2), &(R_GLCDC->TCON.STHA2), &(R_GLCDC->TCON.STHB2),
 };
 
 /* Look-up table for converting display_in_format_t to glcdc_input_interface_format_t */
@@ -488,14 +478,14 @@ fsp_err_t R_GLCDC_Close (display_ctrl_t * const p_api_ctrl)
 
     if (p_ctrl->p_cfg->underflow_2_irq >= 0)
     {
-        R_BSP_IrqDisable(p_ctrl->p_cfg->underflow_1_irq);
+        R_BSP_IrqDisable(p_ctrl->p_cfg->underflow_2_irq);
     }
 
     /* Disable the GLCDC interrupts */
     R_GLCDC->SYSCNT.DTCTEN = 0U;
     R_GLCDC->SYSCNT.INTEN  = 0U;
 
-    /**< disables background plane operation */
+    /* Disable background plane operation */
     R_GLCDC->BG.EN_b.EN = 0U;
 
     /* Reset the GLCDC hardware */
@@ -570,8 +560,7 @@ fsp_err_t R_GLCDC_Stop (display_ctrl_t * const p_api_ctrl)
     FSP_ERROR_RETURN(false == (bool) (R_GLCDC->OUT.VLATCH_b.VEN), FSP_ERR_INVALID_UPDATE_TIMING);
     FSP_ERROR_RETURN(false == (bool) (R_GLCDC->BG.EN_b.VEN), FSP_ERR_INVALID_UPDATE_TIMING);
 
-    /* Stop outputting the vertical and horizontal synchronization signals and screen data. */
-    /**< disables background plane operation */
+    /* Disable background plane operation */
     R_GLCDC->BG.EN_b.EN = 0U;
 
     p_ctrl->state = DISPLAY_STATE_OPENED;
@@ -865,19 +854,6 @@ fsp_err_t R_GLCDC_StatusGet (display_ctrl_t const * const p_api_ctrl, display_st
     return FSP_SUCCESS;
 }
 
-/***********************************************************************************************************************
- * DEPRECATED Get version of R_GLCDC module. Implements display_api_t::versionGet.
- *
- * @retval  FSP_SUCCESS   Got version information successfully.
- * @note    This function is re-entrant.
- **********************************************************************************************************************/
-fsp_err_t R_GLCDC_VersionGet (fsp_version_t * p_version)
-{
-    *p_version = module_version;
-
-    return FSP_SUCCESS;
-}
-
 /* @} (end addtogroup GLCDC) */
 
 /***********************************************************************************************************************
@@ -1261,16 +1237,22 @@ static void r_glcdc_tcon_set (glcdc_tcon_pin_t           tcon,
                               glcdc_tcon_signal_select_t signal,
                               display_signal_polarity_t  polarity)
 {
-    uint32_t invert = 0;
-
     /* Set signal polarity */
     if (DISPLAY_SIGNAL_POLARITY_LOACTIVE == polarity)
     {
-        invert = GLCDC_PRV_TCON_STXX2_INV_SET;
+        /* Set INV bit depending on register */
+        if (GLCDC_TCON_SIGNAL_SELECT_DE != signal)
+        {
+            *g_tcon_lut[signal] |= GLCDC_PRV_TCON_STXX2_INV_SET;
+        }
+        else
+        {
+            R_GLCDC->TCON.DE = 1;
+        }
     }
 
-    /* Write to register */
-    *g_tcon_lut[tcon] = signal | invert;
+    /* Set TCON signal */
+    *g_tcon_lut[tcon] |= signal;
 }
 
 /*******************************************************************************************************************//**
