@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -70,15 +70,6 @@ static inline void r_gpt_write_protect_disable_all(gpt_three_phase_instance_ctrl
  * Private global variables
  **********************************************************************************************************************/
 
-/* Version data structure used by error logger macro. */
-static const fsp_version_t g_gpt_three_phase_version =
-{
-    .api_version_minor  = THREE_PHASE_API_VERSION_MINOR,
-    .api_version_major  = THREE_PHASE_API_VERSION_MAJOR,
-    .code_version_major = GPT_THREE_PHASE_CODE_VERSION_MAJOR,
-    .code_version_minor = GPT_THREE_PHASE_CODE_VERSION_MINOR
-};
-
 /***********************************************************************************************************************
  * Global Variables
  **********************************************************************************************************************/
@@ -93,7 +84,6 @@ const three_phase_api_t g_gpt_three_phase_on_gpt_three_phase =
     .dutyCycleSet = R_GPT_THREE_PHASE_DutyCycleSet,
     .callbackSet  = R_GPT_THREE_PHASE_CallbackSet,
     .close        = R_GPT_THREE_PHASE_Close,
-    .versionGet   = R_GPT_THREE_PHASE_VersionGet
 };
 
 /*******************************************************************************************************************//**
@@ -303,7 +293,8 @@ fsp_err_t R_GPT_THREE_PHASE_DutyCycleSet (three_phase_ctrl_t * const       p_ctr
         p_instance_ctrl->p_reg[ch]->GTCCR[GPT_THREE_PHASE_PRV_GTCCRE] = p_duty_cycle->duty[ch];
 
         /* Set double-buffer registers (if applicable) */
-        if (THREE_PHASE_BUFFER_MODE_DOUBLE == p_instance_ctrl->buffer_mode)
+        if ((THREE_PHASE_BUFFER_MODE_DOUBLE == p_instance_ctrl->buffer_mode) ||
+            (TIMER_MODE_TRIANGLE_WAVE_ASYMMETRIC_PWM_MODE3 == p_instance_ctrl->p_cfg->p_timer_instance[0]->p_cfg->mode))
         {
             p_instance_ctrl->p_reg[ch]->GTCCR[GPT_THREE_PHASE_PRV_GTCCRD] = p_duty_cycle->duty_buffer[ch];
             p_instance_ctrl->p_reg[ch]->GTCCR[GPT_THREE_PHASE_PRV_GTCCRF] = p_duty_cycle->duty_buffer[ch];
@@ -378,25 +369,6 @@ fsp_err_t R_GPT_THREE_PHASE_Close (three_phase_ctrl_t * const p_ctrl)
     return err;
 }
 
-/***********************************************************************************************************************
- * DEPRECATED Sets driver version based on compile time macros. Implements @ref three_phase_api_t::versionGet.
- *
- * @retval FSP_SUCCESS                 Version stored in p_version.
- * @retval FSP_ERR_ASSERTION           p_version was NULL.
- **********************************************************************************************************************/
-fsp_err_t R_GPT_THREE_PHASE_VersionGet (fsp_version_t * const p_version)
-{
-#if GPT_THREE_PHASE_CFG_PARAM_CHECKING_ENABLE
-
-    /* Verify parameters are valid */
-    FSP_ASSERT(NULL != p_version);
-#endif
-
-    p_version->version_id = g_gpt_three_phase_version.version_id;
-
-    return FSP_SUCCESS;
-}
-
 /** @} (end addtogroup GPT_THREE_PHASE) */
 
 /*******************************************************************************************************************//**
