@@ -165,7 +165,7 @@ void            LCDCONF_DisableDave2D(void);
 void            LCDCONF_EnableDave2D(void);
 
  #if EMWIN_JPEG_USE_HW
-extern void JPEG_X_Init(JPEG_X_CONTEXT * pContext);
+extern void JPEG_X_Init(void);
 void        LCDCONF_DrawJPEG(int32_t      LayerIndex,
                              int32_t      x,
                              int32_t      y,
@@ -253,7 +253,7 @@ static void _GraphicsHWInit (void)
     // Initialize JPEG Codec
     //
  #if EMWIN_JPEG_USE_HW
-    JPEG_X_Init(NULL);
+    JPEG_X_Init();
  #endif
 #endif
 
@@ -321,18 +321,8 @@ static void _SetLUTEntry (LCD_COLOR Color, uint8_t Pos)
 
 #if (EMWIN_LCD_NUM_FRAMEBUFFERS == 2)
 
-/*********************************************************************
- *
- *       _SwitchBuffersOnVSYNC
- */
-static void _SwitchBuffersOnVSYNC (int32_t Index)
+static void vsync_wait (void)
 {
-    //
-    // Swap buffer
-    //
-    R_GLCDC_BufferChange(_g_display_emwin->p_ctrl, (uint8_t *) pp_buffer_address[Index], (display_frame_layer_t) 0);
-    GUI_MULTIBUF_ConfirmEx(0, Index);  // Tell emWin that buffer is used
-
  #if EMWIN_LCD_VSYNC_WAIT
   #if EMWIN_CFG_RTOS == 2              // FreeRTOS
     //
@@ -354,6 +344,33 @@ static void _SwitchBuffersOnVSYNC (int32_t Index)
  #endif
 }
 
+/*********************************************************************
+ *
+ *       _SwitchBuffersOnVSYNC
+ */
+static void _SwitchBuffersOnVSYNC (int32_t Index)
+{
+    //
+    // Swap buffer
+    //
+    fsp_err_t err;
+    do
+    {
+        err =
+            R_GLCDC_BufferChange(_g_display_emwin->p_ctrl,
+                                 (uint8_t *) pp_buffer_address[Index],
+                                 (display_frame_layer_t) 0);
+        if (err)
+        {
+            vsync_wait();
+        }
+    } while (FSP_ERR_INVALID_UPDATE_TIMING == err);
+
+    GUI_MULTIBUF_ConfirmEx(0, Index);  // Tell emWin that buffer is used
+
+    vsync_wait();
+}
+
 #endif
 
 /*********************************************************************
@@ -1048,23 +1065,55 @@ static int _DrawArcAA (int x0, int y0, int rx, int ry, long a0, long a1)
     d2_color         Color;
     const GUI_RECT * pRect;
     uint16_t         PenSize;
+    U32              Flag;
 
     GUI_USE_PARA(ry);
-    PenSize = GUI_GetPenSize();
-    Mode    = _GetD2Mode();
-    while (a1 < a0)
+
+    //
+    // If both angles are equal (e.g. 0 and 0 or 180 and 180) nothing has to be done
+    //
+    if (a0 == a1)
     {
-        a1 += 360000;
+        return 0;                      // Nothing to do, no angle - no arc
     }
 
-    nx0 = GUI__SinHQ(a0);
-    ny0 = GUI__CosHQ(a0);
+    if (a1 < a0)
+    {
+        return 0;                      // Nothing to do, emWin doesn't support this one
+    }
+
+    //
+    // If the angles not equal, but meet at the same position
+    // we don't draw an arc but a circle instead.
+    //
+    if (a1 > (a0 + 360000))
+    {
+        return _DrawCircleAA(x0, y0, rx); // a1 meets a0 after one round so we have a circle
+    }
+
+    if ((a0 % 360000) == (a1 % 360000))
+    {
+        return _DrawCircleAA(x0, y0, rx); // Both angles are at the same position but not equal, so we have a circle
+    }
+
+    PenSize = GUI_GetPenSize();
+    Mode    = _GetD2Mode();
+
+    nx0 = -GUI__SinHQ(a0);
+    ny0 = -GUI__CosHQ(a0);
     nx1 = GUI__SinHQ(a1);
     ny1 = GUI__CosHQ(a1);
-    if ((a1 - a0) <= 180000)
+
+    //
+    // If the difference between both is larger than 180 degrees we must use the concave flag
+    //
+    if (((a1 - a0) % 360000) <= 180000)
     {
-        nx0 *= -1;
-        ny0 *= -1;
+        Flag = 0;
+    }
+    else
+    {
+        Flag = d2_wf_concave;
     }
 
     //
@@ -1090,7 +1139,7 @@ static int _DrawArcAA (int x0, int y0, int rx, int ry, long a0, long a1)
                          ny0,
                          nx1,
                          ny1,
-                         0);
+                         Flag);
 
     //
     // Execute render operations
@@ -1215,6 +1264,11 @@ void LCD_X_Config (void)
     //
     _GraphicsHWInit();
 
+    //
+    // Set framebuffer pointer
+    //
+    _fb_emwin = EMWIN_LCD_FRAMEBUFFER_PTR;
+
     //
     // Set graphics HW deinit callback for GUI_Exit
     //
