@@ -16,17 +16,9 @@
 #define PIMORONI_PICOLIPO_16MB
 
 // --- BOARD SPECIFIC ---
-#ifndef PICOLIPO_USER_SW_PIN
 #define PICOLIPO_USER_SW_PIN 23
-#endif
-
-#ifndef PICOLIPO_VBUS_DETECT_PIN
 #define PICOLIPO_VBUS_DETECT_PIN 24
-#endif
-
-#ifndef PICOLIPO_BAT_SENSE_PIN
 #define PICOLIPO_BAT_SENSE_PIN 29
-#endif
 
 // --- UART ---
 #ifndef PICO_DEFAULT_UART
