@@ -4,21 +4,21 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-// For MHZ definitions etc
+// For frequency and PLL definitions etc.
 #include "hardware/clocks.h"
 #include "hardware/pll.h"
 #include "hardware/resets.h"
 
 /// \tag::pll_init_calculations[]
 void pll_init(PLL pll, uint refdiv, uint vco_freq, uint post_div1, uint post_div2) {
-    uint32_t ref_mhz = XOSC_MHZ / refdiv;
+    uint32_t ref_freq = XOSC_KHZ * KHZ / refdiv;
 
     // Check vco freq is in an acceptable range
-    assert(vco_freq >= (PICO_PLL_VCO_MIN_FREQ_MHZ * MHZ) && vco_freq <= (PICO_PLL_VCO_MAX_FREQ_MHZ * MHZ));
+    assert(vco_freq >= (PICO_PLL_VCO_MIN_FREQ_KHZ * KHZ) && vco_freq <= (PICO_PLL_VCO_MAX_FREQ_KHZ * KHZ));
 
     // What are we multiplying the reference clock by to get the vco freq
     // (The regs are called div, because you divide the vco output and compare it to the refclk)
-    uint32_t fbdiv = vco_freq / (ref_mhz * MHZ);
+    uint32_t fbdiv = vco_freq / ref_freq;
 /// \end::pll_init_calculations[]
 
     // fbdiv
@@ -32,7 +32,7 @@ void pll_init(PLL pll, uint refdiv, uint vco_freq, uint post_div1, uint post_div
     // postdiv1 is designed to operate with a higher input frequency than postdiv2
 
     // Check that reference frequency is no greater than vco / 16
-    assert(ref_mhz <= (vco_freq / 16));
+    assert(ref_freq <= (vco_freq / 16));
 
     // div1 feeds into div2 so if div1 is 5 and div2 is 2 then you get a divide by 10
     uint32_t pdiv = (post_div1 << PLL_PRIM_POSTDIV1_LSB) |
