@@ -4,8 +4,8 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#ifndef _HARDWARE_XOSC_H_
-#define _HARDWARE_XOSC_H_
+#ifndef _HARDWARE_XOSC_H
+#define _HARDWARE_XOSC_H
 
 #include "pico.h"
 #include "hardware/structs/xosc.h"
