@@ -4,8 +4,8 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#ifndef _PICO_UNIQUE_ID_H_
-#define _PICO_UNIQUE_ID_H_
+#ifndef _PICO_UNIQUE_ID_H
+#define _PICO_UNIQUE_ID_H
 
 #include "pico.h"
 
