@@ -46,7 +46,7 @@ import xmodem
 
 import_serial_module = True
 
-# When SDK release, plase set SDK_RELEASE as True.
+# When SDK release, please set SDK_RELEASE as True.
 SDK_RELEASE = False
 
 if SDK_RELEASE :
@@ -508,7 +508,7 @@ def main():
 	do_wait_reset = True
 	if ConfigArgs.AUTO_RESET:
 		if subprocess.call("cd " + sys.path[0] + "; ./reset_board.sh", shell=True) == 0:
-			print("auto reset board sucess!!")
+			print("auto reset board success!!")
 			do_wait_reset = False
 			bootrom_msg = writer.cancel_autoboot()
 
@@ -536,7 +536,7 @@ def main():
 
 	# Remove files
 	if ConfigArgs.ERASE_NAME :
-		print(">>> Remove exisiting files ...")
+		print(">>> Remove existing files ...")
 		writer.delete_files(ConfigArgs.ERASE_NAME)
 
 	# Install files
