@@ -47,17 +47,20 @@
 #include "py/ringbuf.h"
 
 #include "mpconfigport.h"
+
+#if MICROPY_PY_ESPNOW
+
 #include "mphalport.h"
 #include "modnetwork.h"
 #include "modespnow.h"
 
-#ifndef MICROPY_ESPNOW_RSSI
+#ifndef MICROPY_PY_ESPNOW_RSSI
 // Include code to track rssi of peers
-#define MICROPY_ESPNOW_RSSI 1
+#define MICROPY_PY_ESPNOW_RSSI 1
 #endif
-#ifndef MICROPY_ESPNOW_EXTRA_PEER_METHODS
+#ifndef MICROPY_PY_ESPNOW_EXTRA_PEER_METHODS
 // Include mod_peer(),get_peer(),peer_count()
-#define MICROPY_ESPNOW_EXTRA_PEER_METHODS 1
+#define MICROPY_PY_ESPNOW_EXTRA_PEER_METHODS 1
 #endif
 
 // Relies on gcc Variadic Macros and Statement Expressions
@@ -71,10 +74,10 @@ static const uint8_t ESPNOW_MAGIC = 0x99;
 typedef struct {
     uint8_t magic;              // = ESPNOW_MAGIC
     uint8_t msg_len;            // Length of the message
-    #if MICROPY_ESPNOW_RSSI
+    #if MICROPY_PY_ESPNOW_RSSI
     uint32_t time_ms;           // Timestamp (ms) when packet is received
     int8_t rssi;                // RSSI value (dBm) (-127 to 0)
-    #endif // MICROPY_ESPNOW_RSSI
+    #endif // MICROPY_PY_ESPNOW_RSSI
 } __attribute__((packed)) espnow_hdr_t;
 
 typedef struct {
@@ -117,9 +120,9 @@ typedef struct _esp_espnow_obj_t {
     size_t peer_count;              // Cache the # of peers for send(sync=True)
     mp_obj_t recv_cb;               // Callback when a packet is received
     mp_obj_t recv_cb_arg;           // Argument passed to callback
-    #if MICROPY_ESPNOW_RSSI
+    #if MICROPY_PY_ESPNOW_RSSI
     mp_obj_t peers_table;           // A dictionary of discovered peers
-    #endif // MICROPY_ESPNOW_RSSI
+    #endif // MICROPY_PY_ESPNOW_RSSI
 } esp_espnow_obj_t;
 
 const mp_obj_type_t esp_espnow_type;
@@ -146,7 +149,7 @@ static esp_espnow_obj_t *_get_singleton_initialised() {
 
 // Allocate and initialise the ESPNow module as a singleton.
 // Returns the initialised espnow_singleton.
-STATIC mp_obj_t espnow_make_new(const mp_obj_type_t *type, size_t n_args,
+static mp_obj_t espnow_make_new(const mp_obj_type_t *type, size_t n_args,
     size_t n_kw, const mp_obj_t *all_args) {
 
     // The espnow_singleton must be defined in MICROPY_PORT_ROOT_POINTERS
@@ -164,11 +167,11 @@ STATIC mp_obj_t espnow_make_new(const mp_obj_type_t *type, size_t n_args,
     self->recv_timeout_ms = DEFAULT_RECV_TIMEOUT_MS;
     self->recv_buffer = NULL;       // Buffer is allocated in espnow_init()
     self->recv_cb = mp_const_none;
-    #if MICROPY_ESPNOW_RSSI
+    #if MICROPY_PY_ESPNOW_RSSI
     self->peers_table = mp_obj_new_dict(0);
     // Prevent user code modifying the dict
     mp_obj_dict_get_map(self->peers_table)->is_fixed = 1;
-    #endif // MICROPY_ESPNOW_RSSI
+    #endif // MICROPY_PY_ESPNOW_RSSI
 
     // Set the global singleton pointer for the espnow protocol.
     MP_STATE_PORT(espnow_singleton) = self;
@@ -177,9 +180,9 @@ STATIC mp_obj_t espnow_make_new(const mp_obj_type_t *type, size_t n_args,
 }
 
 // Forward declare the send and recv ESPNow callbacks
-STATIC void send_cb(const uint8_t *mac_addr, esp_now_send_status_t status);
+static void send_cb(const uint8_t *mac_addr, esp_now_send_status_t status);
 
-STATIC void recv_cb(const esp_now_recv_info_t *recv_info, const uint8_t *msg, int msg_len);
+static void recv_cb(const esp_now_recv_info_t *recv_info, const uint8_t *msg, int msg_len);
 
 // ESPNow.init(): Initialise the data buffers and ESP-NOW functions.
 // Initialise the Espressif ESPNOW software stack, register callbacks and
@@ -202,7 +205,7 @@ static mp_obj_t espnow_init(mp_obj_t _) {
 // ESPNow.deinit(): De-initialise the ESPNOW software stack, disable callbacks
 // and deallocate the recv data buffers.
 // Note: this function is called from main.c:mp_task() to cleanup before soft
-// reset, so cannot be declared STATIC and must guard against self == NULL;.
+// reset, so cannot be declared static and must guard against self == NULL;.
 mp_obj_t espnow_deinit(mp_obj_t _) {
     esp_espnow_obj_t *self = _get_singleton();
     if (self != NULL && self->recv_buffer != NULL) {
@@ -217,7 +220,7 @@ mp_obj_t espnow_deinit(mp_obj_t _) {
     return mp_const_none;
 }
 
-STATIC mp_obj_t espnow_active(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t espnow_active(size_t n_args, const mp_obj_t *args) {
     esp_espnow_obj_t *self = _get_singleton();
     if (n_args > 1) {
         if (mp_obj_is_true(args[1])) {
@@ -228,13 +231,13 @@ STATIC mp_obj_t espnow_active(size_t n_args, const mp_obj_t *args) {
     }
     return self->recv_buffer != NULL ? mp_const_true : mp_const_false;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_active_obj, 1, 2, espnow_active);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_active_obj, 1, 2, espnow_active);
 
 // ESPNow.config(['param'|param=value, ..])
 // Get or set configuration values. Supported config params:
 //    buffer: size of buffer for rx packets (default=514 bytes)
 //    timeout: Default read timeout (default=300,000 milliseconds)
-STATIC mp_obj_t espnow_config(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t espnow_config(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     esp_espnow_obj_t *self = _get_singleton();
     enum { ARG_get, ARG_rxbuf, ARG_timeout_ms, ARG_rate };
     static const mp_arg_t allowed_args[] = {
@@ -275,11 +278,11 @@ STATIC mp_obj_t espnow_config(size_t n_args, const mp_obj_t *pos_args, mp_map_t
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(espnow_config_obj, 1, espnow_config);
+static MP_DEFINE_CONST_FUN_OBJ_KW(espnow_config_obj, 1, espnow_config);
 
 // ESPNow.irq(recv_cb)
 // Set callback function to be invoked when a message is received.
-STATIC mp_obj_t espnow_irq(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t espnow_irq(size_t n_args, const mp_obj_t *args) {
     esp_espnow_obj_t *self = _get_singleton();
     mp_obj_t recv_cb = args[1];
     if (recv_cb != mp_const_none && !mp_obj_is_callable(recv_cb)) {
@@ -289,12 +292,12 @@ STATIC mp_obj_t espnow_irq(size_t n_args, const mp_obj_t *args) {
     self->recv_cb_arg = (n_args > 2) ? args[2] : mp_const_none;
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_irq_obj, 2, 3, espnow_irq);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_irq_obj, 2, 3, espnow_irq);
 
 // ESPnow.stats(): Provide some useful stats.
 // Returns a tuple of:
 //   (tx_pkts, tx_responses, tx_failures, rx_pkts, dropped_rx_pkts)
-STATIC mp_obj_t espnow_stats(mp_obj_t _) {
+static mp_obj_t espnow_stats(mp_obj_t _) {
     const esp_espnow_obj_t *self = _get_singleton();
     return NEW_TUPLE(
         mp_obj_new_int(self->tx_packets),
@@ -303,9 +306,9 @@ STATIC mp_obj_t espnow_stats(mp_obj_t _) {
         mp_obj_new_int(self->rx_packets),
         mp_obj_new_int(self->dropped_rx_pkts));
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(espnow_stats_obj, espnow_stats);
+static MP_DEFINE_CONST_FUN_OBJ_1(espnow_stats_obj, espnow_stats);
 
-#if MICROPY_ESPNOW_RSSI
+#if MICROPY_PY_ESPNOW_RSSI
 // ### Maintaining the peer table and reading RSSI values
 //
 // We maintain a peers table for several reasons, to:
@@ -345,7 +348,7 @@ static mp_map_elem_t *_update_rssi(const uint8_t *peer, int8_t rssi, uint32_t ti
     list->items[1] = mp_obj_new_int(time_ms);
     return item;
 }
-#endif // MICROPY_ESPNOW_RSSI
+#endif // MICROPY_PY_ESPNOW_RSSI
 
 // Return C pointer to byte memory string/bytes/bytearray in obj.
 // Raise ValueError if the length does not match expected len.
@@ -398,7 +401,7 @@ static int ringbuf_get_bytes_wait(ringbuf_t *r, uint8_t *data, size_t len, mp_in
 // loaded into the 3rd and 4th elements.
 // Default timeout is set with ESPNow.config(timeout=milliseconds).
 // Return (None, None) on timeout.
-STATIC mp_obj_t espnow_recvinto(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t espnow_recvinto(size_t n_args, const mp_obj_t *args) {
     esp_espnow_obj_t *self = _get_singleton_initialised();
 
     mp_int_t timeout_ms = ((n_args > 2 && args[2] != mp_const_none)
@@ -413,11 +416,11 @@ STATIC mp_obj_t espnow_recvinto(size_t n_args, const mp_obj_t *args) {
         msg->len += msg->free;   // Make all the space in msg array available
         msg->free = 0;
     }
-    #if MICROPY_ESPNOW_RSSI
+    #if MICROPY_PY_ESPNOW_RSSI
     uint8_t peer_buf[ESP_NOW_ETH_ALEN];
     #else
     uint8_t *peer_buf = _get_bytes_len_w(list->items[0], ESP_NOW_ETH_ALEN);
-    #endif // MICROPY_ESPNOW_RSSI
+    #endif // MICROPY_PY_ESPNOW_RSSI
     uint8_t *msg_buf = _get_bytes_len_w(msg, ESP_NOW_MAX_DATA_LEN);
 
     // Read the packet header from the incoming buffer
@@ -441,7 +444,7 @@ STATIC mp_obj_t espnow_recvinto(size_t n_args, const mp_obj_t *args) {
         msg->free = size - msg_len;
     }
 
-    #if MICROPY_ESPNOW_RSSI
+    #if MICROPY_PY_ESPNOW_RSSI
     // Update rssi value in the peer device table
     mp_map_elem_t *entry = _update_rssi(peer_buf, hdr.rssi, hdr.time_ms);
     list->items[0] = entry->key;  // Set first element of list to peer
@@ -449,19 +452,19 @@ STATIC mp_obj_t espnow_recvinto(size_t n_args, const mp_obj_t *args) {
         list->items[2] = MP_OBJ_NEW_SMALL_INT(hdr.rssi);
         list->items[3] = mp_obj_new_int(hdr.time_ms);
     }
-    #endif // MICROPY_ESPNOW_RSSI
+    #endif // MICROPY_PY_ESPNOW_RSSI
 
     return MP_OBJ_NEW_SMALL_INT(msg_len);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_recvinto_obj, 2, 3, espnow_recvinto);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_recvinto_obj, 2, 3, espnow_recvinto);
 
 // Test if data is available to read from the buffers
-STATIC mp_obj_t espnow_any(const mp_obj_t _) {
+static mp_obj_t espnow_any(const mp_obj_t _) {
     esp_espnow_obj_t *self = _get_singleton_initialised();
 
     return ringbuf_avail(self->recv_buffer) ? mp_const_true : mp_const_false;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(espnow_any_obj, espnow_any);
+static MP_DEFINE_CONST_FUN_OBJ_1(espnow_any_obj, espnow_any);
 
 // Used by espnow_send() for sends() with sync==True.
 // Wait till all pending sent packet responses have been received.
@@ -491,7 +494,7 @@ static void _wait_for_pending_responses(esp_espnow_obj_t *self) {
 //   True  if sync==True and message is received successfully by all recipients
 //   False if sync==True and message is not received by at least one recipient
 // Raises: EAGAIN if the internal espnow buffers are full.
-STATIC mp_obj_t espnow_send(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t espnow_send(size_t n_args, const mp_obj_t *args) {
     esp_espnow_obj_t *self = _get_singleton_initialised();
     // Check the various combinations of input arguments
     const uint8_t *peer = (n_args > 2) ? _get_peer(args[1]) : NULL;
@@ -529,7 +532,7 @@ STATIC mp_obj_t espnow_send(size_t n_args, const mp_obj_t *args) {
     // Return False if sync and any peers did not respond.
     return mp_obj_new_bool(!(sync && self->tx_failures != saved_failures));
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_send_obj, 2, 4, espnow_send);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_send_obj, 2, 4, espnow_send);
 
 // ### The ESP_Now send and recv callback routines
 //
@@ -537,7 +540,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(espnow_send_obj, 2, 4, espnow_send);
 // Callback triggered when a sent packet is acknowledged by the peer (or not).
 // Just count the number of responses and number of failures.
 // These are used in the send() logic.
-STATIC void send_cb(const uint8_t *mac_addr, esp_now_send_status_t status) {
+static void send_cb(const uint8_t *mac_addr, esp_now_send_status_t status) {
     esp_espnow_obj_t *self = _get_singleton();
     self->tx_responses++;
     if (status != ESP_NOW_SEND_SUCCESS) {
@@ -550,7 +553,7 @@ STATIC void send_cb(const uint8_t *mac_addr, esp_now_send_status_t status) {
 // ESPNow packet.
 // If the buffer is full, drop the message and increment the dropped count.
 // Schedules the user callback if one has been registered (ESPNow.config()).
-STATIC void recv_cb(const esp_now_recv_info_t *recv_info, const uint8_t *msg, int msg_len) {
+static void recv_cb(const esp_now_recv_info_t *recv_info, const uint8_t *msg, int msg_len) {
     esp_espnow_obj_t *self = _get_singleton();
     ringbuf_t *buf = self->recv_buffer;
     // TODO: Test this works with ">".
@@ -561,10 +564,10 @@ STATIC void recv_cb(const esp_now_recv_info_t *recv_info, const uint8_t *msg, in
     espnow_hdr_t header;
     header.magic = ESPNOW_MAGIC;
     header.msg_len = msg_len;
-    #if MICROPY_ESPNOW_RSSI
+    #if MICROPY_PY_ESPNOW_RSSI
     header.rssi = recv_info->rx_ctrl->rssi;
     header.time_ms = mp_hal_ticks_ms();
-    #endif // MICROPY_ESPNOW_RSSI
+    #endif // MICROPY_PY_ESPNOW_RSSI
 
     ringbuf_put_bytes(buf, (uint8_t *)&header, sizeof(header));
     ringbuf_put_bytes(buf, recv_info->src_addr, ESP_NOW_ETH_ALEN);
@@ -581,17 +584,17 @@ STATIC void recv_cb(const esp_now_recv_info_t *recv_info, const uint8_t *msg, in
 // Set the ESP-NOW Primary Master Key (pmk) (for encrypted communications).
 // Raise OSError if ESP-NOW functions are not initialised.
 // Raise ValueError if key is not a bytes-like object exactly 16 bytes long.
-STATIC mp_obj_t espnow_set_pmk(mp_obj_t _, mp_obj_t key) {
+static mp_obj_t espnow_set_pmk(mp_obj_t _, mp_obj_t key) {
     check_esp_err(esp_now_set_pmk(_get_bytes_len(key, ESP_NOW_KEY_LEN)));
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_2(espnow_set_pmk_obj, espnow_set_pmk);
+static MP_DEFINE_CONST_FUN_OBJ_2(espnow_set_pmk_obj, espnow_set_pmk);
 
 // Common code for add_peer() and mod_peer() to process the args and kw_args:
 // Raise ValueError if the LMK is not a bytes-like object of exactly 16 bytes.
 // Raise TypeError if invalid keyword args or too many positional args.
 // Return true if all args parsed correctly.
-STATIC bool _update_peer_info(
+static bool _update_peer_info(
     esp_now_peer_info_t *peer, size_t n_args,
     const mp_obj_t *pos_args, mp_map_t *kw_args) {
 
@@ -627,7 +630,7 @@ STATIC bool _update_peer_info(
 // Update the cached peer count in self->peer_count;
 // The peer_count ignores broadcast and multicast addresses and is used for the
 // send() logic and is updated from add_peer(), mod_peer() and del_peer().
-STATIC void _update_peer_count() {
+static void _update_peer_count() {
     esp_espnow_obj_t *self = _get_singleton_initialised();
 
     esp_now_peer_info_t peer = {0};
@@ -651,7 +654,7 @@ STATIC void _update_peer_count() {
 // Raise ValueError if mac or LMK are not bytes-like objects or wrong length.
 // Raise TypeError if invalid keyword args or too many positional args.
 // Return None.
-STATIC mp_obj_t espnow_add_peer(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
+static mp_obj_t espnow_add_peer(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
     esp_now_peer_info_t peer = {0};
     memcpy(peer.peer_addr, _get_peer(args[1]), ESP_NOW_ETH_ALEN);
     _update_peer_info(&peer, n_args - 2, args + 2, kw_args);
@@ -661,13 +664,13 @@ STATIC mp_obj_t espnow_add_peer(size_t n_args, const mp_obj_t *args, mp_map_t *k
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(espnow_add_peer_obj, 2, espnow_add_peer);
+static MP_DEFINE_CONST_FUN_OBJ_KW(espnow_add_peer_obj, 2, espnow_add_peer);
 
 // ESPNow.del_peer(peer_mac): Unregister peer_mac.
 // Raise OSError if ESPNow.init() has not been called.
 // Raise ValueError if peer is not a bytes-like objects or wrong length.
 // Return None.
-STATIC mp_obj_t espnow_del_peer(mp_obj_t _, mp_obj_t peer) {
+static mp_obj_t espnow_del_peer(mp_obj_t _, mp_obj_t peer) {
     uint8_t peer_addr[ESP_NOW_ETH_ALEN];
     memcpy(peer_addr, _get_peer(peer), ESP_NOW_ETH_ALEN);
 
@@ -676,7 +679,7 @@ STATIC mp_obj_t espnow_del_peer(mp_obj_t _, mp_obj_t peer) {
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_2(espnow_del_peer_obj, espnow_del_peer);
+static MP_DEFINE_CONST_FUN_OBJ_2(espnow_del_peer_obj, espnow_del_peer);
 
 // Convert a peer_info struct to python tuple
 // Used by espnow_get_peer() and espnow_get_peers()
@@ -694,7 +697,7 @@ static mp_obj_t _peer_info_to_tuple(const esp_now_peer_info_t *peer) {
 // Return a tuple of tuples:
 //     ((peer_addr, lmk, channel, ifidx, encrypt),
 //      (peer_addr, lmk, channel, ifidx, encrypt), ...)
-STATIC mp_obj_t espnow_get_peers(mp_obj_t _) {
+static mp_obj_t espnow_get_peers(mp_obj_t _) {
     esp_espnow_obj_t *self = _get_singleton_initialised();
 
     // Build and initialise the peer info tuple.
@@ -708,14 +711,14 @@ STATIC mp_obj_t espnow_get_peers(mp_obj_t _) {
 
     return peerinfo_tuple;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(espnow_get_peers_obj, espnow_get_peers);
+static MP_DEFINE_CONST_FUN_OBJ_1(espnow_get_peers_obj, espnow_get_peers);
 
-#if MICROPY_ESPNOW_EXTRA_PEER_METHODS
+#if MICROPY_PY_ESPNOW_EXTRA_PEER_METHODS
 // ESPNow.get_peer(peer_mac): Get the peer info for peer_mac as a tuple.
 // Raise OSError if ESPNow.init() has not been called.
 // Raise ValueError if mac or LMK are not bytes-like objects or wrong length.
 // Return a tuple of (peer_addr, lmk, channel, ifidx, encrypt).
-STATIC mp_obj_t espnow_get_peer(mp_obj_t _, mp_obj_t arg1) {
+static mp_obj_t espnow_get_peer(mp_obj_t _, mp_obj_t arg1) {
     esp_now_peer_info_t peer = {0};
     memcpy(peer.peer_addr, _get_peer(arg1), ESP_NOW_ETH_ALEN);
 
@@ -723,7 +726,7 @@ STATIC mp_obj_t espnow_get_peer(mp_obj_t _, mp_obj_t arg1) {
 
     return _peer_info_to_tuple(&peer);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_2(espnow_get_peer_obj, espnow_get_peer);
+static MP_DEFINE_CONST_FUN_OBJ_2(espnow_get_peer_obj, espnow_get_peer);
 
 // ESPNow.mod_peer(peer_mac, [lmk, [channel, [ifidx, [encrypt]]]]) or
 // ESPNow.mod_peer(peer_mac, [lmk=b'0123456789abcdef'|b''|None|False],
@@ -733,7 +736,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_2(espnow_get_peer_obj, espnow_get_peer);
 // Raise ValueError if mac or LMK are not bytes-like objects or wrong length.
 // Raise TypeError if invalid keyword args or too many positional args.
 // Return None.
-STATIC mp_obj_t espnow_mod_peer(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
+static mp_obj_t espnow_mod_peer(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
     esp_now_peer_info_t peer = {0};
     memcpy(peer.peer_addr, _get_peer(args[1]), ESP_NOW_ETH_ALEN);
     check_esp_err(esp_now_get_peer(peer.peer_addr, &peer));
@@ -745,12 +748,12 @@ STATIC mp_obj_t espnow_mod_peer(size_t n_args, const mp_obj_t *args, mp_map_t *k
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(espnow_mod_peer_obj, 2, espnow_mod_peer);
+static MP_DEFINE_CONST_FUN_OBJ_KW(espnow_mod_peer_obj, 2, espnow_mod_peer);
 
 // ESPNow.espnow_peer_count(): Get the number of registered peers.
 // Raise OSError if ESPNow.init() has not been called.
 // Return a tuple of (num_total_peers, num_encrypted_peers).
-STATIC mp_obj_t espnow_peer_count(mp_obj_t _) {
+static mp_obj_t espnow_peer_count(mp_obj_t _) {
     esp_now_peer_num_t peer_num = {0};
     check_esp_err(esp_now_get_peer_num(&peer_num));
 
@@ -758,10 +761,10 @@ STATIC mp_obj_t espnow_peer_count(mp_obj_t _) {
         mp_obj_new_int(peer_num.total_num),
         mp_obj_new_int(peer_num.encrypt_num));
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(espnow_peer_count_obj, espnow_peer_count);
+static MP_DEFINE_CONST_FUN_OBJ_1(espnow_peer_count_obj, espnow_peer_count);
 #endif
 
-STATIC const mp_rom_map_elem_t esp_espnow_locals_dict_table[] = {
+static const mp_rom_map_elem_t esp_espnow_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_active), MP_ROM_PTR(&espnow_active_obj) },
     { MP_ROM_QSTR(MP_QSTR_config), MP_ROM_PTR(&espnow_config_obj) },
     { MP_ROM_QSTR(MP_QSTR_irq), MP_ROM_PTR(&espnow_irq_obj) },
@@ -777,15 +780,15 @@ STATIC const mp_rom_map_elem_t esp_espnow_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_add_peer), MP_ROM_PTR(&espnow_add_peer_obj) },
     { MP_ROM_QSTR(MP_QSTR_del_peer), MP_ROM_PTR(&espnow_del_peer_obj) },
     { MP_ROM_QSTR(MP_QSTR_get_peers), MP_ROM_PTR(&espnow_get_peers_obj) },
-    #if MICROPY_ESPNOW_EXTRA_PEER_METHODS
+    #if MICROPY_PY_ESPNOW_EXTRA_PEER_METHODS
     { MP_ROM_QSTR(MP_QSTR_mod_peer), MP_ROM_PTR(&espnow_mod_peer_obj) },
     { MP_ROM_QSTR(MP_QSTR_get_peer), MP_ROM_PTR(&espnow_get_peer_obj) },
     { MP_ROM_QSTR(MP_QSTR_peer_count), MP_ROM_PTR(&espnow_peer_count_obj) },
-    #endif // MICROPY_ESPNOW_EXTRA_PEER_METHODS
+    #endif // MICROPY_PY_ESPNOW_EXTRA_PEER_METHODS
 };
-STATIC MP_DEFINE_CONST_DICT(esp_espnow_locals_dict, esp_espnow_locals_dict_table);
+static MP_DEFINE_CONST_DICT(esp_espnow_locals_dict, esp_espnow_locals_dict_table);
 
-STATIC const mp_rom_map_elem_t espnow_globals_dict_table[] = {
+static const mp_rom_map_elem_t espnow_globals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR___name__), MP_ROM_QSTR(MP_QSTR__espnow) },
     { MP_ROM_QSTR(MP_QSTR_ESPNowBase), MP_ROM_PTR(&esp_espnow_type) },
     { MP_ROM_QSTR(MP_QSTR_MAX_DATA_LEN), MP_ROM_INT(ESP_NOW_MAX_DATA_LEN)},
@@ -794,13 +797,13 @@ STATIC const mp_rom_map_elem_t espnow_globals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_MAX_TOTAL_PEER_NUM), MP_ROM_INT(ESP_NOW_MAX_TOTAL_PEER_NUM)},
     { MP_ROM_QSTR(MP_QSTR_MAX_ENCRYPT_PEER_NUM), MP_ROM_INT(ESP_NOW_MAX_ENCRYPT_PEER_NUM)},
 };
-STATIC MP_DEFINE_CONST_DICT(espnow_globals_dict, espnow_globals_dict_table);
+static MP_DEFINE_CONST_DICT(espnow_globals_dict, espnow_globals_dict_table);
 
 // ### Dummy Buffer Protocol support
 // ...so asyncio can poll.ipoll() on this device
 
 // Support ioctl(MP_STREAM_POLL, ) for asyncio
-STATIC mp_uint_t espnow_stream_ioctl(
+static mp_uint_t espnow_stream_ioctl(
     mp_obj_t self_in, mp_uint_t request, uintptr_t arg, int *errcode) {
     if (request != MP_STREAM_POLL) {
         *errcode = MP_EINVAL;
@@ -815,11 +818,11 @@ STATIC mp_uint_t espnow_stream_ioctl(
                ((self->tx_responses < self->tx_packets) ? MP_STREAM_POLL_WR : 0));
 }
 
-STATIC const mp_stream_p_t espnow_stream_p = {
+static const mp_stream_p_t espnow_stream_p = {
     .ioctl = espnow_stream_ioctl,
 };
 
-#if MICROPY_ESPNOW_RSSI
+#if MICROPY_PY_ESPNOW_RSSI
 // Return reference to the dictionary of peers we have seen:
 //   {peer1: (rssi, time_sec), peer2: (rssi, time_msec), ...}
 // where:
@@ -827,7 +830,7 @@ STATIC const mp_stream_p_t espnow_stream_p = {
 //   rssi is the wifi signal strength from the last msg received
 //       (in dBm from -127 to 0)
 //   time_sec is the time in milliseconds since device last booted.
-STATIC void espnow_attr(mp_obj_t self_in, qstr attr, mp_obj_t *dest) {
+static void espnow_attr(mp_obj_t self_in, qstr attr, mp_obj_t *dest) {
     esp_espnow_obj_t *self = _get_singleton();
     if (dest[0] != MP_OBJ_NULL) {   // Only allow "Load" operation
         return;
@@ -838,16 +841,16 @@ STATIC void espnow_attr(mp_obj_t self_in, qstr attr, mp_obj_t *dest) {
     }
     dest[1] = MP_OBJ_SENTINEL;  // Attribute not found
 }
-#endif // MICROPY_ESPNOW_RSSI
+#endif // MICROPY_PY_ESPNOW_RSSI
 
 MP_DEFINE_CONST_OBJ_TYPE(
     esp_espnow_type,
     MP_QSTR_ESPNowBase,
     MP_TYPE_FLAG_NONE,
     make_new, espnow_make_new,
-    #if MICROPY_ESPNOW_RSSI
+    #if MICROPY_PY_ESPNOW_RSSI
     attr, espnow_attr,
-    #endif // MICROPY_ESPNOW_RSSI
+    #endif // MICROPY_PY_ESPNOW_RSSI
     protocol, &espnow_stream_p,
     locals_dict, &esp_espnow_locals_dict
     );
@@ -859,3 +862,5 @@ const mp_obj_module_t mp_module_espnow = {
 
 MP_REGISTER_MODULE(MP_QSTR__espnow, mp_module_espnow);
 MP_REGISTER_ROOT_POINTER(struct _esp_espnow_obj_t *espnow_singleton);
+
+#endif // MICROPY_PY_ESPNOW
