@@ -30,6 +30,7 @@
 #include "py/mpthread.h"
 #include "pico/stdlib.h"
 #include "pico/multicore.h"
+#include "mutex_extra.h"
 
 #if MICROPY_PY_THREAD
 
@@ -38,34 +39,30 @@ extern uint8_t __StackTop, __StackBottom;
 void *core_state[2];
 
 // This will be non-NULL while Python code is executing.
-STATIC void *(*core1_entry)(void *) = NULL;
+static void *(*core1_entry)(void *) = NULL;
 
-STATIC void *core1_arg = NULL;
-STATIC uint32_t *core1_stack = NULL;
-STATIC size_t core1_stack_num_words = 0;
+static void *core1_arg = NULL;
+static uint32_t *core1_stack = NULL;
+static size_t core1_stack_num_words = 0;
 
 // Thread mutex.
-STATIC mp_thread_mutex_t atomic_mutex;
+static mutex_t atomic_mutex;
 
 uint32_t mp_thread_begin_atomic_section(void) {
     if (core1_entry) {
         // When both cores are executing, we also need to provide
         // full mutual exclusion.
-        mp_thread_mutex_lock(&atomic_mutex, 1);
-        // In case this atomic section is for flash access, then
-        // suspend the other core.
-        multicore_lockout_start_blocking();
+        return mutex_enter_blocking_and_disable_interrupts(&atomic_mutex);
+    } else {
+        return save_and_disable_interrupts();
     }
-
-    return save_and_disable_interrupts();
 }
 
 void mp_thread_end_atomic_section(uint32_t state) {
-    restore_interrupts(state);
-
-    if (core1_entry) {
-        multicore_lockout_end_blocking();
-        mp_thread_mutex_unlock(&atomic_mutex);
+    if (atomic_mutex.owner != LOCK_INVALID_OWNER_ID) {
+        mutex_exit_and_restore_interrupts(&atomic_mutex, state);
+    } else {
+        restore_interrupts(state);
     }
 }
 
@@ -73,7 +70,7 @@ void mp_thread_end_atomic_section(uint32_t state) {
 void mp_thread_init(void) {
     assert(get_core_num() == 0);
 
-    mp_thread_mutex_init(&atomic_mutex);
+    mutex_init(&atomic_mutex);
 
     // Allow MICROPY_BEGIN_ATOMIC_SECTION to be invoked from core1.
     multicore_lockout_victim_init();
@@ -105,7 +102,7 @@ void mp_thread_gc_others(void) {
     }
 }
 
-STATIC void core1_entry_wrapper(void) {
+static void core1_entry_wrapper(void) {
     // Allow MICROPY_BEGIN_ATOMIC_SECTION to be invoked from core0.
     multicore_lockout_victim_init();
 
@@ -118,9 +115,9 @@ STATIC void core1_entry_wrapper(void) {
 }
 
 mp_uint_t mp_thread_get_id(void) {
-    // On RP2, there are only two threads, one for each core, so the thread id
-    // is the core number.
-    return get_core_num();
+    // On RP2, there are only two threads, one for each core.
+    // _thread.get_ident() must be non-zero.
+    return get_core_num() + 1;
 }
 
 mp_uint_t mp_thread_create(void *(*entry)(void *), void *arg, size_t *stack_size) {
@@ -152,7 +149,7 @@ mp_uint_t mp_thread_create(void *(*entry)(void *), void *arg, size_t *stack_size
     // Adjust stack_size to provide room to recover from hitting the limit.
     *stack_size -= 512;
 
-    return 1;
+    return 2; // mp_thread_get_id() result for core 1
 }
 
 void mp_thread_start(void) {
