@@ -56,7 +56,7 @@ typedef struct _rp2_dma_ctrl_field_t {
     uint8_t read_only : 1;
 } rp2_dma_ctrl_field_t;
 
-STATIC rp2_dma_ctrl_field_t rp2_dma_ctrl_fields_table[] = {
+static rp2_dma_ctrl_field_t rp2_dma_ctrl_fields_table[] = {
     { MP_QSTR_enable,        0, 1, 0 },
     { MP_QSTR_high_pri,      1, 1, 0 },
     { MP_QSTR_size,          2, 2, 0 },
@@ -76,14 +76,14 @@ STATIC rp2_dma_ctrl_field_t rp2_dma_ctrl_fields_table[] = {
     { MP_QSTR_ahb_err,      31, 1, 1 },
 };
 
-STATIC const uint32_t rp2_dma_ctrl_field_count = MP_ARRAY_SIZE(rp2_dma_ctrl_fields_table);
+static const uint32_t rp2_dma_ctrl_field_count = MP_ARRAY_SIZE(rp2_dma_ctrl_fields_table);
 
 #define REG_TYPE_COUNT       0  // Accept just integers
 #define REG_TYPE_CONF        1  // Accept integers or ctrl values
 #define REG_TYPE_ADDR_READ   2  // Accept integers, buffers or objects that can be read from
 #define REG_TYPE_ADDR_WRITE  3  // Accept integers, buffers or objects that can be written to
 
-STATIC uint32_t rp2_dma_register_value_from_obj(mp_obj_t o, int reg_type) {
+static uint32_t rp2_dma_register_value_from_obj(mp_obj_t o, int reg_type) {
     if (reg_type == REG_TYPE_ADDR_READ || reg_type == REG_TYPE_ADDR_WRITE) {
         mp_buffer_info_t buf_info;
         mp_uint_t flags = (reg_type == REG_TYPE_ADDR_READ) ? MP_BUFFER_READ : MP_BUFFER_WRITE;
@@ -95,7 +95,7 @@ STATIC uint32_t rp2_dma_register_value_from_obj(mp_obj_t o, int reg_type) {
     return mp_obj_get_int_truncated(o);
 }
 
-STATIC void rp2_dma_irq_handler(void) {
+static void rp2_dma_irq_handler(void) {
     // Main IRQ handler
     uint32_t irq_bits = dma_hw->ints0;
 
@@ -113,7 +113,7 @@ STATIC void rp2_dma_irq_handler(void) {
     }
 }
 
-STATIC mp_uint_t rp2_dma_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
+static mp_uint_t rp2_dma_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(self_in);
     irq_set_enabled(DMA_IRQ_0, false);
     self->irq_flag = 0;
@@ -122,7 +122,7 @@ STATIC mp_uint_t rp2_dma_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
     return 0;
 }
 
-STATIC mp_uint_t rp2_dma_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
+static mp_uint_t rp2_dma_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(self_in);
     if (info_type == MP_IRQ_INFO_FLAGS) {
         return self->irq_flag;
@@ -132,12 +132,12 @@ STATIC mp_uint_t rp2_dma_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
     return 0;
 }
 
-STATIC const mp_irq_methods_t rp2_dma_irq_methods = {
+static const mp_irq_methods_t rp2_dma_irq_methods = {
     .trigger = rp2_dma_irq_trigger,
     .info = rp2_dma_irq_info,
 };
 
-STATIC mp_obj_t rp2_dma_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
+static mp_obj_t rp2_dma_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
     mp_arg_check_num(n_args, n_kw, 0, 0, false);
 
     int dma_channel = dma_claim_unused_channel(false);
@@ -145,21 +145,20 @@ STATIC mp_obj_t rp2_dma_make_new(const mp_obj_type_t *type, size_t n_args, size_
         mp_raise_OSError(MP_EBUSY);
     }
 
-    rp2_dma_obj_t *self = m_new_obj_with_finaliser(rp2_dma_obj_t);
-    self->base.type = &rp2_dma_type;
+    rp2_dma_obj_t *self = mp_obj_malloc_with_finaliser(rp2_dma_obj_t, &rp2_dma_type);
     self->channel = dma_channel;
 
     // Return the DMA object.
     return MP_OBJ_FROM_PTR(self);
 }
 
-STATIC void rp2_dma_error_if_closed(rp2_dma_obj_t *self) {
+static void rp2_dma_error_if_closed(rp2_dma_obj_t *self) {
     if (self->channel == CHANNEL_CLOSED) {
         mp_raise_ValueError(MP_ERROR_TEXT("channel closed"));
     }
 }
 
-STATIC void rp2_dma_attr(mp_obj_t self_in, qstr attr_in, mp_obj_t *dest) {
+static void rp2_dma_attr(mp_obj_t self_in, qstr attr_in, mp_obj_t *dest) {
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(self_in);
 
     if (dest[0] == MP_OBJ_NULL) {
@@ -216,13 +215,13 @@ STATIC void rp2_dma_attr(mp_obj_t self_in, qstr attr_in, mp_obj_t *dest) {
     }
 }
 
-STATIC void rp2_dma_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
+static void rp2_dma_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_printf(print, "DMA(%u)", self->channel);
 }
 
 // DMA.config(*, read, write, count, ctrl, trigger)
-STATIC mp_obj_t rp2_dma_config(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t rp2_dma_config(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(*pos_args);
 
     rp2_dma_error_if_closed(self);
@@ -278,10 +277,10 @@ STATIC mp_obj_t rp2_dma_config(size_t n_args, const mp_obj_t *pos_args, mp_map_t
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(rp2_dma_config_obj, 1, rp2_dma_config);
+static MP_DEFINE_CONST_FUN_OBJ_KW(rp2_dma_config_obj, 1, rp2_dma_config);
 
 // DMA.active([value])
-STATIC mp_obj_t rp2_dma_active(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t rp2_dma_active(size_t n_args, const mp_obj_t *args) {
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(args[0]);
     rp2_dma_error_if_closed(self);
 
@@ -296,13 +295,13 @@ STATIC mp_obj_t rp2_dma_active(size_t n_args, const mp_obj_t *args) {
     uint32_t busy = dma_channel_is_busy(self->channel);
     return mp_obj_new_bool((mp_int_t)busy);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_dma_active_obj, 1, 2, rp2_dma_active);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_dma_active_obj, 1, 2, rp2_dma_active);
 
 // Default is quiet, unpaced, read and write incrementing, word transfers, enabled
 #define DEFAULT_DMA_CONFIG (1 << 21) | (0x3f << 15) | (1 << 5) | (1 << 4) | (2 << 2) | (1 << 0)
 
 // DMA.pack_ctrl(...)
-STATIC mp_obj_t rp2_dma_pack_ctrl(size_t n_pos_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t rp2_dma_pack_ctrl(size_t n_pos_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     // Pack keyword settings into a control register value, using either the default for this
     // DMA channel or the provided defaults
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(pos_args[0]);
@@ -347,10 +346,10 @@ STATIC mp_obj_t rp2_dma_pack_ctrl(size_t n_pos_args, const mp_obj_t *pos_args, m
 
     return mp_obj_new_int_from_uint(value);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(rp2_dma_pack_ctrl_obj, 1, rp2_dma_pack_ctrl);
+static MP_DEFINE_CONST_FUN_OBJ_KW(rp2_dma_pack_ctrl_obj, 1, rp2_dma_pack_ctrl);
 
 // DMA.unpack_ctrl(value)
-STATIC mp_obj_t rp2_dma_unpack_ctrl(mp_obj_t value_obj) {
+static mp_obj_t rp2_dma_unpack_ctrl(mp_obj_t value_obj) {
     // Return a dict representing the unpacked fields of a control register value
     mp_obj_t result_dict[rp2_dma_ctrl_field_count * 2];
 
@@ -365,10 +364,10 @@ STATIC mp_obj_t rp2_dma_unpack_ctrl(mp_obj_t value_obj) {
 
     return mp_obj_dict_make_new(&mp_type_dict, 0, rp2_dma_ctrl_field_count, result_dict);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(rp2_dma_unpack_ctrl_fun_obj, rp2_dma_unpack_ctrl);
-STATIC MP_DEFINE_CONST_STATICMETHOD_OBJ(rp2_dma_unpack_ctrl_obj, MP_ROM_PTR(&rp2_dma_unpack_ctrl_fun_obj));
+static MP_DEFINE_CONST_FUN_OBJ_1(rp2_dma_unpack_ctrl_fun_obj, rp2_dma_unpack_ctrl);
+static MP_DEFINE_CONST_STATICMETHOD_OBJ(rp2_dma_unpack_ctrl_obj, MP_ROM_PTR(&rp2_dma_unpack_ctrl_fun_obj));
 
-STATIC mp_obj_t rp2_dma_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t rp2_dma_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     enum { ARG_handler, ARG_hard };
     static const mp_arg_t allowed_args[] = {
         { MP_QSTR_handler, MP_ARG_OBJ, {.u_rom_obj = mp_const_none} },
@@ -410,10 +409,10 @@ STATIC mp_obj_t rp2_dma_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *k
 
     return MP_OBJ_FROM_PTR(irq);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(rp2_dma_irq_obj, 1, rp2_dma_irq);
+static MP_DEFINE_CONST_FUN_OBJ_KW(rp2_dma_irq_obj, 1, rp2_dma_irq);
 
 // DMA.close()
-STATIC mp_obj_t rp2_dma_close(mp_obj_t self_in) {
+static mp_obj_t rp2_dma_close(mp_obj_t self_in) {
     rp2_dma_obj_t *self = MP_OBJ_TO_PTR(self_in);
     uint8_t channel = self->channel;
 
@@ -432,9 +431,9 @@ STATIC mp_obj_t rp2_dma_close(mp_obj_t self_in) {
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(rp2_dma_close_obj, rp2_dma_close);
+static MP_DEFINE_CONST_FUN_OBJ_1(rp2_dma_close_obj, rp2_dma_close);
 
-STATIC const mp_rom_map_elem_t rp2_dma_locals_dict_table[] = {
+static const mp_rom_map_elem_t rp2_dma_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_config), MP_ROM_PTR(&rp2_dma_config_obj) },
     { MP_ROM_QSTR(MP_QSTR_active), MP_ROM_PTR(&rp2_dma_active_obj) },
     { MP_ROM_QSTR(MP_QSTR_irq), MP_ROM_PTR(&rp2_dma_irq_obj) },
@@ -443,7 +442,7 @@ STATIC const mp_rom_map_elem_t rp2_dma_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_pack_ctrl), MP_ROM_PTR(&rp2_dma_pack_ctrl_obj) },
     { MP_ROM_QSTR(MP_QSTR_unpack_ctrl), MP_ROM_PTR(&rp2_dma_unpack_ctrl_obj) },
 };
-STATIC MP_DEFINE_CONST_DICT(rp2_dma_locals_dict, rp2_dma_locals_dict_table);
+static MP_DEFINE_CONST_DICT(rp2_dma_locals_dict, rp2_dma_locals_dict_table);
 
 MP_DEFINE_CONST_OBJ_TYPE(
     rp2_dma_type,
