@@ -65,17 +65,17 @@ typedef struct _rp2_state_machine_irq_obj_t {
     uint8_t trigger;
 } rp2_state_machine_irq_obj_t;
 
-STATIC const rp2_state_machine_obj_t rp2_state_machine_obj[8];
-STATIC uint8_t rp2_state_machine_initial_pc[8];
+static const rp2_state_machine_obj_t rp2_state_machine_obj[8];
+static uint8_t rp2_state_machine_initial_pc[8];
 
 // These masks keep track of PIO instruction memory used by this module.
-STATIC uint32_t rp2_pio_instruction_memory_usage_mask[2];
+static uint32_t rp2_pio_instruction_memory_usage_mask[2];
 
-STATIC const rp2_state_machine_obj_t *rp2_state_machine_get_object(mp_int_t sm_id);
-STATIC void rp2_state_machine_reset_all(void);
-STATIC mp_obj_t rp2_state_machine_init_helper(const rp2_state_machine_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args);
+static const rp2_state_machine_obj_t *rp2_state_machine_get_object(mp_int_t sm_id);
+static void rp2_state_machine_reset_all(void);
+static mp_obj_t rp2_state_machine_init_helper(const rp2_state_machine_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args);
 
-STATIC void pio_irq0(PIO pio) {
+static void pio_irq0(PIO pio) {
     uint32_t ints = pio->ints0;
 
     // Acknowledge SM0-3 IRQs if they are enabled on this IRQ0.
@@ -98,16 +98,16 @@ STATIC void pio_irq0(PIO pio) {
     }
 }
 
-STATIC void pio0_irq0(void) {
+static void pio0_irq0(void) {
     pio_irq0(pio0);
 }
 
-STATIC void pio1_irq0(void) {
+static void pio1_irq0(void) {
     pio_irq0(pio1);
 }
 
 // Calls pio_add_program() and keeps track of used instruction memory.
-STATIC uint rp2_pio_add_managed_program(PIO pio, struct pio_program *pio_program) {
+static uint rp2_pio_add_managed_program(PIO pio, struct pio_program *pio_program) {
     uint offset = pio_add_program(pio, pio_program);
     uint32_t mask = ((1 << pio_program->length) - 1) << offset;
     rp2_pio_instruction_memory_usage_mask[pio_get_index(pio)] |= mask;
@@ -115,7 +115,7 @@ STATIC uint rp2_pio_add_managed_program(PIO pio, struct pio_program *pio_program
 }
 
 // Calls pio_remove_program() and keeps track of used instruction memory.
-STATIC void rp2_pio_remove_managed_program(PIO pio, struct pio_program *pio_program, uint offset) {
+static void rp2_pio_remove_managed_program(PIO pio, struct pio_program *pio_program, uint offset) {
     pio_remove_program(pio, pio_program, offset);
     uint32_t mask = ((1 << pio_program->length) - 1) << offset;
     rp2_pio_instruction_memory_usage_mask[pio_get_index(pio)] &= ~mask;
@@ -123,7 +123,7 @@ STATIC void rp2_pio_remove_managed_program(PIO pio, struct pio_program *pio_prog
 
 // Calls pio_remove_program() for all programs registered with rp2_pio_add_managed_program(),
 // that weren't already removed via rp2_pio_remove_managed_program().
-STATIC void rp2_pio_remove_all_managed_programs(PIO pio) {
+static void rp2_pio_remove_all_managed_programs(PIO pio) {
     uint32_t mask = rp2_pio_instruction_memory_usage_mask[pio_get_index(pio)];
     for (size_t i = 0; i < 32; ++i) {
         if (mask & (1 << i)) {
@@ -188,13 +188,13 @@ typedef struct _asm_pio_config_t {
     uint32_t pinvals;
 } asm_pio_config_t;
 
-STATIC void asm_pio_override_shiftctrl(mp_obj_t arg, uint32_t bits, uint32_t lsb, pio_sm_config *config) {
+static void asm_pio_override_shiftctrl(mp_obj_t arg, uint32_t bits, uint32_t lsb, pio_sm_config *config) {
     if (arg != mp_const_none) {
         config->shiftctrl = (config->shiftctrl & ~bits) | (mp_obj_get_int(arg) << lsb);
     }
 }
 
-STATIC void asm_pio_get_pins(const char *type, mp_obj_t prog_pins, mp_obj_t arg_base, asm_pio_config_t *config) {
+static void asm_pio_get_pins(const char *type, mp_obj_t prog_pins, mp_obj_t arg_base, asm_pio_config_t *config) {
     if (prog_pins != mp_const_none) {
         // The PIO program specified pins for initialisation on out/set/sideset.
         if (mp_obj_is_integer(prog_pins)) {
@@ -223,7 +223,7 @@ STATIC void asm_pio_get_pins(const char *type, mp_obj_t prog_pins, mp_obj_t arg_
     }
 }
 
-STATIC void asm_pio_init_gpio(PIO pio, uint32_t sm, asm_pio_config_t *config) {
+static void asm_pio_init_gpio(PIO pio, uint32_t sm, asm_pio_config_t *config) {
     uint32_t pinmask = ((1 << config->count) - 1) << config->base;
     pio_sm_set_pins_with_mask(pio, sm, config->pinvals << config->base, pinmask);
     pio_sm_set_pindirs_with_mask(pio, sm, config->pindirs << config->base, pinmask);
@@ -235,20 +235,20 @@ STATIC void asm_pio_init_gpio(PIO pio, uint32_t sm, asm_pio_config_t *config) {
 /******************************************************************************/
 // PIO object
 
-STATIC const mp_irq_methods_t rp2_pio_irq_methods;
+static const mp_irq_methods_t rp2_pio_irq_methods;
 
-STATIC rp2_pio_obj_t rp2_pio_obj[] = {
+static rp2_pio_obj_t rp2_pio_obj[] = {
     { { &rp2_pio_type }, pio0, PIO0_IRQ_0 },
     { { &rp2_pio_type }, pio1, PIO1_IRQ_0 },
 };
 
-STATIC void rp2_pio_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
+static void rp2_pio_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     rp2_pio_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_printf(print, "PIO(%u)", self->pio == pio0 ? 0 : 1);
 }
 
 // constructor(id)
-STATIC mp_obj_t rp2_pio_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
+static mp_obj_t rp2_pio_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
     mp_arg_check_num(n_args, n_kw, 1, 1, false);
 
     // Get the PIO object.
@@ -263,7 +263,7 @@ STATIC mp_obj_t rp2_pio_make_new(const mp_obj_type_t *type, size_t n_args, size_
 }
 
 // PIO.add_program(prog)
-STATIC mp_obj_t rp2_pio_add_program(mp_obj_t self_in, mp_obj_t prog_in) {
+static mp_obj_t rp2_pio_add_program(mp_obj_t self_in, mp_obj_t prog_in) {
     rp2_pio_obj_t *self = MP_OBJ_TO_PTR(self_in);
 
     // Get the program data.
@@ -284,10 +284,10 @@ STATIC mp_obj_t rp2_pio_add_program(mp_obj_t self_in, mp_obj_t prog_in) {
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_2(rp2_pio_add_program_obj, rp2_pio_add_program);
+static MP_DEFINE_CONST_FUN_OBJ_2(rp2_pio_add_program_obj, rp2_pio_add_program);
 
 // PIO.remove_program([prog])
-STATIC mp_obj_t rp2_pio_remove_program(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t rp2_pio_remove_program(size_t n_args, const mp_obj_t *args) {
     rp2_pio_obj_t *self = MP_OBJ_TO_PTR(args[0]);
 
     // Default to remove all programs.
@@ -315,10 +315,10 @@ STATIC mp_obj_t rp2_pio_remove_program(size_t n_args, const mp_obj_t *args) {
 
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_pio_remove_program_obj, 1, 2, rp2_pio_remove_program);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_pio_remove_program_obj, 1, 2, rp2_pio_remove_program);
 
 // PIO.state_machine(id, prog, freq=-1, *, set=None)
-STATIC mp_obj_t rp2_pio_state_machine(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t rp2_pio_state_machine(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     rp2_pio_obj_t *self = MP_OBJ_TO_PTR(pos_args[0]);
 
     // Get and verify the state machine id.
@@ -340,7 +340,7 @@ STATIC mp_obj_t rp2_pio_state_machine(size_t n_args, const mp_obj_t *pos_args, m
 MP_DEFINE_CONST_FUN_OBJ_KW(rp2_pio_state_machine_obj, 2, rp2_pio_state_machine);
 
 // PIO.irq(handler=None, trigger=IRQ_SM0|IRQ_SM1|IRQ_SM2|IRQ_SM3, hard=False)
-STATIC mp_obj_t rp2_pio_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t rp2_pio_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     enum { ARG_handler, ARG_trigger, ARG_hard };
     static const mp_arg_t allowed_args[] = {
         { MP_QSTR_handler, MP_ARG_OBJ, {.u_rom_obj = MP_ROM_NONE} },
@@ -388,9 +388,9 @@ STATIC mp_obj_t rp2_pio_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *k
 
     return MP_OBJ_FROM_PTR(irq);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(rp2_pio_irq_obj, 1, rp2_pio_irq);
+static MP_DEFINE_CONST_FUN_OBJ_KW(rp2_pio_irq_obj, 1, rp2_pio_irq);
 
-STATIC const mp_rom_map_elem_t rp2_pio_locals_dict_table[] = {
+static const mp_rom_map_elem_t rp2_pio_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_add_program), MP_ROM_PTR(&rp2_pio_add_program_obj) },
     { MP_ROM_QSTR(MP_QSTR_remove_program), MP_ROM_PTR(&rp2_pio_remove_program_obj) },
     { MP_ROM_QSTR(MP_QSTR_state_machine), MP_ROM_PTR(&rp2_pio_state_machine_obj) },
@@ -413,7 +413,7 @@ STATIC const mp_rom_map_elem_t rp2_pio_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_IRQ_SM2), MP_ROM_INT(0x400) },
     { MP_ROM_QSTR(MP_QSTR_IRQ_SM3), MP_ROM_INT(0x800) },
 };
-STATIC MP_DEFINE_CONST_DICT(rp2_pio_locals_dict, rp2_pio_locals_dict_table);
+static MP_DEFINE_CONST_DICT(rp2_pio_locals_dict, rp2_pio_locals_dict_table);
 
 MP_DEFINE_CONST_OBJ_TYPE(
     rp2_pio_type,
@@ -424,7 +424,7 @@ MP_DEFINE_CONST_OBJ_TYPE(
     locals_dict, &rp2_pio_locals_dict
     );
 
-STATIC mp_uint_t rp2_pio_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
+static mp_uint_t rp2_pio_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
     rp2_pio_obj_t *self = MP_OBJ_TO_PTR(self_in);
     rp2_pio_irq_obj_t *irq = MP_STATE_PORT(rp2_pio_irq_obj[PIO_NUM(self->pio)]);
     irq_set_enabled(self->irq, false);
@@ -434,7 +434,7 @@ STATIC mp_uint_t rp2_pio_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
     return 0;
 }
 
-STATIC mp_uint_t rp2_pio_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
+static mp_uint_t rp2_pio_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
     rp2_pio_obj_t *self = MP_OBJ_TO_PTR(self_in);
     rp2_pio_irq_obj_t *irq = MP_STATE_PORT(rp2_pio_irq_obj[PIO_NUM(self->pio)]);
     if (info_type == MP_IRQ_INFO_FLAGS) {
@@ -445,7 +445,7 @@ STATIC mp_uint_t rp2_pio_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
     return 0;
 }
 
-STATIC const mp_irq_methods_t rp2_pio_irq_methods = {
+static const mp_irq_methods_t rp2_pio_irq_methods = {
     .trigger = rp2_pio_irq_trigger,
     .info = rp2_pio_irq_info,
 };
@@ -454,11 +454,11 @@ STATIC const mp_irq_methods_t rp2_pio_irq_methods = {
 // StateMachine object
 
 // This mask keeps track of state machines claimed by this module.
-STATIC uint32_t rp2_state_machine_claimed_mask;
+static uint32_t rp2_state_machine_claimed_mask;
 
-STATIC const mp_irq_methods_t rp2_state_machine_irq_methods;
+static const mp_irq_methods_t rp2_state_machine_irq_methods;
 
-STATIC const rp2_state_machine_obj_t rp2_state_machine_obj[] = {
+static const rp2_state_machine_obj_t rp2_state_machine_obj[] = {
     { { &rp2_state_machine_type }, pio0, PIO0_IRQ_0, 0, 0 },
     { { &rp2_state_machine_type }, pio0, PIO0_IRQ_0, 1, 1 },
     { { &rp2_state_machine_type }, pio0, PIO0_IRQ_0, 2, 2 },
@@ -469,7 +469,7 @@ STATIC const rp2_state_machine_obj_t rp2_state_machine_obj[] = {
     { { &rp2_state_machine_type }, pio1, PIO1_IRQ_0, 3, 7 },
 };
 
-STATIC const rp2_state_machine_obj_t *rp2_state_machine_get_object(mp_int_t sm_id) {
+static const rp2_state_machine_obj_t *rp2_state_machine_get_object(mp_int_t sm_id) {
     if (!(0 <= sm_id && sm_id < MP_ARRAY_SIZE(rp2_state_machine_obj))) {
         mp_raise_ValueError("invalid StateMachine");
     }
@@ -487,7 +487,7 @@ STATIC const rp2_state_machine_obj_t *rp2_state_machine_get_object(mp_int_t sm_i
     return sm_obj;
 }
 
-STATIC void rp2_state_machine_reset_all(void) {
+static void rp2_state_machine_reset_all(void) {
     for (size_t i = 0; i < MP_ARRAY_SIZE(rp2_state_machine_obj); ++i) {
         if (rp2_state_machine_claimed_mask & (1 << i)) {
             const rp2_state_machine_obj_t *sm_obj = &rp2_state_machine_obj[i];
@@ -498,7 +498,7 @@ STATIC void rp2_state_machine_reset_all(void) {
     rp2_state_machine_claimed_mask = 0;
 }
 
-STATIC void rp2_state_machine_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
+static void rp2_state_machine_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_printf(print, "StateMachine(%u)", self->id);
 }
@@ -508,7 +508,7 @@ STATIC void rp2_state_machine_print(const mp_print_t *print, mp_obj_t self_in, m
 //     sideset_base=None, in_shiftdir=None, out_shiftdir=None,
 //     push_thresh=None, pull_thresh=None,
 // )
-STATIC mp_obj_t rp2_state_machine_init_helper(const rp2_state_machine_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t rp2_state_machine_init_helper(const rp2_state_machine_obj_t *self, size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     enum {
         ARG_prog, ARG_freq,
         ARG_in_base, ARG_out_base, ARG_set_base, ARG_jmp_pin, ARG_sideset_base,
@@ -639,7 +639,7 @@ STATIC mp_obj_t rp2_state_machine_init_helper(const rp2_state_machine_obj_t *sel
 }
 
 // StateMachine(id, ...)
-STATIC mp_obj_t rp2_state_machine_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
+static mp_obj_t rp2_state_machine_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
     mp_arg_check_num(n_args, n_kw, 1, MP_OBJ_FUN_ARGS_MAX, true);
 
     // Get the StateMachine object.
@@ -657,32 +657,32 @@ STATIC mp_obj_t rp2_state_machine_make_new(const mp_obj_type_t *type, size_t n_a
     return MP_OBJ_FROM_PTR(self);
 }
 
-STATIC mp_obj_t rp2_state_machine_init(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
+static mp_obj_t rp2_state_machine_init(size_t n_args, const mp_obj_t *args, mp_map_t *kw_args) {
     return rp2_state_machine_init_helper(MP_OBJ_TO_PTR(args[0]), n_args - 1, args + 1, kw_args);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(rp2_state_machine_init_obj, 1, rp2_state_machine_init);
+static MP_DEFINE_CONST_FUN_OBJ_KW(rp2_state_machine_init_obj, 1, rp2_state_machine_init);
 
 // StateMachine.active([value])
-STATIC mp_obj_t rp2_state_machine_active(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t rp2_state_machine_active(size_t n_args, const mp_obj_t *args) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(args[0]);
     if (n_args > 1) {
         pio_sm_set_enabled(self->pio, self->sm, mp_obj_is_true(args[1]));
     }
     return mp_obj_new_bool((self->pio->ctrl >> self->sm) & 1);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_state_machine_active_obj, 1, 2, rp2_state_machine_active);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_state_machine_active_obj, 1, 2, rp2_state_machine_active);
 
 // StateMachine.restart()
-STATIC mp_obj_t rp2_state_machine_restart(mp_obj_t self_in) {
+static mp_obj_t rp2_state_machine_restart(mp_obj_t self_in) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(self_in);
     pio_sm_restart(self->pio, self->sm);
     pio_sm_exec(self->pio, self->sm, pio_encode_jmp(rp2_state_machine_initial_pc[self->id]));
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(rp2_state_machine_restart_obj, rp2_state_machine_restart);
+static MP_DEFINE_CONST_FUN_OBJ_1(rp2_state_machine_restart_obj, rp2_state_machine_restart);
 
 // StateMachine.exec(instr)
-STATIC mp_obj_t rp2_state_machine_exec(mp_obj_t self_in, mp_obj_t instr_in) {
+static mp_obj_t rp2_state_machine_exec(mp_obj_t self_in, mp_obj_t instr_in) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(self_in);
     mp_int_t encoded = 0;
     if (!mp_obj_get_int_maybe(instr_in, &encoded)) {
@@ -700,10 +700,10 @@ STATIC mp_obj_t rp2_state_machine_exec(mp_obj_t self_in, mp_obj_t instr_in) {
     pio_sm_exec(self->pio, self->sm, encoded);
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_2(rp2_state_machine_exec_obj, rp2_state_machine_exec);
+static MP_DEFINE_CONST_FUN_OBJ_2(rp2_state_machine_exec_obj, rp2_state_machine_exec);
 
 // StateMachine.get(buf=None, shift=0)
-STATIC mp_obj_t rp2_state_machine_get(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t rp2_state_machine_get(size_t n_args, const mp_obj_t *args) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(args[0]);
     mp_buffer_info_t bufinfo;
     bufinfo.buf = NULL;
@@ -729,7 +729,7 @@ STATIC mp_obj_t rp2_state_machine_get(size_t n_args, const mp_obj_t *args) {
     for (;;) {
         while (pio_sm_is_rx_fifo_empty(self->pio, self->sm)) {
             // This delay must be fast.
-            MICROPY_EVENT_POLL_HOOK_FAST;
+            mp_event_handle_nowait();
         }
         uint32_t value = pio_sm_get(self->pio, self->sm) >> shift;
         if (dest == NULL) {
@@ -752,10 +752,10 @@ STATIC mp_obj_t rp2_state_machine_get(size_t n_args, const mp_obj_t *args) {
         }
     }
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_state_machine_get_obj, 1, 3, rp2_state_machine_get);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_state_machine_get_obj, 1, 3, rp2_state_machine_get);
 
 // StateMachine.put(value, shift=0)
-STATIC mp_obj_t rp2_state_machine_put(size_t n_args, const mp_obj_t *args) {
+static mp_obj_t rp2_state_machine_put(size_t n_args, const mp_obj_t *args) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(args[0]);
     uint32_t shift = 0;
     if (n_args > 2) {
@@ -787,30 +787,47 @@ STATIC mp_obj_t rp2_state_machine_put(size_t n_args, const mp_obj_t *args) {
         }
         while (pio_sm_is_tx_fifo_full(self->pio, self->sm)) {
             // This delay must be fast.
-            MICROPY_EVENT_POLL_HOOK_FAST;
+            mp_event_handle_nowait();
         }
         pio_sm_put(self->pio, self->sm, value << shift);
     }
     return mp_const_none;
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_state_machine_put_obj, 2, 3, rp2_state_machine_put);
+static MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(rp2_state_machine_put_obj, 2, 3, rp2_state_machine_put);
 
 // StateMachine.rx_fifo()
-STATIC mp_obj_t rp2_state_machine_rx_fifo(mp_obj_t self_in) {
+static mp_obj_t rp2_state_machine_rx_fifo(mp_obj_t self_in) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(self_in);
     return MP_OBJ_NEW_SMALL_INT(pio_sm_get_rx_fifo_level(self->pio, self->sm));
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(rp2_state_machine_rx_fifo_obj, rp2_state_machine_rx_fifo);
+static MP_DEFINE_CONST_FUN_OBJ_1(rp2_state_machine_rx_fifo_obj, rp2_state_machine_rx_fifo);
 
 // StateMachine.tx_fifo()
-STATIC mp_obj_t rp2_state_machine_tx_fifo(mp_obj_t self_in) {
+static mp_obj_t rp2_state_machine_tx_fifo(mp_obj_t self_in) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(self_in);
     return MP_OBJ_NEW_SMALL_INT(pio_sm_get_tx_fifo_level(self->pio, self->sm));
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_1(rp2_state_machine_tx_fifo_obj, rp2_state_machine_tx_fifo);
+static MP_DEFINE_CONST_FUN_OBJ_1(rp2_state_machine_tx_fifo_obj, rp2_state_machine_tx_fifo);
+
+// Buffer protocol implementation for StateMachine.
+// The buffer represents one of the FIFO ports of the state machine. Note that a different
+// pointer is returned depending on if this is for reading or writing.
+static mp_int_t rp2_state_machine_get_buffer(mp_obj_t o_in, mp_buffer_info_t *bufinfo, mp_uint_t flags) {
+    rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(o_in);
+
+    bufinfo->len = 4;
+    bufinfo->typecode = 'I';
+
+    if (flags & MP_BUFFER_WRITE) {
+        bufinfo->buf = (void *)&self->pio->txf[self->sm];
+    } else {
+        bufinfo->buf = (void *)&self->pio->rxf[self->sm];
+    }
+    return 0;
+}
 
 // StateMachine.irq(handler=None, trigger=0|1, hard=False)
-STATIC mp_obj_t rp2_state_machine_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+static mp_obj_t rp2_state_machine_irq(size_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
     enum { ARG_handler, ARG_trigger, ARG_hard };
     static const mp_arg_t allowed_args[] = {
         { MP_QSTR_handler, MP_ARG_OBJ, {.u_rom_obj = MP_ROM_NONE} },
@@ -863,9 +880,9 @@ STATIC mp_obj_t rp2_state_machine_irq(size_t n_args, const mp_obj_t *pos_args, m
 
     return MP_OBJ_FROM_PTR(irq);
 }
-STATIC MP_DEFINE_CONST_FUN_OBJ_KW(rp2_state_machine_irq_obj, 1, rp2_state_machine_irq);
+static MP_DEFINE_CONST_FUN_OBJ_KW(rp2_state_machine_irq_obj, 1, rp2_state_machine_irq);
 
-STATIC const mp_rom_map_elem_t rp2_state_machine_locals_dict_table[] = {
+static const mp_rom_map_elem_t rp2_state_machine_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_init), MP_ROM_PTR(&rp2_state_machine_init_obj) },
     { MP_ROM_QSTR(MP_QSTR_active), MP_ROM_PTR(&rp2_state_machine_active_obj) },
     { MP_ROM_QSTR(MP_QSTR_restart), MP_ROM_PTR(&rp2_state_machine_restart_obj) },
@@ -876,7 +893,7 @@ STATIC const mp_rom_map_elem_t rp2_state_machine_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_tx_fifo), MP_ROM_PTR(&rp2_state_machine_tx_fifo_obj) },
     { MP_ROM_QSTR(MP_QSTR_irq), MP_ROM_PTR(&rp2_state_machine_irq_obj) },
 };
-STATIC MP_DEFINE_CONST_DICT(rp2_state_machine_locals_dict, rp2_state_machine_locals_dict_table);
+static MP_DEFINE_CONST_DICT(rp2_state_machine_locals_dict, rp2_state_machine_locals_dict_table);
 
 MP_DEFINE_CONST_OBJ_TYPE(
     rp2_state_machine_type,
@@ -884,10 +901,11 @@ MP_DEFINE_CONST_OBJ_TYPE(
     MP_TYPE_FLAG_NONE,
     make_new, rp2_state_machine_make_new,
     print, rp2_state_machine_print,
+    buffer, rp2_state_machine_get_buffer,
     locals_dict, &rp2_state_machine_locals_dict
     );
 
-STATIC mp_uint_t rp2_state_machine_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
+static mp_uint_t rp2_state_machine_irq_trigger(mp_obj_t self_in, mp_uint_t new_trigger) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(self_in);
     rp2_state_machine_irq_obj_t *irq = MP_STATE_PORT(rp2_state_machine_irq_obj[PIO_NUM(self->pio)]);
     irq_set_enabled(self->irq, false);
@@ -897,7 +915,7 @@ STATIC mp_uint_t rp2_state_machine_irq_trigger(mp_obj_t self_in, mp_uint_t new_t
     return 0;
 }
 
-STATIC mp_uint_t rp2_state_machine_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
+static mp_uint_t rp2_state_machine_irq_info(mp_obj_t self_in, mp_uint_t info_type) {
     rp2_state_machine_obj_t *self = MP_OBJ_TO_PTR(self_in);
     rp2_state_machine_irq_obj_t *irq = MP_STATE_PORT(rp2_state_machine_irq_obj[PIO_NUM(self->pio)]);
     if (info_type == MP_IRQ_INFO_FLAGS) {
@@ -908,7 +926,7 @@ STATIC mp_uint_t rp2_state_machine_irq_info(mp_obj_t self_in, mp_uint_t info_typ
     return 0;
 }
 
-STATIC const mp_irq_methods_t rp2_state_machine_irq_methods = {
+static const mp_irq_methods_t rp2_state_machine_irq_methods = {
     .trigger = rp2_state_machine_irq_trigger,
     .info = rp2_state_machine_irq_info,
 };
