@@ -32,6 +32,7 @@
 #include "extmod/modnetwork.h"
 #include "mpu.h"
 #include "eth.h"
+#include "eth_phy.h"
 
 #if defined(MICROPY_HW_ETH_MDC)
 
@@ -40,29 +41,20 @@
 #include "lwip/dhcp.h"
 #include "netif/ethernet.h"
 
-// ETH PHY register definitions (for LAN8742)
-
-#undef PHY_BCR
-#define PHY_BCR                 (0x0000)
-#define PHY_BCR_SOFT_RESET      (0x8000)
-#define PHY_BCR_AUTONEG_EN      (0x1000)
-#define PHY_BCR_POWER_DOWN      (0x0800U)
-
-#undef PHY_BSR
-#define PHY_BSR                 (0x0001)
-#define PHY_BSR_LINK_STATUS     (0x0004)
-#define PHY_BSR_AUTONEG_DONE    (0x0020)
-
-#define PHY_SCSR                (0x001f)
-#define PHY_SCSR_SPEED_Pos      (2)
-#define PHY_SCSR_SPEED_Msk      (7 << PHY_SCSR_SPEED_Pos)
-#define PHY_SCSR_SPEED_10HALF   (1 << PHY_SCSR_SPEED_Pos)
-#define PHY_SCSR_SPEED_10FULL   (5 << PHY_SCSR_SPEED_Pos)
-#define PHY_SCSR_SPEED_100HALF  (2 << PHY_SCSR_SPEED_Pos)
-#define PHY_SCSR_SPEED_100FULL  (6 << PHY_SCSR_SPEED_Pos)
-
 // ETH DMA RX and TX descriptor definitions
-#if defined(STM32H7)
+#if defined(STM32H5)
+#define RX_DESCR_3_OWN_Pos      (31)
+#define RX_DESCR_3_IOC_Pos      (30)
+#define RX_DESCR_3_BUF1V_Pos    (24)
+#define RX_DESCR_3_PL_Msk       (0x7fff)
+
+#define TX_DESCR_3_OWN_Pos      (31)
+#define TX_DESCR_3_FD_Pos       (29)
+#define TX_DESCR_3_LD_Pos       (28)
+#define TX_DESCR_3_CIC_Pos      (16)
+#define TX_DESCR_2_B1L_Pos      (0)
+#define TX_DESCR_2_B1L_Msk      (0x3fff << TX_DESCR_2_B1L_Pos)
+#elif defined(STM32H7)
 #define RX_DESCR_3_OWN_Pos      (31)
 #define RX_DESCR_3_IOC_Pos      (30)
 #define RX_DESCR_3_BUF1V_Pos    (24)
@@ -125,20 +117,24 @@ typedef struct _eth_t {
     uint32_t trace_flags;
     struct netif netif;
     struct dhcp dhcp_struct;
+    uint32_t phy_addr;
+    int16_t (*phy_get_link_status)(uint32_t phy_addr);
 } eth_t;
 
 static eth_dma_t eth_dma __attribute__((aligned(16384)));
 
 eth_t eth_instance;
 
-STATIC void eth_mac_deinit(eth_t *self);
-STATIC void eth_process_frame(eth_t *self, size_t len, const uint8_t *buf);
+static void eth_mac_deinit(eth_t *self);
+static void eth_process_frame(eth_t *self, size_t len, const uint8_t *buf);
 
-STATIC void eth_phy_write(uint32_t reg, uint32_t val) {
-    #if defined(STM32H7)
+void eth_phy_write(uint32_t phy_addr, uint32_t reg, uint32_t val) {
+    #if defined(STM32H5) || defined(STM32H7)
     while (ETH->MACMDIOAR & ETH_MACMDIOAR_MB) {
     }
     uint32_t ar = ETH->MACMDIOAR;
+    ar &= ~ETH_MACMDIOAR_PA_Msk;
+    ar |= (phy_addr << ETH_MACMDIOAR_PA_Pos);
     ar &= ~ETH_MACMDIOAR_RDA_Msk;
     ar |= reg << ETH_MACMDIOAR_RDA_Pos;
     ar &= ~ETH_MACMDIOAR_MOC_Msk;
@@ -153,18 +149,20 @@ STATIC void eth_phy_write(uint32_t reg, uint32_t val) {
     }
     ETH->MACMIIDR = val;
     uint32_t ar = ETH->MACMIIAR;
-    ar = reg << ETH_MACMIIAR_MR_Pos | (ar & ETH_MACMIIAR_CR_Msk) | ETH_MACMIIAR_MW | ETH_MACMIIAR_MB;
+    ar = (phy_addr << ETH_MACMIIAR_PA_Pos) | (reg << ETH_MACMIIAR_MR_Pos) | (ar & ETH_MACMIIAR_CR_Msk) | ETH_MACMIIAR_MW | ETH_MACMIIAR_MB;
     ETH->MACMIIAR = ar;
     while (ETH->MACMIIAR & ETH_MACMIIAR_MB) {
     }
     #endif
 }
 
-STATIC uint32_t eth_phy_read(uint32_t reg) {
-    #if defined(STM32H7)
+uint32_t eth_phy_read(uint32_t phy_addr, uint32_t reg) {
+    #if defined(STM32H5) || defined(STM32H7)
     while (ETH->MACMDIOAR & ETH_MACMDIOAR_MB) {
     }
     uint32_t ar = ETH->MACMDIOAR;
+    ar &= ~ETH_MACMDIOAR_PA_Msk;
+    ar |= (phy_addr << ETH_MACMDIOAR_PA_Pos);
     ar &= ~ETH_MACMDIOAR_RDA_Msk;
     ar |= reg << ETH_MACMDIOAR_RDA_Pos;
     ar &= ~ETH_MACMDIOAR_MOC_Msk;
@@ -178,7 +176,7 @@ STATIC uint32_t eth_phy_read(uint32_t reg) {
     while (ETH->MACMIIAR & ETH_MACMIIAR_MB) {
     }
     uint32_t ar = ETH->MACMIIAR;
-    ar = reg << ETH_MACMIIAR_MR_Pos | (ar & ETH_MACMIIAR_CR_Msk) | ETH_MACMIIAR_MB;
+    ar = (phy_addr << ETH_MACMIIAR_PA_Pos) | (reg << ETH_MACMIIAR_MR_Pos) | (ar & ETH_MACMIIAR_CR_Msk) | ETH_MACMIIAR_MB;
     ETH->MACMIIAR = ar;
     while (ETH->MACMIIAR & ETH_MACMIIAR_MB) {
     }
@@ -186,9 +184,17 @@ STATIC uint32_t eth_phy_read(uint32_t reg) {
     #endif
 }
 
-void eth_init(eth_t *self, int mac_idx) {
+int eth_init(eth_t *self, int mac_idx, uint32_t phy_addr, int phy_type) {
     mp_hal_get_mac(mac_idx, &self->netif.hwaddr[0]);
     self->netif.hwaddr_len = 6;
+    self->phy_addr = phy_addr;
+    if (phy_type == ETH_PHY_DP83825 || phy_type == ETH_PHY_DP83848) {
+        self->phy_get_link_status = eth_phy_dp838xx_get_link_status;
+    } else if (phy_type == ETH_PHY_LAN8720 || phy_type == ETH_PHY_LAN8742) {
+        self->phy_get_link_status = eth_phy_lan87xx_get_link_status;
+    } else {
+        return -1;
+    }
 
     // Configure GPIO
     mp_hal_pin_config_alt_static(MICROPY_HW_ETH_MDC, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_NONE, STATIC_AF_ETH_MDC);
@@ -202,45 +208,61 @@ void eth_init(eth_t *self, int mac_idx) {
     mp_hal_pin_config_alt_static(MICROPY_HW_ETH_RMII_TXD1, MP_HAL_PIN_MODE_ALT, MP_HAL_PIN_PULL_NONE, STATIC_AF_ETH_RMII_TXD1);
 
     // Enable peripheral clock
-    #if defined(STM32H7)
+    #if defined(STM32H5)
+    __HAL_RCC_ETH_CLK_ENABLE();
+    __HAL_RCC_ETHTX_CLK_ENABLE();
+    __HAL_RCC_ETHRX_CLK_ENABLE();
+    #elif defined(STM32H7)
     __HAL_RCC_ETH1MAC_CLK_ENABLE();
     __HAL_RCC_ETH1TX_CLK_ENABLE();
     __HAL_RCC_ETH1RX_CLK_ENABLE();
     #else
     __HAL_RCC_ETH_CLK_ENABLE();
     #endif
+    return 0;
 }
 
 void eth_set_trace(eth_t *self, uint32_t value) {
     self->trace_flags = value;
 }
 
-STATIC int eth_mac_init(eth_t *self) {
+static int eth_mac_init(eth_t *self) {
     // Configure MPU
     uint32_t irq_state = mpu_config_start();
+    #if defined(STM32H5)
+    mpu_config_region(MPU_REGION_ETH, (uint32_t)&eth_dma, MPU_CONFIG_ETH(16 * 1024));
+    #else
     mpu_config_region(MPU_REGION_ETH, (uint32_t)&eth_dma, MPU_CONFIG_ETH(MPU_REGION_SIZE_16KB));
+    #endif
     mpu_config_end(irq_state);
 
-    // Enable peripheral clock
-    #if defined(STM32H7)
-    __HAL_RCC_ETH1MAC_CLK_ENABLE();
-    __HAL_RCC_ETH1TX_CLK_ENABLE();
-    __HAL_RCC_ETH1RX_CLK_ENABLE();
+    // Set MAC to reset state
+    #if defined(STM32H5)
+    __HAL_RCC_ETH_FORCE_RESET();
+    #elif defined(STM32H7)
     __HAL_RCC_ETH1MAC_FORCE_RESET();
     #else
-    __HAL_RCC_ETH_CLK_ENABLE();
     __HAL_RCC_ETHMAC_FORCE_RESET();
     #endif
 
     // Select RMII interface
-    #if defined(STM32H7)
+    #if defined(STM32H5)
+    __HAL_RCC_SBS_CLK_ENABLE();
+    SBS->PMCR = (SBS->PMCR & ~SBS_PMCR_ETH_SEL_PHY_Msk) | SBS_PMCR_ETH_SEL_PHY_2;
+    #elif defined(STM32H7)
     SYSCFG->PMCR = (SYSCFG->PMCR & ~SYSCFG_PMCR_EPIS_SEL_Msk) | SYSCFG_PMCR_EPIS_SEL_2;
     #else
     __HAL_RCC_SYSCFG_CLK_ENABLE();
     SYSCFG->PMC |= SYSCFG_PMC_MII_RMII_SEL;
     #endif
 
-    #if defined(STM32H7)
+    #if defined(STM32H5)
+    __HAL_RCC_ETH_RELEASE_RESET();
+
+    __HAL_RCC_ETH_CLK_SLEEP_ENABLE();
+    __HAL_RCC_ETHTX_CLK_SLEEP_ENABLE();
+    __HAL_RCC_ETHRX_CLK_SLEEP_ENABLE();
+    #elif defined(STM32H7)
     __HAL_RCC_ETH1MAC_RELEASE_RESET();
 
     __HAL_RCC_ETH1MAC_CLK_SLEEP_ENABLE();
@@ -255,7 +277,7 @@ STATIC int eth_mac_init(eth_t *self) {
     #endif
 
     // Do a soft reset of the MAC core
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     #define ETH_SOFT_RESET(eth) do { eth->DMAMR = ETH_DMAMR_SWR; } while (0)
     #define ETH_IS_RESET(eth) (eth->DMAMR & ETH_DMAMR_SWR)
     #else
@@ -277,7 +299,23 @@ STATIC int eth_mac_init(eth_t *self) {
     // Set MII clock range
     uint32_t hclk = HAL_RCC_GetHCLKFreq();
     uint32_t cr_div;
-    #if defined(STM32H7)
+    #if defined(STM32H5)
+    cr_div = ETH->MACMDIOAR & ~ETH_MACMDIOAR_CR;
+    if (hclk < 35000000) {
+        cr_div |= ETH_MACMDIOAR_CR_DIV16;
+    } else if (hclk < 60000000) {
+        cr_div |= ETH_MACMDIOAR_CR_DIV26;
+    } else if (hclk < 100000000) {
+        cr_div |= ETH_MACMDIOAR_CR_DIV42;
+    } else if (hclk < 150000000) {
+        cr_div |= ETH_MACMDIOAR_CR_DIV62;
+    } else if (hclk < 250000000) {
+        cr_div |= ETH_MACMDIOAR_CR_DIV102;
+    } else {
+        cr_div |= ETH_MACMDIOAR_CR_DIV124;
+    }
+    ETH->MACMDIOAR = cr_div;
+    #elif defined(STM32H7)
     cr_div = ETH->MACMDIOAR & ~ETH_MACMDIOAR_CR;
     if (hclk < 35000000) {
         cr_div |= ETH_MACMDIOAR_CR_DIV16;
@@ -306,13 +344,13 @@ STATIC int eth_mac_init(eth_t *self) {
     ETH->MACMIIAR = cr_div;
     #endif
 
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     // don't skip 32bit words since our descriptors are continuous in memory
     ETH->DMACCR &= ~(ETH_DMACCR_DSL_Msk);
     #endif
 
     // Reset the PHY
-    eth_phy_write(PHY_BCR, PHY_BCR_SOFT_RESET);
+    eth_phy_write(self->phy_addr, PHY_BCR, PHY_BCR_SOFT_RESET);
     mp_hal_delay_ms(50);
 
     // Wait for the PHY link to be established
@@ -323,8 +361,8 @@ STATIC int eth_mac_init(eth_t *self) {
             eth_mac_deinit(self);
             return -MP_ETIMEDOUT;
         }
-        uint16_t bcr = eth_phy_read(0);
-        uint16_t bsr = eth_phy_read(1);
+        uint16_t bcr = eth_phy_read(self->phy_addr, PHY_BCR);
+        uint16_t bsr = eth_phy_read(self->phy_addr, PHY_BSR);
         switch (phy_state) {
             case 0:
                 if (!(bcr & PHY_BCR_SOFT_RESET)) {
@@ -333,7 +371,15 @@ STATIC int eth_mac_init(eth_t *self) {
                 break;
             case 1:
                 if (bsr & PHY_BSR_LINK_STATUS) {
-                    eth_phy_write(PHY_BCR, PHY_BCR_AUTONEG_EN);
+                    // Announce all modes
+                    eth_phy_write(self->phy_addr, PHY_ANAR,
+                        PHY_ANAR_SPEED_10HALF |
+                        PHY_ANAR_SPEED_10FULL |
+                        PHY_ANAR_SPEED_100HALF |
+                        PHY_ANAR_SPEED_100FULL |
+                        PHY_ANAR_IEEE802_3);
+                    // Start autonegotiate.
+                    eth_phy_write(self->phy_addr, PHY_BCR, PHY_BCR_AUTONEG_EN);
                     phy_state = 2;
                 }
                 break;
@@ -348,10 +394,10 @@ STATIC int eth_mac_init(eth_t *self) {
     }
 
     // Get register with link status
-    uint16_t phy_scsr = eth_phy_read(PHY_SCSR);
+    uint16_t phy_scsr = self->phy_get_link_status(self->phy_addr);
 
     // Burst mode configuration
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     ETH->DMASBMR = ETH->DMASBMR & ~ETH_DMASBMR_AAL & ~ETH_DMASBMR_FB;
     #else
     ETH->DMABMR = 0;
@@ -359,7 +405,7 @@ STATIC int eth_mac_init(eth_t *self) {
     mp_hal_delay_ms(2);
 
     // Select DMA interrupts
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     ETH->DMACIER = ETH->DMACIER
         | ETH_DMACIER_NIE // enable normal interrupts
         | ETH_DMACIER_RIE // enable RX interrupt
@@ -373,7 +419,7 @@ STATIC int eth_mac_init(eth_t *self) {
 
     // Configure RX descriptor lists
     for (size_t i = 0; i < RX_BUF_NUM; ++i) {
-        #if defined(STM32H7)
+        #if defined(STM32H5) || defined(STM32H7)
         eth_dma.rx_descr[i].rdes3 =
             1 << RX_DESCR_3_OWN_Pos
                 | (1 << RX_DESCR_3_BUF1V_Pos) // buf1 address valid
@@ -391,7 +437,7 @@ STATIC int eth_mac_init(eth_t *self) {
         #endif
     }
 
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     ETH->DMACRDLAR = (uint32_t)&eth_dma.rx_descr[0];
     #else
     ETH->DMARDLAR = (uint32_t)&eth_dma.rx_descr[0];
@@ -400,7 +446,7 @@ STATIC int eth_mac_init(eth_t *self) {
 
     // Configure TX descriptor lists
     for (size_t i = 0; i < TX_BUF_NUM; ++i) {
-        #if defined(STM32H7)
+        #if defined(STM32H5) || defined(STM32H7)
         eth_dma.tx_descr[i].tdes0 = 0;
         eth_dma.tx_descr[i].tdes1 = 0;
         eth_dma.tx_descr[i].tdes2 = TX_BUF_SIZE & TX_DESCR_2_B1L_Msk;
@@ -413,7 +459,7 @@ STATIC int eth_mac_init(eth_t *self) {
         #endif
     }
 
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     // set number of descriptors and buffers
     ETH->DMACTDRLR = TX_BUF_NUM - 1;
     ETH->DMACRDRLR = RX_BUF_NUM - 1;
@@ -425,7 +471,7 @@ STATIC int eth_mac_init(eth_t *self) {
     eth_dma.tx_descr_idx = 0;
 
     // Configure DMA
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     // read from RX FIFO only after a full frame is written
     ETH->MTLRQOMR = ETH_MTLRQOMR_RSF;
     // transmission starts when a full packet resides in the Tx queue
@@ -439,7 +485,7 @@ STATIC int eth_mac_init(eth_t *self) {
     mp_hal_delay_ms(2);
 
     // Select MAC filtering options
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     ETH->MACPFR = ETH_MACPFR_RA; // pass all frames up
     #else
     ETH->MACFFR =
@@ -457,9 +503,9 @@ STATIC int eth_mac_init(eth_t *self) {
 
     // Set main MAC control register
     ETH->MACCR =
-        (phy_scsr & PHY_SCSR_SPEED_Msk) == PHY_SCSR_SPEED_10FULL ? ETH_MACCR_DM
-        : (phy_scsr & PHY_SCSR_SPEED_Msk) == PHY_SCSR_SPEED_100HALF ? ETH_MACCR_FES
-        : (phy_scsr & PHY_SCSR_SPEED_Msk) == PHY_SCSR_SPEED_100FULL ? (ETH_MACCR_FES | ETH_MACCR_DM)
+        phy_scsr == PHY_SPEED_10FULL ? ETH_MACCR_DM
+        : phy_scsr == PHY_SPEED_100HALF ? ETH_MACCR_FES
+        : phy_scsr == PHY_SPEED_100FULL ? (ETH_MACCR_FES | ETH_MACCR_DM)
         : 0
     ;
     mp_hal_delay_ms(2);
@@ -472,7 +518,7 @@ STATIC int eth_mac_init(eth_t *self) {
     mp_hal_delay_ms(2);
 
     // Start DMA layer
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     ETH->DMACRCR |= ETH_DMACRCR_SR; // start RX
     ETH->DMACTCR |= ETH_DMACTCR_ST; // start TX
     #else
@@ -490,10 +536,14 @@ STATIC int eth_mac_init(eth_t *self) {
     return 0;
 }
 
-STATIC void eth_mac_deinit(eth_t *self) {
+static void eth_mac_deinit(eth_t *self) {
     (void)self;
     HAL_NVIC_DisableIRQ(ETH_IRQn);
-    #if defined(STM32H7)
+    #if defined(STM32H5)
+    __HAL_RCC_ETH_FORCE_RESET();
+    __HAL_RCC_ETH_RELEASE_RESET();
+    __HAL_RCC_ETH_CLK_DISABLE();
+    #elif defined(STM32H7)
     __HAL_RCC_ETH1MAC_FORCE_RESET();
     __HAL_RCC_ETH1MAC_RELEASE_RESET();
     __HAL_RCC_ETH1MAC_CLK_DISABLE();
@@ -504,7 +554,7 @@ STATIC void eth_mac_deinit(eth_t *self) {
     #endif
 }
 
-STATIC int eth_tx_buf_get(size_t len, uint8_t **buf) {
+static int eth_tx_buf_get(size_t len, uint8_t **buf) {
     if (len > TX_BUF_SIZE) {
         return -MP_EINVAL;
     }
@@ -513,7 +563,7 @@ STATIC int eth_tx_buf_get(size_t len, uint8_t **buf) {
     eth_dma_tx_descr_t *tx_descr = &eth_dma.tx_descr[eth_dma.tx_descr_idx];
     uint32_t t0 = mp_hal_ticks_ms();
     for (;;) {
-        #if defined(STM32H7)
+        #if defined(STM32H5) || defined(STM32H7)
         if (!(tx_descr->tdes3 & (1 << TX_DESCR_3_OWN_Pos))) {
             break;
         }
@@ -527,7 +577,7 @@ STATIC int eth_tx_buf_get(size_t len, uint8_t **buf) {
         }
     }
 
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     // Update TX descriptor with length and buffer pointer
     *buf = &eth_dma.tx_buf[eth_dma.tx_descr_idx * TX_BUF_SIZE];
     tx_descr->tdes2 = len & TX_DESCR_2_B1L_Msk;
@@ -543,13 +593,13 @@ STATIC int eth_tx_buf_get(size_t len, uint8_t **buf) {
     return 0;
 }
 
-STATIC int eth_tx_buf_send(void) {
+static int eth_tx_buf_send(void) {
     // Get TX descriptor and move to next one
     eth_dma_tx_descr_t *tx_descr = &eth_dma.tx_descr[eth_dma.tx_descr_idx];
     eth_dma.tx_descr_idx = (eth_dma.tx_descr_idx + 1) % TX_BUF_NUM;
 
     // Schedule to send next outgoing frame
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     tx_descr->tdes3 =
         1 << TX_DESCR_3_OWN_Pos     // owned by DMA
             | 1 << TX_DESCR_3_LD_Pos // last segment
@@ -568,7 +618,7 @@ STATIC int eth_tx_buf_send(void) {
 
     // Notify ETH DMA that there is a new TX descriptor for sending
     __DMB();
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     if (ETH->DMACSR & ETH_DMACSR_TBU) {
         ETH->DMACSR = ETH_DMACSR_TBU;
     }
@@ -583,14 +633,14 @@ STATIC int eth_tx_buf_send(void) {
     return 0;
 }
 
-STATIC void eth_dma_rx_free(void) {
+static void eth_dma_rx_free(void) {
     // Get RX descriptor, RX buffer and move to next one
     eth_dma_rx_descr_t *rx_descr = &eth_dma.rx_descr[eth_dma.rx_descr_idx];
     uint8_t *buf = &eth_dma.rx_buf[eth_dma.rx_descr_idx * RX_BUF_SIZE];
     eth_dma.rx_descr_idx = (eth_dma.rx_descr_idx + 1) % RX_BUF_NUM;
 
     // Schedule to get next incoming frame
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     rx_descr->rdes0 = (uint32_t)buf;
     rx_descr->rdes3 = 1 << RX_DESCR_3_OWN_Pos;  // owned by DMA
     rx_descr->rdes3 |= 1 << RX_DESCR_3_BUF1V_Pos; // buf 1 address valid
@@ -607,7 +657,7 @@ STATIC void eth_dma_rx_free(void) {
 
     // Notify ETH DMA that there is a new RX descriptor available
     __DMB();
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     ETH->DMACRDTPR = (uint32_t)&rx_descr[eth_dma.rx_descr_idx];
     #else
     ETH->DMARPDR = 0;
@@ -615,7 +665,7 @@ STATIC void eth_dma_rx_free(void) {
 }
 
 void ETH_IRQHandler(void) {
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     uint32_t sr = ETH->DMACSR;
     ETH->DMACSR = ETH_DMACSR_NIS;
     uint32_t rx_interrupt = sr & ETH_DMACSR_RI;
@@ -625,13 +675,13 @@ void ETH_IRQHandler(void) {
     uint32_t rx_interrupt = sr & ETH_DMASR_RS;
     #endif
     if (rx_interrupt) {
-        #if defined(STM32H7)
+        #if defined(STM32H5) || defined(STM32H7)
         ETH->DMACSR = ETH_DMACSR_RI;
         #else
         ETH->DMASR = ETH_DMASR_RS;
         #endif
         for (;;) {
-            #if defined(STM32H7)
+            #if defined(STM32H5) || defined(STM32H7)
             eth_dma_rx_descr_t *rx_descr_l = &eth_dma.rx_descr[eth_dma.rx_descr_idx];
             if (rx_descr_l->rdes3 & (1 << RX_DESCR_3_OWN_Pos)) {
                 // No more RX descriptors ready to read
@@ -646,13 +696,13 @@ void ETH_IRQHandler(void) {
             #endif
 
             // Get RX buffer containing new frame
-            #if defined(STM32H7)
+            #if defined(STM32H5) || defined(STM32H7)
             size_t len = (rx_descr_l->rdes3 & RX_DESCR_3_PL_Msk);
             #else
             size_t len = (rx_descr->rdes0 & RX_DESCR_0_FL_Msk) >> RX_DESCR_0_FL_Pos;
             #endif
             len -= 4; // discard CRC at end
-            #if defined(STM32H7)
+            #if defined(STM32H5) || defined(STM32H7)
             uint8_t *buf = &eth_dma.rx_buf[eth_dma.rx_descr_idx * RX_BUF_SIZE];
             #else
             uint8_t *buf = (uint8_t *)rx_descr->rdes2;
@@ -673,7 +723,7 @@ void ETH_IRQHandler(void) {
 #define TRACE_ETH_RX (0x0004)
 #define TRACE_ETH_FULL (0x0008)
 
-STATIC void eth_trace(eth_t *self, size_t len, const void *data, unsigned int flags) {
+static void eth_trace(eth_t *self, size_t len, const void *data, unsigned int flags) {
     if (((flags & NETUTILS_TRACE_IS_TX) && (self->trace_flags & TRACE_ETH_TX))
         || (!(flags & NETUTILS_TRACE_IS_TX) && (self->trace_flags & TRACE_ETH_RX))) {
         const uint8_t *buf;
@@ -693,7 +743,7 @@ STATIC void eth_trace(eth_t *self, size_t len, const void *data, unsigned int fl
     }
 }
 
-STATIC err_t eth_netif_output(struct netif *netif, struct pbuf *p) {
+static err_t eth_netif_output(struct netif *netif, struct pbuf *p) {
     // This function should always be called from a context where PendSV-level IRQs are disabled
 
     LINK_STATS_INC(link.xmit);
@@ -709,7 +759,7 @@ STATIC err_t eth_netif_output(struct netif *netif, struct pbuf *p) {
     return ret ? ERR_BUF : ERR_OK;
 }
 
-STATIC err_t eth_netif_init(struct netif *netif) {
+static err_t eth_netif_init(struct netif *netif) {
     netif->linkoutput = eth_netif_output;
     netif->output = etharp_output;
     netif->mtu = 1500;
@@ -724,7 +774,7 @@ STATIC err_t eth_netif_init(struct netif *netif) {
     return ERR_OK;
 }
 
-STATIC void eth_lwip_init(eth_t *self) {
+static void eth_lwip_init(eth_t *self) {
     ip_addr_t ipconfig[4];
     IP4_ADDR(&ipconfig[0], 0, 0, 0, 0);
     IP4_ADDR(&ipconfig[2], 192, 168, 0, 1);
@@ -750,7 +800,7 @@ STATIC void eth_lwip_init(eth_t *self) {
     MICROPY_PY_LWIP_EXIT
 }
 
-STATIC void eth_lwip_deinit(eth_t *self) {
+static void eth_lwip_deinit(eth_t *self) {
     MICROPY_PY_LWIP_ENTER
     for (struct netif *netif = netif_list; netif != NULL; netif = netif->next) {
         if (netif == &self->netif) {
@@ -762,7 +812,7 @@ STATIC void eth_lwip_deinit(eth_t *self) {
     MICROPY_PY_LWIP_EXIT
 }
 
-STATIC void eth_process_frame(eth_t *self, size_t len, const uint8_t *buf) {
+static void eth_process_frame(eth_t *self, size_t len, const uint8_t *buf) {
     eth_trace(self, len, buf, NETUTILS_TRACE_NEWLINE);
 
     struct netif *netif = &self->netif;
@@ -791,7 +841,7 @@ int eth_link_status(eth_t *self) {
             return 2; // link no-ip;
         }
     } else {
-        if (eth_phy_read(PHY_BSR) & PHY_BSR_LINK_STATUS) {
+        if (eth_phy_read(self->phy_addr, PHY_BSR) & PHY_BSR_LINK_STATUS) {
             return 1; // link up
         } else {
             return 0; // link down
@@ -829,10 +879,10 @@ void eth_low_power_mode(eth_t *self, bool enable) {
     __HAL_RCC_ETH_CLK_ENABLE();
     #endif
 
-    uint16_t bcr = eth_phy_read(PHY_BCR);
+    uint16_t bcr = eth_phy_read(self->phy_addr, PHY_BCR);
     if (enable) {
         // Enable low-power mode.
-        eth_phy_write(PHY_BCR, bcr | PHY_BCR_POWER_DOWN);
+        eth_phy_write(self->phy_addr, PHY_BCR, bcr | PHY_BCR_POWER_DOWN);
         // Disable eth clock.
         #if defined(STM32H7)
         __HAL_RCC_ETH1MAC_CLK_DISABLE();
@@ -841,7 +891,7 @@ void eth_low_power_mode(eth_t *self, bool enable) {
         #endif
     } else {
         // Disable low-power mode.
-        eth_phy_write(PHY_BCR, bcr & (~PHY_BCR_POWER_DOWN));
+        eth_phy_write(self->phy_addr, PHY_BCR, bcr & (~PHY_BCR_POWER_DOWN));
     }
 }
 #endif // defined(MICROPY_HW_ETH_MDC)
