@@ -32,11 +32,16 @@
 /*****************************************************************************/
 // Feature settings with defaults
 
-// Whether to include the stm module, with peripheral register constants
+// Whether to include the stm module
 #ifndef MICROPY_PY_STM
 #define MICROPY_PY_STM (1)
 #endif
 
+// Whether to include named register constants in the stm module
+#ifndef MICROPY_PY_STM_CONST
+#define MICROPY_PY_STM_CONST (MICROPY_PY_STM)
+#endif
+
 // Whether to include the pyb module
 #ifndef MICROPY_PY_PYB
 #define MICROPY_PY_PYB (1)
@@ -251,6 +256,21 @@
 #define MICROPY_HW_USB_INTERFACE_FS_STRING      "Pyboard Interface"
 #endif
 
+// Must be 8 bytes.
+#ifndef MICROPY_HW_USB_MSC_INQUIRY_VENDOR_STRING
+#define MICROPY_HW_USB_MSC_INQUIRY_VENDOR_STRING "MicroPy "
+#endif
+
+// Must be 16 bytes.
+#ifndef MICROPY_HW_USB_MSC_INQUIRY_PRODUCT_STRING
+#define MICROPY_HW_USB_MSC_INQUIRY_PRODUCT_STRING "pyboard Flash   "
+#endif
+
+// Must be 4 bytes.
+#ifndef MICROPY_HW_USB_MSC_INQUIRY_REVISION_STRING
+#define MICROPY_HW_USB_MSC_INQUIRY_REVISION_STRING "1.00"
+#endif
+
 // Amount of incoming buffer space for each CDC instance.
 // This must be 2 or greater, and a power of 2.
 #ifndef MICROPY_HW_USB_CDC_RX_DATA_SIZE
@@ -339,6 +359,16 @@
 #define MICROPY_HW_MAX_UART (5) // UART1-5 + LPUART1
 #define MICROPY_HW_MAX_LPUART (1)
 
+// Configuration for STM32H5 series
+#elif defined(STM32H5)
+
+#define MP_HAL_UNIQUE_ID_ADDRESS (mp_hal_unique_id_address)
+#define PYB_EXTI_NUM_VECTORS (58)
+#define MICROPY_HW_MAX_I2C (4)
+#define MICROPY_HW_MAX_TIMER (17)
+#define MICROPY_HW_MAX_UART (12)
+#define MICROPY_HW_MAX_LPUART (1)
+
 // Configuration for STM32H7A3/B3 series
 #elif defined(STM32H7A3xx) || defined(STM32H7A3xxQ) || \
     defined(STM32H7B3xx) || defined(STM32H7B3xxQ)
@@ -360,6 +390,15 @@
 #define MICROPY_HW_MAX_UART (8)
 #define MICROPY_HW_MAX_LPUART (1)
 
+#if defined(MICROPY_HW_ANALOG_SWITCH_PA0) \
+    || defined(MICROPY_HW_ANALOG_SWITCH_PA1) \
+    || defined(MICROPY_HW_ANALOG_SWITCH_PC2) \
+    || defined(MICROPY_HW_ANALOG_SWITCH_PC3)
+#define MICROPY_HW_ENABLE_ANALOG_ONLY_PINS (1)
+#else
+#define MICROPY_HW_ENABLE_ANALOG_ONLY_PINS (0)
+#endif
+
 // Configuration for STM32L0 series
 #elif defined(STM32L0)
 
@@ -370,6 +409,16 @@
 #define MICROPY_HW_MAX_UART (5)
 #define MICROPY_HW_MAX_LPUART (1)
 
+// Configuration for STM32L1 series
+#elif defined(STM32L1)
+#define MP_HAL_UNIQUE_ID_ADDRESS (UID_BASE)
+#define PYB_EXTI_NUM_VECTORS (23)
+#define MICROPY_HW_MAX_I2C (2)
+// TODO: L1 has 9 timers but tim0 and tim1 don't exist.
+#define MICROPY_HW_MAX_TIMER (11)
+#define MICROPY_HW_MAX_UART (5)
+#define MICROPY_HW_MAX_LPUART (0)
+
 // Configuration for STM32L4 series
 #elif defined(STM32L4)
 
@@ -451,8 +500,12 @@
 #define MICROPY_HW_RCC_HSI_STATE (RCC_HSI_OFF)
 #define MICROPY_HW_RCC_FLAG_HSxRDY (RCC_FLAG_HSERDY)
 #if MICROPY_HW_CLK_USE_BYPASS
+#if !defined(STM32WL)
 #define MICROPY_HW_RCC_HSE_STATE (RCC_HSE_BYPASS)
 #else
+#define MICROPY_HW_RCC_HSE_STATE (RCC_HSE_BYPASS_PWR)
+#endif
+#else
 #define MICROPY_HW_RCC_HSE_STATE (RCC_HSE_ON)
 #endif
 #endif
@@ -489,6 +542,21 @@
 #define MICROPY_HW_BDEV_WRITEBLOCK flash_bdev_writeblock
 #endif
 
+#if defined(MICROPY_HW_BDEV_SPIFLASH)
+// Provide block device macros using spi_bdev.
+// The board must provide settings for:
+//  - MICROPY_HW_BDEV_SPIFLASH - pointer to a spi_bdev_t
+//  - MICROPY_HW_BDEV_SPIFLASH_CONFIG - pointer to an mp_spiflash_config_t
+//  - MICROPY_HW_BDEV_SPIFLASH_SIZE_BYTES - size in bytes of the SPI flash
+#define MICROPY_HW_BDEV_IOCTL(op, arg) ( \
+    (op) == BDEV_IOCTL_NUM_BLOCKS ? (MICROPY_HW_BDEV_SPIFLASH_SIZE_BYTES / FLASH_BLOCK_SIZE) : \
+    (op) == BDEV_IOCTL_INIT ? spi_bdev_ioctl(MICROPY_HW_BDEV_SPIFLASH, (op), (uint32_t)MICROPY_HW_BDEV_SPIFLASH_CONFIG) : \
+    spi_bdev_ioctl(MICROPY_HW_BDEV_SPIFLASH, (op), (arg)) \
+    )
+#define MICROPY_HW_BDEV_READBLOCKS(dest, bl, n) spi_bdev_readblocks(MICROPY_HW_BDEV_SPIFLASH, (dest), (bl), (n))
+#define MICROPY_HW_BDEV_WRITEBLOCKS(src, bl, n) spi_bdev_writeblocks(MICROPY_HW_BDEV_SPIFLASH, (src), (bl), (n))
+#endif
+
 // Whether to enable caching for external SPI flash, to allow block writes that are
 // smaller than the native page-erase size of the SPI flash, eg when FAT FS is used.
 // Enabling this enables spi_bdev_readblocks() and spi_bdev_writeblocks() functions,
@@ -532,10 +600,10 @@
 
 // Enable I2S if there are any peripherals defined
 #if defined(MICROPY_HW_I2S1) || defined(MICROPY_HW_I2S2)
-#define MICROPY_HW_ENABLE_I2S (1)
+#define MICROPY_PY_MACHINE_I2S (1)
 #define MICROPY_HW_MAX_I2S (2)
 #else
-#define MICROPY_HW_ENABLE_I2S (0)
+#define MICROPY_PY_MACHINE_I2S (0)
 #define MICROPY_HW_MAX_I2S (0)
 #endif
 
@@ -549,7 +617,9 @@
 #endif
 
 // Whether the USB peripheral is device-only, or multiple OTG
-#if defined(STM32L0) || defined(STM32L432xx) || defined(STM32WB)
+// For STM32G0 and STM32H5 the USB peripheral supports device and host mode,
+// but otherwise acts like a non-multi-OTG peripheral.
+#if defined(STM32G0) || defined(STM32G4) || defined(STM32H5) || defined(STM32L0) || defined(STM32L1) || defined(STM32L432xx) || defined(STM32WB)
 #define MICROPY_HW_USB_IS_MULTI_OTG (0)
 #else
 #define MICROPY_HW_USB_IS_MULTI_OTG (1)
@@ -571,12 +641,12 @@
 
 // D-cache clean/invalidate helpers
 #if __DCACHE_PRESENT == 1
-#define MP_HAL_CLEANINVALIDATE_DCACHE(addr, size) \
-    (SCB_CleanInvalidateDCache_by_Addr((uint32_t *)((uint32_t)addr & ~0x1f), \
-    ((uint32_t)((uint8_t *)addr + size + 0x1f) & ~0x1f) - ((uint32_t)addr & ~0x1f)))
-#define MP_HAL_CLEAN_DCACHE(addr, size) \
-    (SCB_CleanDCache_by_Addr((uint32_t *)((uint32_t)addr & ~0x1f), \
-    ((uint32_t)((uint8_t *)addr + size + 0x1f) & ~0x1f) - ((uint32_t)addr & ~0x1f)))
+// Note: The SCB_Clean<...> functions automatically align their arguments to cover full cache lines.
+// CLEANINVALIDATE will write back (flush) any dirty lines in this region to RAM, then
+// invalidate (evict) the whole region from the cache.
+#define MP_HAL_CLEANINVALIDATE_DCACHE(addr, size) SCB_CleanInvalidateDCache_by_Addr((volatile void *)(addr), (size))
+// CLEAN will write back (flush) any dirty lines in this region to RAM.
+#define MP_HAL_CLEAN_DCACHE(addr, size) SCB_CleanDCache_by_Addr((volatile void *)(addr), (size))
 #else
 #define MP_HAL_CLEANINVALIDATE_DCACHE(addr, size)
 #define MP_HAL_CLEAN_DCACHE(addr, size)
