@@ -48,6 +48,8 @@
 #define POWERCTRL_GET_VOLTAGE_SCALING() PWR_REGULATOR_VOLTAGE_SCALE0
 #elif defined(STM32H723xx)
 #define POWERCTRL_GET_VOLTAGE_SCALING() LL_PWR_GetRegulVoltageScaling()
+#elif defined(STM32H5)
+#define POWERCTRL_GET_VOLTAGE_SCALING() LL_PWR_GetRegulVoltageScaling()
 #else
 #define POWERCTRL_GET_VOLTAGE_SCALING()     \
     (((PWR->CSR1 & PWR_CSR1_ACTVOS) && (SYSCFG->PWRCR & SYSCFG_PWRCR_ODEN)) ? \
@@ -160,14 +162,14 @@ typedef struct _sysclk_scaling_table_entry_t {
 } sysclk_scaling_table_entry_t;
 
 #if defined(STM32F7)
-STATIC const sysclk_scaling_table_entry_t volt_scale_table[] = {
+static const sysclk_scaling_table_entry_t volt_scale_table[] = {
     { 151, PWR_REGULATOR_VOLTAGE_SCALE3 },
     { 180, PWR_REGULATOR_VOLTAGE_SCALE2 },
     // Above 180MHz uses default PWR_REGULATOR_VOLTAGE_SCALE1
 };
 #elif defined(STM32H7A3xx) || defined(STM32H7A3xxQ) || \
     defined(STM32H7B3xx) || defined(STM32H7B3xxQ)
-STATIC const sysclk_scaling_table_entry_t volt_scale_table[] = {
+static const sysclk_scaling_table_entry_t volt_scale_table[] = {
     // See table 15 "FLASH recommended number of wait states and programming delay" of RM0455.
     {88, PWR_REGULATOR_VOLTAGE_SCALE3},
     {160, PWR_REGULATOR_VOLTAGE_SCALE2},
@@ -175,7 +177,7 @@ STATIC const sysclk_scaling_table_entry_t volt_scale_table[] = {
     {280, PWR_REGULATOR_VOLTAGE_SCALE0},
 };
 #elif defined(STM32H7)
-STATIC const sysclk_scaling_table_entry_t volt_scale_table[] = {
+static const sysclk_scaling_table_entry_t volt_scale_table[] = {
     // See table 55 "Kernel clock distribution overview" of RM0433.
     {200, PWR_REGULATOR_VOLTAGE_SCALE3},
     {300, PWR_REGULATOR_VOLTAGE_SCALE2},
@@ -184,7 +186,7 @@ STATIC const sysclk_scaling_table_entry_t volt_scale_table[] = {
 };
 #endif
 
-STATIC int powerctrl_config_vos(uint32_t sysclk_mhz) {
+static int powerctrl_config_vos(uint32_t sysclk_mhz) {
     #if defined(STM32F7) || defined(STM32H7)
     uint32_t volt_scale = PWR_REGULATOR_VOLTAGE_SCALE1;
     for (int i = 0; i < MP_ARRAY_SIZE(volt_scale_table); ++i) {
@@ -289,7 +291,7 @@ int powerctrl_rcc_clock_config_pll(RCC_ClkInitTypeDef *rcc_init, uint32_t sysclk
 
 #if !defined(STM32F0) && !defined(STM32G0) && !defined(STM32L0) && !defined(STM32L1) && !defined(STM32L4)
 
-STATIC uint32_t calc_ahb_div(uint32_t wanted_div) {
+static uint32_t calc_ahb_div(uint32_t wanted_div) {
     #if defined(STM32H7)
     if (wanted_div <= 1) {
         return RCC_HCLK_DIV1;
@@ -333,7 +335,7 @@ STATIC uint32_t calc_ahb_div(uint32_t wanted_div) {
     #endif
 }
 
-STATIC uint32_t calc_apb1_div(uint32_t wanted_div) {
+static uint32_t calc_apb1_div(uint32_t wanted_div) {
     #if defined(STM32H7)
     if (wanted_div <= 1) {
         return RCC_APB1_DIV1;
@@ -361,7 +363,7 @@ STATIC uint32_t calc_apb1_div(uint32_t wanted_div) {
     #endif
 }
 
-STATIC uint32_t calc_apb2_div(uint32_t wanted_div) {
+static uint32_t calc_apb2_div(uint32_t wanted_div) {
     #if defined(STM32H7)
     if (wanted_div <= 1) {
         return RCC_APB2_DIV1;
@@ -498,6 +500,9 @@ set_clk:
     RCC_OscInitStruct.OscillatorType = MICROPY_HW_RCC_OSCILLATOR_TYPE;
     RCC_OscInitStruct.HSEState = MICROPY_HW_RCC_HSE_STATE;
     RCC_OscInitStruct.HSIState = MICROPY_HW_RCC_HSI_STATE;
+    #if defined(STM32G0) || defined(STM32H5)
+    RCC_OscInitStruct.HSIDiv = RCC_HSI_DIV1;
+    #endif
     RCC_OscInitStruct.HSICalibrationValue = RCC_HSICALIBRATION_DEFAULT;
     RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
     RCC_OscInitStruct.PLL.PLLSource = MICROPY_HW_RCC_PLL_SRC;
@@ -506,7 +511,24 @@ set_clk:
     RCC_OscInitStruct.PLL.PLLP = p;
     RCC_OscInitStruct.PLL.PLLQ = q;
 
-    #if defined(STM32H7)
+    #if defined(STM32H5)
+    RCC_OscInitStruct.PLL.PLLR = 0;
+    if (MICROPY_HW_CLK_VALUE / 1000000 <= 2 * m) {
+        RCC_OscInitStruct.PLL.PLLRGE = RCC_PLL1_VCIRANGE_0; // 1-2MHz
+    } else if (MICROPY_HW_CLK_VALUE / 1000000 <= 4 * m) {
+        RCC_OscInitStruct.PLL.PLLRGE = RCC_PLL1_VCIRANGE_1; // 2-4MHz
+    } else if (MICROPY_HW_CLK_VALUE / 1000000 <= 8 * m) {
+        RCC_OscInitStruct.PLL.PLLRGE = RCC_PLL1_VCIRANGE_2; // 4-8MHz
+    } else {
+        RCC_OscInitStruct.PLL.PLLRGE = RCC_PLL1_VCIRANGE_3; // 8-16MHz
+    }
+    if (MICROPY_HW_CLK_VALUE / 1000000 * n <= 420 * m) {
+        RCC_OscInitStruct.PLL.PLLVCOSEL = RCC_PLL1_VCORANGE_MEDIUM; // 150-420MHz
+    } else {
+        RCC_OscInitStruct.PLL.PLLVCOSEL = RCC_PLL1_VCORANGE_WIDE; // 192-836MHz
+    }
+    RCC_OscInitStruct.PLL.PLLFRACN = 0;
+    #elif defined(STM32H7)
     RCC_OscInitStruct.PLL.PLLR = 0;
     if (MICROPY_HW_CLK_VALUE / 1000000 <= 2 * m) {
         RCC_OscInitStruct.PLL.PLLRGE = RCC_PLL1VCIRANGE_0; // 1-2MHz
@@ -777,6 +799,14 @@ void powerctrl_enter_stop_mode(void) {
     HAL_PWREx_EnableFlashPowerDown();
     #endif
 
+    #if defined(STM32H5)
+    // Save RCC CR to re-enable OSCs and PLLs after wake up from low power mode.
+    uint32_t rcc_cr = RCC->CR;
+
+    // Save the current voltage scaling level to restore after exiting low power mode.
+    uint32_t vscaling = POWERCTRL_GET_VOLTAGE_SCALING();
+    #endif
+
     #if defined(STM32H7)
     // Save RCC CR to re-enable OSCs and PLLs after wake up from low power mode.
     uint32_t rcc_cr = RCC->CR;
@@ -817,9 +847,9 @@ void powerctrl_enter_stop_mode(void) {
     while (__HAL_RCC_GET_SYSCLK_SOURCE() != RCC_CFGR_SWS_HSI48) {
     }
 
-    #else
+    #else // defined(STM32F0)
 
-    #if defined(STM32H7)
+    #if defined(STM32H5) || defined(STM32H7)
     // When exiting from Stop or Standby modes, the Run mode voltage scaling is reset to
     // the default VOS3 value. Restore the voltage scaling to the previous voltage scale.
     if (vscaling != POWERCTRL_GET_VOLTAGE_SCALING()) {
@@ -859,7 +889,7 @@ void powerctrl_enter_stop_mode(void) {
     while (LL_RCC_GetSysClkSource() != LL_RCC_SYS_CLKSOURCE_STATUS_PLL1) {
     }
 
-    #else
+    #else // defined(STM32H5)
 
     // enable PLL
     __HAL_RCC_PLL_ENABLE();
@@ -879,7 +909,7 @@ void powerctrl_enter_stop_mode(void) {
     }
     #endif
 
-    #endif
+    #endif // defined(STM32H5)
 
     powerctrl_disable_hsi_if_unused();
 
@@ -892,6 +922,15 @@ void powerctrl_enter_stop_mode(void) {
     }
     #endif
 
+    #if defined(STM32H5)
+    if (rcc_cr & RCC_CR_HSI48ON) {
+        // Enable HSI48.
+        LL_RCC_HSI48_Enable();
+        while (!LL_RCC_HSI48_IsReady()) {
+        }
+    }
+    #endif
+
     #if defined(STM32H7)
     // Enable HSI
     if (rcc_cr & RCC_CR_HSION) {
@@ -954,7 +993,7 @@ void powerctrl_enter_stop_mode(void) {
     enable_irq(irq_state);
 }
 
-void powerctrl_enter_standby_mode(void) {
+NORETURN void powerctrl_enter_standby_mode(void) {
     rtc_init_finalise();
 
     #if defined(MICROPY_BOARD_ENTER_STANDBY)
@@ -1075,5 +1114,7 @@ void powerctrl_enter_standby_mode(void) {
 
     // enter standby mode
     HAL_PWR_EnterSTANDBYMode();
-    // we never return; MCU is reset on exit from standby
+
+    // MCU is reset on exit from standby, but just in case it's not, do an explicit reset.
+    powerctrl_mcu_reset();
 }
