@@ -98,7 +98,7 @@ extern PCD_HandleTypeDef pcd_hs_handle;
 // More information about decoding the fault registers can be found here:
 // http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.dui0646a/Cihdjcfc.html
 
-STATIC char *fmt_hex(uint32_t val, char *buf) {
+static char *fmt_hex(uint32_t val, char *buf) {
     const char *hexDig = "0123456789abcdef";
 
     buf[0] = hexDig[(val >> 28) & 0x0f];
@@ -114,7 +114,7 @@ STATIC char *fmt_hex(uint32_t val, char *buf) {
     return buf;
 }
 
-STATIC void print_reg(const char *label, uint32_t val) {
+static void print_reg(const char *label, uint32_t val) {
     char hexStr[9];
 
     mp_hal_stdout_tx_str(label);
@@ -122,7 +122,7 @@ STATIC void print_reg(const char *label, uint32_t val) {
     mp_hal_stdout_tx_str("\r\n");
 }
 
-STATIC void print_hex_hex(const char *label, uint32_t val1, uint32_t val2) {
+static void print_hex_hex(const char *label, uint32_t val1, uint32_t val2) {
     char hex_str[9];
     mp_hal_stdout_tx_str(label);
     mp_hal_stdout_tx_str(fmt_hex(val1, hex_str));
@@ -304,6 +304,14 @@ void USB_UCPD1_2_IRQHandler(void) {
 }
 #endif
 
+#elif defined(STM32H5)
+
+#if MICROPY_HW_USB_FS
+void USB_DRD_FS_IRQHandler(void) {
+    HAL_PCD_IRQHandler(&pcd_fs_handle);
+}
+#endif
+
 #elif defined(STM32L0) || defined(STM32L432xx)
 
 #if MICROPY_HW_USB_FS
@@ -312,7 +320,7 @@ void USB_IRQHandler(void) {
 }
 #endif
 
-#elif defined(STM32WB)
+#elif defined(STM32G4) || defined(STM32L1) || defined(STM32WB)
 
 #if MICROPY_HW_USB_FS
 void USB_LP_IRQHandler(void) {
@@ -348,7 +356,7 @@ void OTG_HS_IRQHandler(void) {
   * @param  *pcd_handle for FS or HS
   * @retval None
   */
-STATIC void OTG_CMD_WKUP_Handler(PCD_HandleTypeDef *pcd_handle) {
+static void OTG_CMD_WKUP_Handler(PCD_HandleTypeDef *pcd_handle) {
 
     if (pcd_handle->Init.low_power_enable) {
         /* Reset SLEEPDEEP bit of Cortex System Control Register */
@@ -404,7 +412,7 @@ void OTG_FS_WKUP_IRQHandler(void) {
 
     #if defined(STM32L4)
     EXTI->PR1 = USB_OTG_FS_WAKEUP_EXTI_LINE;
-    #elif !defined(STM32H7)
+    #elif !defined(STM32H5) && !defined(STM32H7)
     /* Clear EXTI pending Bit*/
     __HAL_USB_FS_EXTI_CLEAR_FLAG();
     #endif
@@ -424,7 +432,7 @@ void OTG_HS_WKUP_IRQHandler(void) {
 
     OTG_CMD_WKUP_Handler(&pcd_hs_handle);
 
-    #if !defined(STM32H7)
+    #if !defined(STM32H5) && !defined(STM32H7)
     /* Clear EXTI pending Bit*/
     __HAL_USB_HS_EXTI_CLEAR_FLAG();
     #endif
@@ -528,7 +536,13 @@ void ETH_WKUP_IRQHandler(void) {
 }
 #endif
 
-#if defined(STM32L1)
+#if defined(STM32H5)
+void TAMP_IRQHandler(void) {
+    IRQ_ENTER(TAMP_IRQn);
+    Handle_EXTI_Irq(EXTI_RTC_TAMP);
+    IRQ_EXIT(TAMP_IRQn);
+}
+#elif defined(STM32L1)
 void TAMPER_STAMP_IRQHandler(void) {
     IRQ_ENTER(TAMPER_STAMP_IRQn);
     Handle_EXTI_Irq(EXTI_RTC_TIMESTAMP);
@@ -542,10 +556,17 @@ void TAMP_STAMP_IRQHandler(void) {
 }
 #endif
 
-void RTC_WKUP_IRQHandler(void) {
+#if defined(STM32H5)
+void RTC_IRQHandler(void)
+#else
+void RTC_WKUP_IRQHandler(void)
+#endif
+{
     IRQ_ENTER(RTC_WKUP_IRQn);
     #if defined(STM32G0) || defined(STM32G4) || defined(STM32WL)
     RTC->MISR &= ~RTC_MISR_WUTMF; // clear wakeup interrupt flag
+    #elif defined(STM32H5)
+    RTC->SCR = RTC_SCR_CWUTF; // clear wakeup interrupt flag
     #elif defined(STM32H7A3xx) || defined(STM32H7A3xxQ) || defined(STM32H7B3xx) || defined(STM32H7B3xxQ)
     RTC->SR &= ~RTC_SR_WUTF; // clear wakeup interrupt flag
     #else
@@ -765,7 +786,7 @@ void TIM8_UP_TIM13_IRQHandler(void) {
     IRQ_EXIT(TIM8_UP_TIM13_IRQn);
 }
 
-#if defined(STM32G4) || defined(STM32L4)
+#if defined(STM32G4) || defined(STM32H5) || defined(STM32L4)
 void TIM8_UP_IRQHandler(void) {
     IRQ_ENTER(TIM8_UP_IRQn);
     timer_irq_handler(8);
@@ -994,6 +1015,14 @@ void UART10_IRQHandler(void) {
 }
 #endif
 
+#if defined(USART10)
+void USART10_IRQHandler(void) {
+    IRQ_ENTER(USART10_IRQn);
+    uart_irq_handler(10);
+    IRQ_EXIT(USART10_IRQn);
+}
+#endif
+
 #endif
 
 #if defined(LPUART1)
