@@ -38,7 +38,7 @@ typedef struct _mp_obj_sensor_t {
     const struct device *dev;
 } mp_obj_sensor_t;
 
-STATIC mp_obj_t sensor_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
+static mp_obj_t sensor_make_new(const mp_obj_type_t *type, size_t n_args, size_t n_kw, const mp_obj_t *args) {
     mp_arg_check_num(n_args, n_kw, 1, 1, false);
     mp_obj_sensor_t *o = mp_obj_malloc(mp_obj_sensor_t, type);
     o->dev = device_get_binding(mp_obj_str_get_str(args[0]));
@@ -48,7 +48,7 @@ STATIC mp_obj_t sensor_make_new(const mp_obj_type_t *type, size_t n_args, size_t
     return MP_OBJ_FROM_PTR(o);
 }
 
-STATIC mp_obj_t sensor_measure(mp_obj_t self_in) {
+static mp_obj_t sensor_measure(mp_obj_t self_in) {
     mp_obj_sensor_t *self = MP_OBJ_TO_PTR(self_in);
     int st = sensor_sample_fetch(self->dev);
     if (st != 0) {
@@ -58,7 +58,7 @@ STATIC mp_obj_t sensor_measure(mp_obj_t self_in) {
 }
 MP_DEFINE_CONST_FUN_OBJ_1(sensor_measure_obj, sensor_measure);
 
-STATIC void sensor_get_internal(mp_obj_t self_in, mp_obj_t channel_in, struct sensor_value *res) {
+static void sensor_get_internal(mp_obj_t self_in, mp_obj_t channel_in, struct sensor_value *res) {
     mp_obj_sensor_t *self = MP_OBJ_TO_PTR(self_in);
 
     int st = sensor_channel_get(self->dev, mp_obj_get_int(channel_in), res);
@@ -67,35 +67,35 @@ STATIC void sensor_get_internal(mp_obj_t self_in, mp_obj_t channel_in, struct se
     }
 }
 
-STATIC mp_obj_t sensor_get_float(mp_obj_t self_in, mp_obj_t channel_in) {
+static mp_obj_t sensor_get_float(mp_obj_t self_in, mp_obj_t channel_in) {
     struct sensor_value val;
     sensor_get_internal(self_in, channel_in, &val);
     return mp_obj_new_float(val.val1 + (mp_float_t)val.val2 / 1000000);
 }
 MP_DEFINE_CONST_FUN_OBJ_2(sensor_get_float_obj, sensor_get_float);
 
-STATIC mp_obj_t sensor_get_micros(mp_obj_t self_in, mp_obj_t channel_in) {
+static mp_obj_t sensor_get_micros(mp_obj_t self_in, mp_obj_t channel_in) {
     struct sensor_value val;
     sensor_get_internal(self_in, channel_in, &val);
     return MP_OBJ_NEW_SMALL_INT(val.val1 * 1000000 + val.val2);
 }
 MP_DEFINE_CONST_FUN_OBJ_2(sensor_get_micros_obj, sensor_get_micros);
 
-STATIC mp_obj_t sensor_get_millis(mp_obj_t self_in, mp_obj_t channel_in) {
+static mp_obj_t sensor_get_millis(mp_obj_t self_in, mp_obj_t channel_in) {
     struct sensor_value val;
     sensor_get_internal(self_in, channel_in, &val);
     return MP_OBJ_NEW_SMALL_INT(val.val1 * 1000 + val.val2 / 1000);
 }
 MP_DEFINE_CONST_FUN_OBJ_2(sensor_get_millis_obj, sensor_get_millis);
 
-STATIC mp_obj_t sensor_get_int(mp_obj_t self_in, mp_obj_t channel_in) {
+static mp_obj_t sensor_get_int(mp_obj_t self_in, mp_obj_t channel_in) {
     struct sensor_value val;
     sensor_get_internal(self_in, channel_in, &val);
     return MP_OBJ_NEW_SMALL_INT(val.val1);
 }
 MP_DEFINE_CONST_FUN_OBJ_2(sensor_get_int_obj, sensor_get_int);
 
-STATIC const mp_rom_map_elem_t sensor_locals_dict_table[] = {
+static const mp_rom_map_elem_t sensor_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_measure), MP_ROM_PTR(&sensor_measure_obj) },
     { MP_ROM_QSTR(MP_QSTR_get_float), MP_ROM_PTR(&sensor_get_float_obj) },
     { MP_ROM_QSTR(MP_QSTR_get_micros), MP_ROM_PTR(&sensor_get_micros_obj) },
@@ -103,9 +103,9 @@ STATIC const mp_rom_map_elem_t sensor_locals_dict_table[] = {
     { MP_ROM_QSTR(MP_QSTR_get_int), MP_ROM_PTR(&sensor_get_int_obj) },
 };
 
-STATIC MP_DEFINE_CONST_DICT(sensor_locals_dict, sensor_locals_dict_table);
+static MP_DEFINE_CONST_DICT(sensor_locals_dict, sensor_locals_dict_table);
 
-STATIC MP_DEFINE_CONST_OBJ_TYPE(
+static MP_DEFINE_CONST_OBJ_TYPE(
     sensor_type,
     MP_QSTR_Sensor,
     MP_TYPE_FLAG_NONE,
@@ -113,7 +113,7 @@ STATIC MP_DEFINE_CONST_OBJ_TYPE(
     locals_dict, &sensor_locals_dict
     );
 
-STATIC const mp_rom_map_elem_t mp_module_zsensor_globals_table[] = {
+static const mp_rom_map_elem_t mp_module_zsensor_globals_table[] = {
     { MP_ROM_QSTR(MP_QSTR___name__), MP_ROM_QSTR(MP_QSTR_zsensor) },
     { MP_ROM_QSTR(MP_QSTR_Sensor), MP_ROM_PTR(&sensor_type) },
 
@@ -136,7 +136,7 @@ STATIC const mp_rom_map_elem_t mp_module_zsensor_globals_table[] = {
 #undef C
 };
 
-STATIC MP_DEFINE_CONST_DICT(mp_module_zsensor_globals, mp_module_zsensor_globals_table);
+static MP_DEFINE_CONST_DICT(mp_module_zsensor_globals, mp_module_zsensor_globals_table);
 
 const mp_obj_module_t mp_module_zsensor = {
     .base = { &mp_type_module },
@@ -145,4 +145,4 @@ const mp_obj_module_t mp_module_zsensor = {
 
 MP_REGISTER_MODULE(MP_QSTR_zsensor, mp_module_zsensor);
 
-#endif // MICROPY_PY_UHASHLIB
+#endif // MICROPY_PY_HASHLIB
