@@ -28,6 +28,7 @@
 
 #include "py/mpstate.h"
 #include "py/pystack.h"
+#include "py/stackctrl.h"
 
 // For use with mp_call_function_1_from_nlr_jump_callback.
 #define MP_DEFINE_NLR_JUMP_CALLBACK_FUNCTION_1(ctx, f, a) \
@@ -109,6 +110,23 @@ bool mp_sched_schedule(mp_obj_t function, mp_obj_t arg);
 bool mp_sched_schedule_node(mp_sched_node_t *node, mp_sched_callback_t callback);
 #endif
 
+// Handles any pending MicroPython events without waiting for an interrupt or event.
+void mp_event_handle_nowait(void);
+
+// Handles any pending MicroPython events and then suspends execution until the
+// next interrupt or event.
+//
+// Note: on "tickless" ports this can suspend execution for a long time,
+// don't call unless you know an interrupt is coming to continue execution.
+// On "ticked" ports it may return early due to the tick interrupt.
+void mp_event_wait_indefinite(void);
+
+// Handle any pending MicroPython events and then suspends execution until the
+// next interrupt or event, or until timeout_ms milliseconds have elapsed.
+//
+// On "ticked" ports it may return early due to the tick interrupt.
+void mp_event_wait_ms(mp_uint_t timeout_ms);
+
 // extra printing method specifically for mp_obj_t's which are integral type
 int mp_print_mp_int(const mp_print_t *print, mp_obj_t x, int base, int base_char, int flags, char fill, int width, int prec);
 
@@ -137,6 +155,32 @@ static inline void mp_globals_set(mp_obj_dict_t *d) {
 void mp_globals_locals_set_from_nlr_jump_callback(void *ctx_in);
 void mp_call_function_1_from_nlr_jump_callback(void *ctx_in);
 
+#if MICROPY_PY_THREAD
+static inline void mp_thread_init_state(mp_state_thread_t *ts, size_t stack_size, mp_obj_dict_t *locals, mp_obj_dict_t *globals) {
+    mp_thread_set_state(ts);
+
+    mp_stack_set_top(ts + 1); // need to include ts in root-pointer scan
+    mp_stack_set_limit(stack_size);
+
+    // GC starts off unlocked
+    ts->gc_lock_depth = 0;
+
+    // There are no pending jump callbacks or exceptions yet
+    ts->nlr_jump_callback_top = NULL;
+    ts->mp_pending_exception = MP_OBJ_NULL;
+
+    // If locals/globals are not given, inherit from main thread
+    if (locals == NULL) {
+        locals = mp_state_ctx.thread.dict_locals;
+    }
+    if (globals == NULL) {
+        globals = mp_state_ctx.thread.dict_globals;
+    }
+    mp_locals_set(locals);
+    mp_globals_set(globals);
+}
+#endif
+
 mp_obj_t mp_load_name(qstr qst);
 mp_obj_t mp_load_global(qstr qst);
 mp_obj_t mp_load_build_class(void);
